/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#ifndef __dctclasses__
#define __dctclasses__

#include <Unidraw/classes.h>

class Editor;
class Field;
class Interactor;
class Graphic;

#define REC_COMP			2020
#define EXTLINK_COMP		2021
#define EXAMINE_TOOL		2024
#define FIELDEDIT_CMD		2025
#define FIELDEXPORT_CMD		2026
#define NODE_VAR			2027
#define NODE_COMP			2028
#define WIRE_COMP			2029
#define PVLINK_VAR			2030
#define RENAMEDB_CMD		2032
#define MAKEDB_CMD			2033
#define RENAMEREC_CMD		2034
#define LINKFIELD_CMD		2035
#define DCTGRAPHIC_COMPS    2036
#define IMPORTREC_CMD	    2037
#define NOWHEREEDIT_CMD	    2038
#define DCTSAVECOMPAS_CMD   2039
#define DCTSAVECOMPASVDCT_CMD   2045
#define DCTSAVECOMP_CMD	    2040
#define DCTNEWCOMP_CMD	    2041
#define DCTVIEWCOMP_CMD	    2042
#define DCTPS_VIEW			2043
#define PREFIX_CMD		    2044

#define REC_VIEW			Combine(REC_COMP,COMPONENT_VIEW)
#define EXTLINK_VIEW		Combine(EXTLINK_COMP,COMPONENT_VIEW)
#define NODE_VIEW           Combine(NODE_COMP, COMPONENT_VIEW)
#define WIRE_VIEW           Combine(WIRE_COMP, COMPONENT_VIEW)
#define DCTGRAPHIC_VIEWS    Combine(DCTGRAPHIC_COMPS, COMPONENT_VIEW)

#define PS_REC             Combine(REC_COMP, POSTSCRIPT_VIEW)
#define PS_EXTLINK         Combine(EXTLINK_COMP, POSTSCRIPT_VIEW)
#define PS_NODE            Combine(NODE_COMP, POSTSCRIPT_VIEW)
#define PS_WIRE            Combine(WIRE_COMP, POSTSCRIPT_VIEW)
#define PS_DCTGRAPHIC      Combine(DCTGRAPHIC_COMPS, POSTSCRIPT_VIEW)

#define VDCT_VIRTUAL_WIDTH  5000
#define VDCT_VIRTUAL_HEIGHT 5000

extern const char* KLBL_REC,            *CODE_REC;
extern const char* KLBL_EXTLINK,        *CODE_EXTLINK;
extern const char* KLBL_EXAMINE,        *CODE_EXAMINE;
extern const char* KLBL_FIELDEDIT,      *CODE_FIELDEDIT;
extern const char* KLBL_FIELDEXPORT,    *CODE_FIELDEXPORT;
extern const char* KLBL_MAKEDB,   		*CODE_MAKEDB;
extern const char* KLBL_RENAMEDB,   	*CODE_RENAMEDB;
extern const char* KLBL_RENAMEREC,   	*CODE_RENAMEREC;
extern const char* KLBL_LINKFIELD,   	*CODE_LINKFIELD;
extern const char* KLBL_WIRE,   		*CODE_WIRE;
extern const char* KLBL_IMPORTREC,   	*CODE_IMPORTREC;
extern const char* KLBL_NOWHEREEDIT,   	*CODE_NOWHEREEDIT;
extern const char* KLBL_DCTVIEWCMD,   	*CODE_DCTVIEWCMD;
extern const char* KLBL_DCTSAVEASCMD,  	*CODE_DCTSAVEASCMD;
extern const char* KLBL_DCTSAVEASVDCTCMD,  	*CODE_DCTSAVEASVDCTCMD;
extern const char* KLBL_DCTSAVECMD,  	*CODE_DCTSAVECMD;
extern const char* KLBL_DCTNEWCMD,  	*CODE_DCTNEWCMD;
extern const char* KLBL_PREFIX, 	 	*CODE_PREFIX;
extern const char* KLBL_EDITTEXT, 	 	*CODE_EDITTEXT;

extern void global_dialog_wn_delete(Interactor*);
extern void global_warning(Editor*, const char*);
extern void global_world_warning(Interactor*, const char*);
extern char global_confirm(Editor*, const char*);
extern char* global_getastring(Editor* ed, const char* t,
		const char* s, const char* n);
extern void PageToVDCT(float xpage, float ypage, int &xvdct, int &yvdct);

typedef enum { PP_ERR, PP, NPP } pass_process;
typedef enum { MS_ERR, MS, NMS } max_severity;

extern Graphic* my_stdgraphic;

extern int use_gdct_style_stuff;

extern float page_width,page_height;

#endif

