/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#include <InterViews/world.h>
#include <InterViews/window.h>
#include <Unidraw/catalog.h>
#include <Unidraw/ulist.h>
#include <Unidraw/editor.h>

#include "dctclasses.h"
#include "dctcmds.h"
#include "dctrec.h"
#include "dcted.h"
#include "dctdial.h"
#include "dctcomps.h"
#include "dctnowhere.h"
#include "dctext.h"

#include <stream.h>
#include <string.h>

extern "C" {
#include "dbStaticLib.h"
};

// -----------------------field edit command------------------------

ClassId FieldEditCmd::GetClassId () { return FIELDEDIT_CMD; }
 
boolean FieldEditCmd::IsA (ClassId id)
 { return FIELDEDIT_CMD == id || Command::IsA(id); }
 
FieldEditCmd::FieldEditCmd (ControlInfo* c, GraphicComp* comp) : Command(c)
{ _comp = comp; }
 
FieldEditCmd::FieldEditCmd (Editor* ed, GraphicComp* comp) : Command(ed)
{ _comp = comp;}
 
Command* FieldEditCmd::Copy ()
{
    Command* newCmd = new FieldEditCmd(CopyControlInfo(), GetGraphicComp());
    InitCopy(newCmd);
    return newCmd;
}
 
void FieldEditCmd::Execute ()
{
    Editor* ed = GetEditor();
    GraphicComp* comp = (_comp == nil) ? Command::GetGraphicComp() : _comp;
 
    if ( comp->IsA(REC_COMP) )
	{
		// need to edit the fields of a record
        Modify(comp);
    }
	else if(comp->IsA(WIRE_COMP))
	{
		// need to modify the pp/npp attributes
		ModifyPPMS(comp);
	}
	// there should be an else condition presenting message
}

void FieldEditCmd::Modify (GraphicComp* comp)
{
    Editor* ed = GetEditor();
	RecComp* rec_comp = (RecComp*)comp;
	World* w = unidraw->GetWorld();
	Coord x,y;
	FieldEditDialog fld_edit(rec_comp->GetRec());

	x=(Coord)(ed->GetWindow()->left());
	y=(Coord)((ed->GetWindow()->bottom())+(ed->GetWindow()->height()));

	w->Insert(&fld_edit,x,y,TopLeft);

    if (fld_edit.Accept())
	{
		w->Remove(&fld_edit);
		ModifStatusVar* mnv=(ModifStatusVar*)ed->GetState("ModifStatusVar");
		mnv->SetModifStatus(true);
        comp->Notify();
        unidraw->Update();
    }
	else
		w->Remove(&fld_edit);
}
 
void FieldEditCmd::ModifyPPMS (GraphicComp* comp)
{
    Editor* ed = GetEditor();
	WireComp* wire_comp = (WireComp*)comp;
	PVLinkVar* pv = wire_comp->GetPVLinkVar();

	ppms_edit ppms_ed("PP/MS Editor",pv->GetLinkPP(),pv->GetLinkMS());
    ed->InsertDialog(&ppms_ed);
	global_dialog_wn_delete(&ppms_ed);
 
    if (ppms_ed.Accept())
	{
		pv->SetLink(ppms_ed.GetPP(),ppms_ed.GetMS());
        ed->RemoveDialog(&ppms_ed);
        unidraw->Update();
    }
	else
	{
        ed->RemoveDialog(&ppms_ed);
    }
}
 
boolean FieldEditCmd::Reversible () { return false; }
 
void FieldEditCmd::Read (istream& in)
{
    Command::Read(in);
    _comp = (GraphicComp*) unidraw->GetCatalog()->ReadComponent(in);
}
 
void FieldEditCmd::Write (ostream& out)
{
    Command::Write(out);
    unidraw->GetCatalog()->WriteComponent(GetGraphicComp(), out);
}

// -----------------------field export command------------------------

ClassId FieldExportCmd::GetClassId () { return FIELDEXPORT_CMD; }
 
boolean FieldExportCmd::IsA (ClassId id)
 { return FIELDEXPORT_CMD == id || Command::IsA(id); }
 
FieldExportCmd::FieldExportCmd (ControlInfo* c, GraphicComp* comp) : Command(c)
{ _comp = comp; }
 
FieldExportCmd::FieldExportCmd (Editor* ed, GraphicComp* comp) : Command(ed)
{ _comp = comp;}
 
Command* FieldExportCmd::Copy ()
{
    Command* newCmd = new FieldExportCmd(CopyControlInfo(), GetGraphicComp());
    InitCopy(newCmd);
    return newCmd;
}
 
void FieldExportCmd::Execute ()
{
    Editor* ed = GetEditor();
    GraphicComp* comp = (_comp == nil) ? Command::GetGraphicComp() : _comp;
 
    if ( comp->IsA(REC_COMP) )
	{
        Modify(comp);
    }
	// there should be an else condition presenting message
}

void FieldExportCmd::Modify (GraphicComp* comp)
{
	Editor* ed = GetEditor();
	RecComp* rec_comp = (RecComp*)comp;
	UList new_add_list, new_remove_list;
	int i;
 
    ExportList exp_list;
	exp_list.my_go(rec_comp->GetRec(), rec_comp);

    ed->InsertDialog(&exp_list);
	global_dialog_wn_delete(&exp_list);
 
    if (exp_list.Accept())
	{
        ed->RemoveDialog(&exp_list);

		// put the selected add fields into a list
		for(i=0;i<exp_list.CountAdd();i++)
		{
			new_add_list.Append(new UList(exp_list.GetSelectAdd(i)));
		}

		// put the selected remove fields into a list
		for(i=0;i<exp_list.CountRemove();i++)
		{
			new_remove_list.Append(new UList(exp_list.GetSelectRemove(i)));
		}

	rec_comp->FixUpFields(&new_add_list, &new_remove_list);
        rec_comp->Notify();
        unidraw->Update();
    }
	else
	{
        ed->RemoveDialog(&exp_list);
    }
}

boolean FieldExportCmd::Reversible () { return false; }
 
void FieldExportCmd::Read (istream& in)
{
    Command::Read(in);
    _comp = (GraphicComp*) unidraw->GetCatalog()->ReadComponent(in);
}
 
void FieldExportCmd::Write (ostream& out)
{
    Command::Write(out);
    unidraw->GetCatalog()->WriteComponent(GetGraphicComp(), out);
}

// -----------------------prefix command------------------------

ClassId PrefixCmd::GetClassId () { return PREFIX_CMD; }
 
boolean PrefixCmd::IsA (ClassId id)
 { return PREFIX_CMD == id || Command::IsA(id); }
 
PrefixCmd::PrefixCmd (ControlInfo* c) : Command(c) {}
PrefixCmd::PrefixCmd (Editor* ed) : Command(ed) {}
boolean PrefixCmd::Reversible () { return false; }
 
Command* PrefixCmd::Copy ()
{
    Command* newCmd = new PrefixCmd(CopyControlInfo());
    InitCopy(newCmd);
    return newCmd;
}
 
void PrefixCmd::Execute ()
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
    Dct* ed = (Dct*)GetEditor();
	char* cc;

	cc = global_getastring(ed,"Enter Record Prefix","",du->GetPrefix());

	if(cc && cc[0]!='\0')
	{
		du->SetPrefix(cc);

		ModifStatusVar* mv = (ModifStatusVar*)ed->GetState("ModifStatusVar");
		mv->SetModifStatus(true);
	}
	delete cc;
}

// -----------------------rename record command------------------------

ClassId RenameRecCmd::GetClassId () { return RENAMEREC_CMD; }
 
boolean RenameRecCmd::IsA (ClassId id)
 { return RENAMEREC_CMD == id || Command::IsA(id); }
 
RenameRecCmd::RenameRecCmd (ControlInfo* c, GraphicComp* comp) : Command(c)
{ _comp = comp; }
 
RenameRecCmd::RenameRecCmd (Editor* ed, GraphicComp* comp) : Command(ed)
{ _comp = comp;}
 
Command* RenameRecCmd::Copy ()
{
    Command* newCmd = new RenameRecCmd(CopyControlInfo(), GetGraphicComp());
    InitCopy(newCmd);
    return newCmd;
}
 
void RenameRecCmd::Execute ()
{
    Editor* ed = GetEditor();
    GraphicComp* comp = (_comp == nil) ? Command::GetGraphicComp() : _comp;
 
    if ( comp->IsA(REC_COMP) )
	{
		RecComp* rc = (RecComp*)comp;
		DBENTRY* rp = rc->GetRec();
		char* cc;

		cc=global_getastring(ed,"Enter New Record Name:","",
			dbGetRecordName(rp));

		if(cc && cc[0]!='\0')
		{
			if(rc->RenameRec(cc))
			{
				global_warning(ed,"Could not rename the record");
			}
		}
		delete cc;
    }
    else if( comp->IsA(EXTLINK_COMP) )
	{
		ExtLinkComp* elc = (ExtLinkComp*)comp;
		char* cc;

		cc=global_getastring(ed,"Enter New External Link Name:","",
			elc->GetLinkName());

		if(cc && cc[0]!='\0')
		{
			if(elc->RenameLink(cc))
			{
				global_warning(ed,"Could not rename the link");
			}
		}
		delete cc;
    }
}

boolean RenameRecCmd::Reversible () { return false; }
 
void RenameRecCmd::Read (istream& in)
{
    Command::Read(in);
    _comp = (GraphicComp*) unidraw->GetCatalog()->ReadComponent(in);
}
 
void RenameRecCmd::Write (ostream& out)
{
    Command::Write(out);
    unidraw->GetCatalog()->WriteComponent(GetGraphicComp(), out);
}

// -----------------------link a field command------------------------

ClassId LinkFieldCmd::GetClassId () { return LINKFIELD_CMD; }
 
boolean LinkFieldCmd::IsA (ClassId id)
 { return LINKFIELD_CMD == id || Command::IsA(id); }
 
LinkFieldCmd::LinkFieldCmd (ControlInfo* c, GraphicComp* comp) : Command(c)
{ _comp = comp; }
 
LinkFieldCmd::LinkFieldCmd (Editor* ed, GraphicComp* comp) : Command(ed)
{ _comp = comp;}
 
Command* LinkFieldCmd::Copy ()
{
    Command* newCmd = new LinkFieldCmd(CopyControlInfo(), GetGraphicComp());
    InitCopy(newCmd);
    return newCmd;
}
 
void LinkFieldCmd::Execute ()
{
    GraphicComp* comp = (_comp == nil) ? Command::GetGraphicComp() : _comp;
 
    if ( comp->IsA(NODE_COMP) )
	{
		NodeComp* nc = (NodeComp*)comp;
		// DBENTRY* rp = nc->GetRecord();
		// if(rp) cerr << "In Linkfieldcmd::execute " << rp->name() << "\n";
		// else cerr << "In linkfieldcmd::execute no record\n";
		// if(nc->GetTransMethod()==Out) cerr << "nodecomp is type out\n";
		// else cerr << "nodecomp is type In\n";
		// if(nc->GetParent()==nil) cerr << "nodecomp parent is nil\n";
		nc->Transmit();
    }
}

boolean LinkFieldCmd::Reversible () { return false; }
 
void LinkFieldCmd::Read (istream& in)
{
    Command::Read(in);
    _comp = (GraphicComp*) unidraw->GetCatalog()->ReadComponent(in);
}
 
void LinkFieldCmd::Write (ostream& out)
{
    Command::Write(out);
    unidraw->GetCatalog()->WriteComponent(GetGraphicComp(), out);
}

// -----------------------import records to nowhere command------------------

ClassId ImportRecCmd::GetClassId () { return IMPORTREC_CMD; }
boolean ImportRecCmd::IsA (ClassId id)
 { return IMPORTREC_CMD == id || Command::IsA(id); }
 
ImportRecCmd::ImportRecCmd (ControlInfo* c) : Command(c) {}
ImportRecCmd::ImportRecCmd (Editor* ed) : Command(ed) {}
 
Command* ImportRecCmd::Copy ()
{
    Command* newCmd = new ImportRecCmd(CopyControlInfo());
    InitCopy(newCmd);
    return newCmd;
}

boolean ImportRecCmd::Reversible () { return false; }

void ImportRecCmd::Execute ()
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();
    Dct* ed = (Dct*)GetEditor();
	char* cc;

	cc = global_getastring(ed,"Enter File To Import:","","");

	if(cc && cc[0]!='\0')
	{
		if(dbReadDatabase(&db,(char*)cc,NULL,NULL))
			global_warning(ed,"Cannot import records");
		else
		{
			ModifStatusVar* mnv=(ModifStatusVar*)ed->GetState("ModifStatusVar");
			mnv->SetModifStatus(true);
		}
	}
	delete [] cc;
}

// -----------------------edit the nowhere records------------------

ClassId NowhereEditCmd::GetClassId () { return NOWHEREEDIT_CMD; }
boolean NowhereEditCmd::IsA (ClassId id)
 { return NOWHEREEDIT_CMD == id || Command::IsA(id); }
 
NowhereEditCmd::NowhereEditCmd (ControlInfo* c) : Command(c) {}
NowhereEditCmd::NowhereEditCmd (Editor* ed) : Command(ed) {}
 
Command* NowhereEditCmd::Copy ()
{
    Command* newCmd = new NowhereEditCmd(CopyControlInfo());
    InitCopy(newCmd);
    return newCmd;
}

boolean NowhereEditCmd::Reversible () { return false; }

void NowhereEditCmd::Execute ()
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
    Dct* ed = (Dct*)GetEditor();
	World* w = unidraw->GetWorld();
	Coord x,y;

	NowhereEdit ne("Edit Hidden Records");

	x=(Coord)(ed->GetWindow()->left());
	y=(Coord)((ed->GetWindow()->bottom())+(ed->GetWindow()->height()));

	w->Insert(&ne,x,y,TopLeft);
    ne.Accept();
	// ne.GetData();
	w->Remove(&ne);

	ModifStatusVar* nv=(ModifStatusVar*)ed->GetState("ModifStatusVar");
	nv->SetModifStatus(true);

	unidraw->Update();
}

