/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#ifndef __dctcmds_h__
#define __dctcmds_h__

#include <stream.h>
#include <Unidraw/Commands/command.h>

class PopupMenu;
class Command;
class TextGraphic;
class PinComp;
class PinView;
class UList;
class Line;

//-------------------- field edit command--------------------------
class FieldEditCmd : public Command
{
public:
    FieldEditCmd(ControlInfo*, GraphicComp* comp = nil);
    FieldEditCmd(Editor* = nil, GraphicComp* comp = nil);
 
    virtual void Execute();
    virtual boolean Reversible();
 
    GraphicComp* GetGraphicComp();
    void SetGraphicComp(GraphicComp*);
 
    virtual Command* Copy();
    virtual void Read(istream&);
    virtual void Write(ostream&);
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    void Modify(GraphicComp*);
    void ModifyPPMS(GraphicComp*);

    GraphicComp* _comp;
};

inline GraphicComp* FieldEditCmd::GetGraphicComp () { return _comp; }
inline void FieldEditCmd::SetGraphicComp (GraphicComp* comp) { _comp = comp; }

//-------------------- export list command--------------------------
class FieldExportCmd : public Command
{
public:
    FieldExportCmd(ControlInfo*, GraphicComp* comp = nil);
    FieldExportCmd(Editor* = nil, GraphicComp* comp = nil);
 
    virtual void Execute();
    virtual boolean Reversible();
 
    GraphicComp* GetGraphicComp();
    void SetGraphicComp(GraphicComp*);
 
    virtual Command* Copy();
    virtual void Read(istream&);
    virtual void Write(ostream&);
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    void Modify(GraphicComp*);
private:
    GraphicComp* _comp;
};

inline GraphicComp* FieldExportCmd::GetGraphicComp () { return _comp; }
inline void FieldExportCmd::SetGraphicComp (GraphicComp* comp) { _comp = comp; }

//-------------------- rename record command--------------------------
class RenameRecCmd : public Command
{
public:
    RenameRecCmd(ControlInfo*, GraphicComp* comp = nil);
    RenameRecCmd(Editor* = nil, GraphicComp* comp = nil);
 
    virtual void Execute();
    virtual boolean Reversible();
 
    GraphicComp* GetGraphicComp();
    void SetGraphicComp(GraphicComp*);
 
    virtual Command* Copy();
    virtual void Read(istream&);
    virtual void Write(ostream&);
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    GraphicComp* _comp;
};

inline GraphicComp* RenameRecCmd::GetGraphicComp () { return _comp; }
inline void RenameRecCmd::SetGraphicComp (GraphicComp* comp) { _comp = comp; }

//-------------------- Set global prefix command--------------------------
class PrefixCmd : public Command
{
public:
    PrefixCmd(ControlInfo*);
    PrefixCmd(Editor* = nil);
 
    virtual void Execute();
    virtual boolean Reversible();
 
    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
};

//-------------------- import record to nowhere command--------------------
class ImportRecCmd : public Command
{
public:
    ImportRecCmd(ControlInfo*);
    ImportRecCmd(Editor* = nil);
 
    virtual void Execute();
    virtual boolean Reversible();
 
    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
};

//-------------------- command to edit the nowhere records-----------------
class NowhereEditCmd : public Command
{
public:
    NowhereEditCmd(ControlInfo*);
    NowhereEditCmd(Editor* = nil);
 
    virtual void Execute();
    virtual boolean Reversible();
 
    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
};

//-------------------- link a field command--------------------------
class LinkFieldCmd : public Command
{
public:
    LinkFieldCmd(ControlInfo*, GraphicComp* comp = nil);
    LinkFieldCmd(Editor* = nil, GraphicComp* comp = nil);
 
    virtual void Execute();
    virtual boolean Reversible();
 
    GraphicComp* GetGraphicComp();
    void SetGraphicComp(GraphicComp*);
 
    virtual Command* Copy();
    virtual void Read(istream&);
    virtual void Write(ostream&);
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    GraphicComp* _comp;
};

inline GraphicComp* LinkFieldCmd::GetGraphicComp () { return _comp; }
inline void LinkFieldCmd::SetGraphicComp (GraphicComp* comp) { _comp = comp; }


#endif
