/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#include "dctclasses.h"
#include "dctcomps.h"
#include "dctmanip.h"
#include "dcted.h"
#include "dctcmds.h"

#include <Unidraw/catalog.h>
#include <Unidraw/clipboard.h>
#include <Unidraw/ulist.h>
#include <Unidraw/editor.h>
#include <Unidraw/manips.h>
#include <Unidraw/selection.h>
#include <Unidraw/transfns.h>
#include <Unidraw/unidraw.h>
#include <Unidraw/viewer.h>
#include <Unidraw/Commands/edit.h>
#include <Unidraw/Commands/command.h>
#include <Unidraw/Commands/macro.h>
#include <Unidraw/Commands/transforms.h>
#include <Unidraw/Graphic/util.h>
#include <Unidraw/Graphic/pspaint.h>
#include <Unidraw/Graphic/polygons.h>
#include <Unidraw/Graphic/lines.h>
#include <Unidraw/Tools/tool.h>
#include <InterViews/transformer.h>
#include <InterViews/painter.h>
#include <InterViews/rubline.h>

#include <math.h>
#include <stdlib.h>
#include <stream.h>
#include <string.h>

extern "C" {
#include "dbStaticLib.h"
};

// Node, Wire, and RubberWire are defined in here

// ----------------------------pp/ms graphic stuff----------------------
PropGraphic::PropGraphic(const char* c)
{
	Coord left, bottom, right, top;
	Catalog* cat = unidraw->GetCatalog();
	PSColor* fgblue = cat->ReadColor("fgcolor",7);
	PSColor* fgblack = cat->ReadColor("fgcolor",1);
	PSColor* bgwhite = cat->ReadColor("bgcolor",10);
	PSFont* helv10;
	if(use_gdct_style_stuff)
		helv10 = cat->ReadFont("font",13);
	else
		helv10 = cat->ReadFont("font",12);
    proc_name = new TextGraphic(c,12);
    proc_name->FillBg(0);
    proc_name->SetColors(fgblue,nil);
    proc_name->SetFont(helv10);
    proc_name->SetPattern(new PSPattern(0, (float)1));
    proc_name->SetTransformer(new Transformer(1, 0, 0, 1, 588.672, 482.704));
	proc_name->GetBox(left,bottom,right,top);

    // rectangle around the record name
    SF_Rect* rect = new SF_Rect(left,bottom,right,top,stdgraphic);
    rect->SetColors(fgblack,bgwhite);
    rect->FillBg(1);
    rect->SetPattern(new PSPattern(0, (float)1));
    rect->SetTransformer(new Transformer(1, 0, 0, 1, 588.672, 482.704));
    rect->Align(TopCenter,proc_name,TopCenter);
    this->Append(rect,proc_name);
}

PropGraphic::~PropGraphic() { }

/************************** Node Component ***********************************/

NodeComp::NodeComp (PinGraphic* pingr) : PinComp(pingr)
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetCatDatabase();

	NodeComp::SetBinding(new NodeVar("node"));
	dbInitEntry(db,&_record);
	_side='l';
	_node_type=ConNode;
	_record_valid=0;
	_field_valid=0;
}

char* NodeComp::GetFieldName()
	{ return _field_valid?dbGetFieldName(&_record):NULL; }

char* NodeComp::GetRecordName()
	{ return _record_valid?dbGetRecordName(&_record):NULL; }

Component* NodeComp::Copy()
{ 
    NodeComp* copy = new NodeComp((PinGraphic*)GetGraphic()->Copy());
    copy->_mobility = _mobility;
    copy->SetTransMethod(GetTransMethod());
	memcpy(copy->GetNodeVar(),GetNodeVar(),sizeof(NodeVar));
    // *(copy->GetNodeVar()) = *GetNodeVar();
    return copy;
}

NodeComp* NodeComp::FindNodeToConnect()
{
	// for each node in the connected list, run this thing
	UList* ul = GetConnections();
	UList* u;
	NodeComp* nc;

	// is this the one?
	if(GetNodeType()!=ConNode) return this;

	// try to find the one that is
	for (u = ul->First(); u != ul->End(); u = u->Next())
	{
		nc = (NodeComp*)(*u)();
		nc = nc->FindNodeToConnect();
		if(nc) return nc;
	}
	return nil;
}

void NodeComp::Connect(Connector* c, CGlue* cg)
{
	NodeComp* nc;

	if(c->IsA(NODE_COMP))
	{
		nc = (NodeComp*)c;
		NodeComp* cn = FindNodeToConnect();
		if(cn) PinComp::Connect(cn,cg);
		else PinComp::Connect(c,cg);
	}
	else
	{
		// getting here means something is probably screwed up
		PinComp::Connect(c,cg);
	}
}

void NodeComp::Disconnect(Connector* c) { PinComp::Disconnect(c); }
void NodeComp::Interpret(Command* cmd) { PinComp::Interpret(cmd); }

boolean NodeComp::HasConnections()
	{ if(GetConnections()->IsEmpty()) return false; else return true; }

void NodeComp::Update()
{
	char *fname,*cp;
	char link_name[50],old_name[50];

	if(GetRecord())
	{
		NodeVar* nv = GetNodeVar();
		strcpy(link_name,nv->GetRecord());

		if( nv->GetRecord()==NULL || strcmp(link_name,"0")==0 )
			dbPutString(GetRecord(),"0"); // clear the link here
		else
		{
			strcat(link_name,".");
			fname=nv->GetField();

			if(fname==NULL || fname[0]=='\0' || strcmp(fname,"0")==0 )
				strcat(link_name,"VAL");
			else
				strcat(link_name,fname);

			strcpy(old_name,dbGetString(GetRecord()));

			if(strcmp(old_name,"0") && dbGetLinkType(GetRecord())==DCT_LINK_PV)
			{
				// this attached the old pp/ms value to the string
				// cerr << "if succeeded - copying old pp/ms\n";
				cp=strchr(old_name,' ');
				if(cp!=NULL) strcat(link_name,cp);
			}
			else
			{
				// cerr << "if failed, appending pp/ms\n";
				strcat(link_name," PP MS");
			}
	
			// cerr << "link name = <" << link_name << ">\n";
			if(dbPutString(GetRecord(),link_name))
				cerr << "ERROR: Cannot update a field in the database\n";
		}
	}

	PinComp::Update();
}

void NodeComp::Read(istream& in)
{
	char rec_name[50], field_name[50];
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetCatDatabase();
	int a;

	PinComp::Read(in);
	in >> rec_name >> field_name >> _side >> a;
	// cerr << "nodecomp::read field_name=(" << field_name << ")\n";
	_node_type = (NodeType)a;

	_record_valid=0;
	_field_valid=0;
	dbInitEntry(db,&_record);

	if( strcmp(rec_name,"0") )
	{
		dbFindRecord(&_record,rec_name);
		_record_valid=1;

		if( strcmp(field_name,"0")==0 )
			strcpy(field_name,"VAL");

		if(dbFindField(&_record,field_name))
		{
			cerr << "Error: field " << field_name;
			cerr << " not found for record " << rec_name << "\n";
		}
		else
			_field_valid=1;
	}
	// else
	// cerr << "Error: record name missing from graphic file - Corruption!\n";

	char junko[50];
	in >> junko;
}

void NodeComp::Write(ostream& out)
{
	char* fname;
	PinComp::Write(out);

	if(_record_valid)
	{
		out << dbGetRecordName(GetRecord()) << " ";

		if(_field_valid)
		{
			fname=dbGetFieldName(GetRecord());
			if(fname && fname[0])
				out << fname << " ";
			else
				out << "VAL ";
		}
		else
			out << "0 ";
	}
	else out << "0 0 ";

	int a = (int)_node_type;
	out << _side << " " << a << " ";
	out << "nodecomp ";
}

void NodeComp::WriteVDCT(FILE *fp)
{
	fprintf(fp,"#! SKIP(1)\n#! [NODE_COMP]\n");
}

ClassId NodeComp::GetClassId () { return NODE_COMP; }
boolean NodeComp::IsA (ClassId id) { return NODE_COMP==id || PinComp::IsA(id);}
void NodeComp::SetSide(char c) { _side=c; }
NodeVar* NodeComp::GetNodeVar () { return (NodeVar*) GetBinding(); }

/*****************************************************************************/

NodeView::NodeView(NodeComp* subj):PinView(subj) { }
ClassId NodeView::GetClassId () { return NODE_VIEW; }
boolean NodeView::IsA(ClassId id) { return NODE_VIEW==id || PinView::IsA(id);}

void NodeView::Update()
{
	NodeComp* nc = GetNodeComp();
#if 0
	if(nc->GetRecord())
		cerr << "In nodeview::update " << nc->GetRecord()->name() << "\n";
	else
		cerr << "In nodeview::update no record\n";
#endif
	PinView::Update();
}

NodeComp* NodeView::GetNodeComp() { return (NodeComp*)GetPinComp(); }
PinComp* NodeView::NewSubject (PinGraphic* pingr) {return new NodeComp(pingr);}

/*****************************************************************************/

static MultiLine* CreateMultiLine (Coord* x, Coord* y, int n)
{
    int last = n-1;
    Transformer* t = new Transformer;

    if (x[0] == x[last]) {
        ++x[last];
        t->Translate(-x[0], 0);
        t->Scale(0, 1);
        t->Translate(x[0], 0);
    }
    if (y[0] == y[last]) {
        ++y[last];
        t->Translate(0, -y[0]);
        t->Scale(1, 0);
        t->Translate(0, y[0]);
    }
    MultiLine* ml = new S_MultiLine(x, y, n, stdgraphic);
    ml->SetTransformer(t);
    Unref(t);
    return ml;
}

WireComp::WireComp()
{
	SetPVLinkVar(new PVLinkVar(this));
}

WireComp::WireComp (Coord* x, Coord* y, int n, Graphic*)
{
    Graphic* pict = new Picture();
    MultiLine* ml = CreateMultiLine(x, y, n);
    PinGraphic* pin1 = new PinGraphic(x[0], y[0]);
    PinGraphic* pin2 = new PinGraphic(x[n-1], y[n-1]);
	_prop_graphic = new PropGraphic("PP/MS");
	SetPVLinkVar(new PVLinkVar(this));
    pin1->SetBrush(psnonebr);
    pin2->SetBrush(psnonebr);
    _node1 = new NodeComp(pin1);
    _node2 = new NodeComp(pin2);
	pin1->Align(CenterLeft,_prop_graphic,CenterRight);
    pict->Append(ml, pin1, pin2,_prop_graphic);
    SetGraphic(pict);
	_transfn = new TF_Direct;
}

WireComp::~WireComp ()
{
	Graphic* parent = GetGraphic();
	parent->Remove(_node1->GetGraphic());
	parent->Remove(_node2->GetGraphic());
    delete _node1;
    delete _node2;    
    delete _transfn;
	delete _pvlink;
}

void WireComp::AlignNode()
{
	NodeComp* to_node;
	Graphic* to_pin;
	DBENTRY* fp;
	char old_link[50];
	char* cp;

	if(_node1->GetTransMethod()==Out) to_node=_node1;
	else to_node=_node2;

	to_pin = to_node->GetGraphic();

	if( strcmp(_prop_graphic->GetString(),_pvlink->GetLinkString()) )
	{
		Graphic* gr = GetGraphic();
		gr->Remove(_prop_graphic);
		delete _prop_graphic;
		_prop_graphic = new PropGraphic(_pvlink->GetLinkString());
		gr->Append(_prop_graphic);
	}

	if(to_node->GetSide()=='r')
		to_pin->Align(CenterRight,_prop_graphic,CenterLeft);
	else
		to_pin->Align(CenterLeft,_prop_graphic,CenterRight);

	// update the database, this is tricky.
	fp=_pvlink->GetRecord();
	strcpy(old_link,dbGetString(fp));
	cp = strchr(old_link,' ');
	if(cp) *(cp+1) = '\0';
	strcat(old_link," ");
	strcat(old_link,_pvlink->GetLinkPP());
	strcat(old_link," ");
	strcat(old_link,_pvlink->GetLinkMS());
	// cerr << "old link name = <" << old_link << ">\n";
	dbPutString(fp,old_link);
}

void WireComp::Interpret (Command* cmd)
{
    if (cmd->IsA(DELETE_CMD) || cmd->IsA(CUT_CMD))
	{
		NodeComp* to_node;

		if(_node1->GetTransMethod()==Out) to_node=_node1;
		else to_node=_node2;

		NodeVar* nv = to_node->GetNodeVar();
		nv->SetRecField("0","0");
		to_node->Transmit();

        _node1->Interpret(cmd);
        _node2->Interpret(cmd);
    }
	else if(cmd->IsA(DUP_CMD))
	{
		// do nothing
	}
	else GraphicComp::Interpret(cmd);
}

void WireComp::Uninterpret (Command* cmd)
{
    if (cmd->IsA(DELETE_CMD) || cmd->IsA(CUT_CMD))
	{
        _node1->Uninterpret(cmd);
        _node2->Uninterpret(cmd);
    }
	else GraphicComp::Uninterpret(cmd);
}

void WireComp::Update()
{
    MultiLine* ml = GetMultiLine();
	PVLinkVar* pv = WireComp::GetPVLinkVar();
    const Coord* x, *y;
    int n = ml->GetOriginal(x, y);
    int last = n-1;

    Transformer context, total(ml->GetTransformer());
    ml->Parent()->TotalTransformation(context);
    total.Postmultiply(&context);
    
    float cx1, cx2, mlx1, mlx2, cy1, cy2, mly1, mly2;
    _node1->GetCenter(cx1, cy1);
    _node2->GetCenter(cx2, cy2);
    total.Transform(float(x[0]), float(y[0]), mlx1, mly1);
    total.Transform(float(x[last]), float(y[last]), mlx2, mly2);

    if (round(cx1) != round(mlx1) || round(cx2) != round(mlx2) ||
        round(cy1) != round(mly1) || round(cy2) != round(mly2))
	{
        context.Invert();
        Transformer* t = new Transformer;
        ml->SetTransformer(t);
        Unref(t);

        float sx = (cx2 - cx1) / float(x[last] - x[0]);
        float sy = (cy2 - cy1) / float(y[last] - y[0]);

        t->Translate(-x[0], -y[0]);
        t->Scale(sx, sy);
        t->Translate(cx1, cy1);
        t->Postmultiply(&context);
    }

	AlignNode();
   	Notify();
}

void WireComp::First (Iterator& i) { i.SetValue(_node1); }
void WireComp::Last (Iterator& i) { i.SetValue(_node2); }
boolean WireComp::Done (Iterator i) { return i.GetValue() == nil; }

void WireComp::Next (Iterator& i)
{ 
    void* v = i.GetValue();

    if(v == nil) i.SetValue(_node1);
    else if(v == _node1) i.SetValue(_node2);
    else i.SetValue(nil);
}

void WireComp::Prev (Iterator& i)
{ 
    void* v = i.GetValue();

    if (v == nil) i.SetValue(_node2);
    else if(v == _node1) i.SetValue(nil);
    else i.SetValue(_node1);
}

TransferFunct* WireComp::GetTransferFunct () { return _transfn; }
GraphicComp* WireComp::GetComp (Iterator i){return (GraphicComp*)i.GetValue();}
void WireComp::SetComp (GraphicComp* gc, Iterator& i) { i.SetValue(gc); }

void WireComp::SetMobility (Mobility m) {
    _node1->SetMobility(m);
    _node2->SetMobility(m);
}

void WireComp::GetNodes (NodeComp*& node1, NodeComp*& node2) {
    node1 = _node1;
    node2 = _node2;
}

MultiLine* WireComp::GetMultiLine () {
    Iterator i;
    Graphic* gr = GetGraphic();
    gr->First(i);
    return (MultiLine*) gr->GetGraphic(i);
}

Component* WireComp::Copy(){ return unidraw->GetCatalog()->Copy(this); }

void WireComp::Read (istream& in) {
    GraphicComp::Read(in);
    Catalog* catalog = unidraw->GetCatalog();
    Coord* x, *y;
    int count;

    ReadVertices(in, x, y, count);
    S_MultiLine* ml = new S_MultiLine(x, y, count);
    delete x;
    delete y;
    PSColor* fg = ReadColor(in);
    PSColor* bg = ReadColor(in);
    ml->SetColors(fg, bg);
    ml->SetBrush(ReadBrush(in));
    ml->SetTransformer(ReadTransformer(in));

    _node1 = (NodeComp*) catalog->ReadComponent(in);
    _node2 = (NodeComp*) catalog->ReadComponent(in);
    _transfn = (TF_Direct*) catalog->ReadTransferFunct(in);

    Graphic* pict = new Picture;
    pict->FillBg(ReadBgFilled(in));
    fg = ReadColor(in);
    bg = ReadColor(in);
    // pict->SetColors(fg, bg);
    pict->SetBrush(ReadBrush(in));
    pict->SetTransformer(ReadTransformer(in));
    pict->Append(ml, _node1->GetGraphic(), _node2->GetGraphic());
    SetGraphic(pict);

	_pvlink = (PVLinkVar*)catalog->ReadStateVar(in);
	_pvlink->SetComponent(this);
    _prop_graphic = new PropGraphic("xxxx");
	AlignNode();
}

void WireComp::Write (ostream& out)
{
    GraphicComp::Write(out);
    Catalog* catalog = unidraw->GetCatalog();
    MultiLine* ml = GetMultiLine();
    const Coord* x, *y;
    int count = ml->GetOriginal(x, y);

    WriteVertices(x, y, count, out);

    WriteColor(ml->GetFgColor(), out);
    WriteColor(ml->GetBgColor(), out);
    WriteBrush(ml->GetBrush(), out);
    WriteTransformer(ml->GetTransformer(), out);

    catalog->WriteComponent(_node1, out);
    catalog->WriteComponent(_node2, out);
    catalog->WriteTransferFunct(_transfn, out);

    Graphic* pict = GetGraphic();
    WriteBgFilled(pict->BgFilled(), out);
    WriteColor(pict->GetFgColor(), out);
    WriteColor(pict->GetBgColor(), out);
    WriteBrush(pict->GetBrush(), out);
    WriteTransformer(pict->GetTransformer(), out);

	catalog->WriteStateVar((StateVar*)_pvlink,out); // p variable for pp/ms
}

void WireComp::WriteVDCT(FILE *fp)
{
	fprintf(fp,"#! SKIP(1)\n#! [WIRE_COMP]\n");
}

ClassId WireComp::GetClassId () { return WIRE_COMP; }
boolean WireComp::IsA(ClassId id) {return WIRE_COMP==id||GraphicComp::IsA(id);}

/*****************************************************************************/

WireView::WireView (WireComp* subj) : GraphicView(subj)
{
	_nodeView1 = _nodeView2 = nil;
	_prop_graphic = nil;
}

WireView::~WireView ()
{
    if (_nodeView1 != nil)
	{
        Graphic* parent = GetGraphic();
        parent->Remove(_nodeView1->GetGraphic());
        parent->Remove(_nodeView2->GetGraphic());

        delete _nodeView1;
        delete _nodeView2;
    }
}

void WireView::Update ()
{
    WireComp* wireComp = GetWireComp();
	PVLinkVar* pv = wireComp->GetPVLinkVar();
	Graphic* wire = GetGraphic();

    IncurDamage(wire);

	AlignNode();

    *wire = *wireComp->GetGraphic();
    UpdateMultiLine();
    IncurDamage(wire);
    EraseHandles();
}

void WireView::AlignNode()
{
	NodeComp* node1 = _nodeView1->GetNodeComp();
	NodeComp* node2 = _nodeView2->GetNodeComp();
	NodeView* to_node_view;
	NodeComp* to_node;
	Graphic* to_ng;

	if(node1->GetTransMethod()==Out)
	{
		to_node_view = _nodeView1;
		to_node = node1;
	}
	else
	{
		to_node_view = _nodeView2;
		to_node = node2;
	}

	to_ng = to_node_view->GetGraphic();
	PVLinkVar* pv = GetWireComp()->GetPVLinkVar();

    if( strcmp(_prop_graphic->GetString(),pv->GetLinkString()) )
    {
		Graphic* gr = GetGraphic();
		gr->Remove(_prop_graphic);
        delete _prop_graphic;
        _prop_graphic = new PropGraphic(pv->GetLinkString());
		gr->Append(_prop_graphic);

		Viewer* dct_v = GetViewer();
		if(dct_v)
		{
			Dct* dct_ed = (Dct*)GetViewer()->GetEditor();
			ModifStatusVar* nv=(ModifStatusVar*)dct_ed->GetState("ModifStatusVar");
			nv->SetModifStatus(true);
    	}
    }

	if(to_node->GetSide()=='r')
		to_ng->Align(CenterRight,_prop_graphic,CenterLeft);
	else
		to_ng->Align(CenterLeft,_prop_graphic,CenterRight);
}

Manipulator* WireView::CreateManipulator (
    Viewer* v, Event& e, Transformer* rel, Tool* tool)
{
    Manipulator* m = nil;

    if(tool->IsA(GRAPHIC_COMP_TOOL))
	{
        m = CreateWireCompManip(v, e, rel, tool);
    }
	else if(tool->IsA(MOVE_TOOL))
	{
        Coord* x, *y;
        int n;
		GetVertices(x, y, n);
		Rubberband* rub = new SlidingLineList(nil, nil, x, y, n, e.x, e.y);
        delete x;
        delete y;
        m = new DragManip(v, rub, rel, tool);

    }
    return m;
}

Command* WireView::InterpretManipulator (Manipulator* m)
{
    DragManip* dm = (DragManip*) m;
    Editor* ed = dm->GetViewer()->GetEditor();
    Tool* tool = dm->GetTool();
    Transformer* rel = dm->GetTransformer();
    Command* cmd = nil;

    if (tool->IsA(GRAPHIC_COMP_TOOL))
	{
        cmd = InterpWireCompManip(dm);
    }
	else if(tool->IsA(MOVE_TOOL))
	{
        int dummy;
        Coord* xorig, *yorig, *xcur, *ycur;
        float fx0, fy0, fx1, fy1;

        SlidingLineList* sll = (SlidingLineList*) dm->GetRubberband();
        sll->GetOriginal(xorig, yorig, dummy);
        sll->GetCurrent(xcur, ycur, dummy);

        if (rel != nil)
		{
            rel->InvTransform(float(xorig[0]), float(yorig[0]), fx0, fy0);
            rel->InvTransform(float(xcur[0]), float(ycur[0]), fx1, fy1);
        }
        delete xorig; delete yorig; delete xcur; delete ycur;

        cmd = new MoveCmd(ed, fx1-fx0, fy1-fy0);
    }
	else if(tool->IsA(RESHAPE_TOOL)) { } // unimplemented

    return cmd;
}

void WireView::First (Iterator& i) { i.SetValue(_nodeView1); }
void WireView::Last (Iterator& i) { i.SetValue(_nodeView2); }
boolean WireView::Done (Iterator i) { return i.GetValue() == nil; }

void WireView::Next (Iterator& i) 
{ 
    void* v = i.GetValue();

    if (v == nil) i.SetValue(_nodeView1);
    else if (v == _nodeView1) i.SetValue(_nodeView2);
    else i.SetValue(nil);
}

void WireView::Prev (Iterator& i)
{ 
    void* v = i.GetValue();

    if (v == nil) i.SetValue(_nodeView2);
    else if (v == _nodeView1) i.SetValue(nil);
    else i.SetValue(_nodeView1);
}

GraphicView* WireView::GetView (Iterator i){return (GraphicView*)i.GetValue();}
void WireView::SetView (GraphicView* gv, Iterator& i) { i.SetValue(gv); }
WireComp* WireView::GetWireComp () { return (WireComp*) GetSubject(); }

Graphic* WireView::GetGraphic ()
{
    Graphic* gr = GraphicView::GetGraphic();

    if (gr == nil)
	{
        WireComp* wireComp = GetWireComp();
        gr = new Picture/*(wireComp->GetGraphic())*/;
        // SetGraphic(gr); old place

        NodeComp* n1, *n2;
        wireComp->GetNodes(n1, n2);
        _nodeView1 = (NodeView*) n1->Create(COMPONENT_VIEW);
        _nodeView2 = (NodeView*) n2->Create(COMPONENT_VIEW);

		_prop_graphic = new PropGraphic("PP/MS");

        n1->Attach(_nodeView1);
        n2->Attach(_nodeView2);
        _nodeView1->Update();
        _nodeView2->Update();

		_nodeView1->GetGraphic()->Align(CenterLeft,_prop_graphic,CenterRight);

        Graphic* ml = wireComp->GetMultiLine()->Copy();
        gr->Append(ml, _nodeView1->GetGraphic(), _nodeView2->GetGraphic());
		gr->Append(_prop_graphic);
        SetGraphic(gr);
    }
    return gr;
}

ClassId WireView::GetClassId () { return WIRE_VIEW; }
boolean WireView::IsA (ClassId id){return WIRE_VIEW==id||GraphicView::IsA(id);}

void WireView::CreateHandles () {
    Coord x[2], y[2];
    Viewer* v = GetViewer();
    
    if (v != nil) {
        GetEndpoints(x[0], y[0], x[1], y[1]);
        _handles = new RubberHandles(nil, nil, x, y, 2, 0, HANDLE_SIZE);
        v->InitRubberband(_handles);
    }
}

MultiLine* WireView::GetMultiLine () {
    Iterator i;
    Graphic* gr = GetGraphic();

    gr->First(i);
    return (MultiLine*) gr->GetGraphic(i);
}

void WireView::GetEndpoints (Coord& x0, Coord& y0, Coord& x1, Coord& y1) {
    float x, y;

    _nodeView1->GetGraphic()->GetCenter(x, y);
    x0 = round(x); y0 = round(y);

    _nodeView2->GetGraphic()->GetCenter(x, y);
    x1 = round(x); y1 = round(y);
}

void WireView::GetVertices (Coord*& x, Coord*& y, int& n) {
    const Coord *vx, *vy;
    Transformer total;
    MultiLine* ml = GetMultiLine();

    n = ml->GetOriginal(vx, vy);
    ml->TotalTransformation(total);
    ArrayDup(vx, vy, n, x, y);
    total.TransformList(x, y, n);
}

void WireView::UpdateMultiLine () {
    Graphic* gr = GetGraphic();
    MultiLine* ml = GetMultiLine();

    gr->Remove(ml);
    delete ml;
    gr->Prepend(GetWireComp()->GetMultiLine()->Copy());
}

Manipulator* WireView::CreateWireCompManip (
    Viewer* v, Event& e, Transformer* rel, Tool* tool)
{
    Selection* s = v->GetSelection();
    GraphicView* views = v->GetGraphicView();
    ConnectorView* target = views->ConnectorIntersecting(
        e.x-SLOP, e.y-SLOP, e.x+SLOP, e.y+SLOP);
    Coord x[1], y[1];
    x[0] = e.x;
    y[0] = e.y;

    s->Clear();

    if (target != nil) {
        float fx, fy, tx, ty;
        target->GetConnector()->GetCenter(fx, fy);
        rel->Transform(fx, fy, tx, ty);
        x[0] = round(tx);
        y[0] = round(ty);
    }
    RubberWire* rw = new RubberWire(nil, nil, x, y, 1);
    return new WireManip(v, rw, rel, tool);
}

Command* WireView::InterpWireCompManip (Manipulator* m)
{
    Viewer* v = m->GetViewer();
    Editor* ed = v->GetEditor();
    GraphicView* view = v->GetGraphicView();
    BrushVar* brVar = (BrushVar*) ed->GetState("BrushVar");
    WireManip* wm = (WireManip*) m;
    Transformer* rel = wm->GetTransformer();
    RubberWire* rw = wm->GetRubberWire();
    MacroCmd* macroCmd = new MacroCmd(ed);
    ConnectorView* target1, *target2;

    if (rel != nil)
	{
        rel = new Transformer(rel);
        rel->Invert();
    }

    FullGraphic gs(stdgraphic);
    if (brVar != nil)
	{
        gs.SetBrush(brVar->GetBrush());
    }
    gs.SetTransformer(rel);

    Coord* x, *y;
    int n;
    
    rw->GetCurrent(x, y, n);
    WireComp* wireComp = new WireComp(x, y, n, &gs);
    Coord x0 = x[0], y0 = y[0];
    Coord x1 = x[n-1], y1 = y[n-1];
    delete x;
    delete y;
    NodeComp *n1, *n2;
    wireComp->GetNodes(n1, n2);
    NodeComp *ctarget1, *ctarget2;

    macroCmd->Append(new PasteCmd(ed, new Clipboard(wireComp)));

    target1 = view->ConnectorIntersecting(x0-SLOP, y0-SLOP, x0+SLOP, y0+SLOP);
    target2 = view->ConnectorIntersecting(x1-SLOP, y1-SLOP, x1+SLOP, y1+SLOP);

	if(target1 !=nil && target2 !=nil )
	{
		ctarget1=(NodeComp*)target1->GetConnector();
		ctarget2=(NodeComp*)target2->GetConnector();
		ctarget1=ctarget1->FindNodeToConnect();
		ctarget2=ctarget2->FindNodeToConnect();
	}
	else
	{
		// put up a warning dialog that link could not be established
		global_warning(ed,"Link Does Not Connect to Anything");
		DeleteCmd* del_cmd = new DeleteCmd(ed);
		delete macroCmd;
		return del_cmd;
	}

	if(ctarget1 != nil && ctarget2 != nil)
	{
		// get the record-field associated with the connecting nodes
		DBENTRY* f1 = ctarget1->GetRecord();
		DBENTRY* f2= ctarget2->GetRecord();
		int f1_is_link=0,f1_type;
		int f2_is_link=0,f2_type;

		// validate the link using db methods here and report errors
		// all link type should be accounted for here:
		// a forward link to a record, a link field to a data field
		// and an external string to a link field
		if(f1)
		{
			f1_type=dbGetFieldType(f1);
			if(f1_type==DCT_INLINK||f1_type==DCT_OUTLINK||f1_type==DCT_FWDLINK)
				f1_is_link=1;
		}
		if(f2)
		{
			f2_type=dbGetFieldType(f2);
			if(f2_type==DCT_INLINK||f2_type==DCT_OUTLINK||f2_type==DCT_FWDLINK)
				f2_is_link=1;
		}
		
		TransMethod tm1 = ctarget1->GetTransMethod();
		TransMethod tm2 = ctarget2->GetTransMethod();

		if(	
			(ctarget1->GetTransMethod()!=ctarget2->GetTransMethod()) && (
			((f1&&f2) && (
			  (f1_is_link && !f2_is_link) || (f2_is_link && !f1_is_link))) || 
			( (f1&&f2==NULL) && f1_is_link ) ||
			( (f2&&f1==NULL) && f2_is_link )
			))
		{
			// ok to link if here
        	macroCmd->Append(new ConnectCmd(ed, n1, ctarget1));
        	macroCmd->Append(new ConnectCmd(ed, n2, ctarget2));

			// the side is used for alignment purposes
			n1->SetSide(ctarget1->GetSide());
			n2->SetSide(ctarget2->GetSide());

			// be sure that method are correct for data flow
			if(ctarget1->GetTransMethod()==In) n1->SetTransMethod(Out);
			else n1->SetTransMethod(In);

			if(ctarget2->GetTransMethod()==In) n2->SetTransMethod(Out);
			else n2->SetTransMethod(In);

			PVLinkVar* pv = wireComp->GetPVLinkVar();
			TF_Direct* tfdir = (TF_Direct*)wireComp->GetTransferFunct();

			if(ctarget1->GetTransMethod()==Out)
			{
				// set in-out for TF_Direct here ctarget2 links to ctarget1
    			tfdir->SetInput(n1->GetBinding());
    			tfdir->SetOutput(n2->GetBinding());
				pv->SetRecField(ctarget2->GetRecord());
        		macroCmd->Append(new LinkFieldCmd(ed, ctarget1));
			}
			else
			{
				// set in-out for TF_Direct here ctarget1 links to ctarget2
    			tfdir->SetInput(n2->GetBinding());
    			tfdir->SetOutput(n1->GetBinding());
				pv->SetRecField(ctarget1->GetRecord());
        		macroCmd->Append(new LinkFieldCmd(ed, ctarget2));
			}
		}
		else
		{
			// put up a warning dialog that link could not be established
			global_warning(ed,"Link Cannot be Established.");
			DeleteCmd* del_cmd = new DeleteCmd(ed);
			delete macroCmd;
			return del_cmd;
		}
	}
	else
	{
		// put up a warning dialog that link could not be established
		global_warning(ed,"Invalid Link\n");
		DeleteCmd* del_cmd = new DeleteCmd(ed);
		delete macroCmd;
		return del_cmd;
	}
    return macroCmd;
}

/**************************the great rubber wire stuff************************/

RubberWire::RubberWire (
    Painter* p, Canvas* c, Coord px[], Coord py[], int n, boolean manhattan
) : GrowingVertices(p, c, px, py, n) {
    _manhattan = manhattan;
    CheckBufs();
    x[n] = x[n-1];
    y[n] = y[n-1];
}

void RubberWire::AppendVertex (Coord vx, Coord vy) {
    ConstrainVertices(vx, vy);
    GrowingVertices::AppendVertex(x[count], y[count]);
}

void RubberWire::Draw () {
    if (!drawn && count > 0) {
        ConstrainVertices(trackx, tracky);
        output->MultiLine(canvas, x, y, count+1);
        drawn = true;
    }
}

void RubberWire::Orthonormalize (Coord tx, Coord ty) {
    int last = count-1;
    boolean vert = abs(tx - x[last]) < abs(ty - y[last]);
    x[count] = vert ? x[last] : tx;
    y[count] = vert ? ty : y[last];
}

void RubberWire::ConstrainVertices (Coord tx, Coord ty) {
    CheckBufs();

    if (_manhattan) {
        int last = count-1;

        if (count > 1 && x[last] == x[last-1]) {
            x[count] = tx;
            y[count] = ty;
            y[last] = ty;
            
        } else if (count > 1 && y[last] == y[last-1]) {
            x[count] = tx;
            y[count] = ty;
            x[last] = tx;

        } else {
            Orthonormalize(tx, ty);
        }
    } else {
        x[count] = tx;
        y[count] = ty;
    }
}

void RubberWire::Manhattan (boolean m) { 
    if (_manhattan != m) {
        boolean wasDrawn = drawn;
        Erase();
        _manhattan = m;

        if (wasDrawn) {
            Draw();
        }
    }
}

