/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#define DCTDIAL_SRC 1

#include <InterViews/strbrowser.h>
#include <InterViews/scroller.h>
#include <InterViews/viewport.h>
#include <InterViews/canvas.h>
#include <InterViews/adjuster.h>
#include <InterViews/border.h>
#include <InterViews/perspective.h>
#include <InterViews/event.h> // for event stuff
#include <InterViews/sensor.h>
#include <InterViews/frame.h>
#include <InterViews/shape.h>
#include <InterViews/glue.h>
#include <InterViews/message.h>
#include <InterViews/world.h>
#include <InterViews/menu.h>
#include <InterViews/filebrowser.h>
#include <InterViews/streditor.h>
#include <InterViews/box.h>
#include <InterViews/banner.h>
#include <Unidraw/unidraw.h>
#include <Unidraw/ulist.h>
 
#include <stream.h>
#include <memory.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
 
extern "C" {
#include "epicsVersion.h"
#include "dbStaticLib.h"
};
#include "dctdial.h"
#include "dctclasses.h"
#include "dctcomps.h"
#include "dctrec.h"

class my_StringEditor;

// -----------------------shaper thing---------------------------
class Shaper : public MonoScene
{
public:
    Shaper(const char*, int, int, int, int, int, int);
    void Reconfig();
private:
    int _h, _v;
    int _hstr, _vstr;
    int _hshr, _vshr;
};
 
Shaper::Shaper(const char* name,int h,int v,int hstr,int vstr,int hshr,int vshr)
{
    SetInstance(name);
    _h = h,
	_v = v;
	_hstr = hstr,
	_vstr = vstr;
	_hshr = hshr,
	_vshr = vshr;
}
 
void Shaper::Reconfig()
{
    MonoScene::Reconfig();

    // shape->width = _h;
	// warning: width is not set here, it uses the width of the component
	if(shape->height > _v) shape->height=_v;
    shape->hstretch = _hstr;
    shape->vstretch = _vstr;
    shape->hshrink = _hshr;
    shape->vshrink = _vshr;
}

class my_PulldownMenu : public PulldownMenu
{
public:
	my_PulldownMenu(const char* n, const char* s,DBENTRY* d):PulldownMenu(n,s)
		{ dbCopyEntryContents(d,&field); }

	DBENTRY* GetField() { return &field; }
private:
	DBENTRY field;
};

//------------------class for string editor-------------------------
class my_StringEditor : public StringEditor
{
public:
	my_StringEditor(ButtonState*, const char*,const char*);

	void SetField(DBENTRY* fp) { if(fp) dbCopyEntryContents(fp,&field); }
	DBENTRY* GetField() { return &field; }
	void FieldUpdate();
	void RereadField();

	virtual void Handle(Event&);
	virtual void Update();
private:
	DBENTRY field;
};

// ----------------------------string browser private button for menu---------

class menu_ButtonState : public ButtonState
{
public:
	menu_ButtonState(int,my_StringEditor*);
	virtual void Notify();
	void Wundo();
	void SetInter(Interactor* i) { inter=i; }
	void SetField(DBENTRY* fp) { if(fp) dbCopyEntryContents(fp,&field); }
	DBENTRY* GetField() { return &field; }
private:
	Interactor* inter;
	DBENTRY field;
	my_StringEditor* ass_se;
};

menu_ButtonState::menu_ButtonState(int i,my_StringEditor* se) : ButtonState(i)
{
	ass_se=se;
}

void menu_ButtonState::Notify()
{
	int v;
	ButtonState::Notify();

	GetValue(v);
	if(v)
	{
		SetValue(0);
		StringBrowser* sb=(StringBrowser*)inter;

		if(dbPutString(GetField(),sb->String(sb->Selection(0))))
			global_world_warning(inter,"Field update failed");

		if(ass_se) ass_se->RereadField();
	}
}

//----------------------form control function--------------------
static void devform(Interactor* inter, DBENTRY* field)
{
	DBENTRY assoc_fp;
	char **value, **prompt;
	int total;
	char var_area[100];
	char* pc;
    Coord x,y;
	my_PB* pb = (my_PB*)inter;

	total=dbAllocForm(field);
	if(total==0) return;

	prompt=dbGetFormPrompt(field);
	value=dbGetFormValue(field);

	World* w = inter->GetWorld();
	inter->GetPosition(x,y);
	Canvas* c = inter->GetCanvas();

	x+=c->pwidth()/2;
	y+=c->pheight()/2;

	/*
    inter->GetRelative(x,y,w);
	if( x < 0 ) x=0;
	if( x > w->Width() ) x=w->Width();
	if( y < 0 ) y=0;
	if( y > w->Height() ) y=w->Height();
	*/

	if(pc=dbGetRelatedField(field))
	{
		dbCopyEntryContents(field,&assoc_fp);
		if(dbFindField(&assoc_fp,pc))
		{
			cerr << "Serious problem!!!\n";
			cerr << "  record " << dbGetRecordName(field);
			cerr << "." << dbGetFieldName(field) << "\n";
			cerr << "has associated that does not exist (" << pc << ")\n";
			dbFreeForm(field);
			return;
		}
	}

	strcpy(var_area,dbGetString(&assoc_fp));
	MenuFormDialog mfd(value,prompt,total,var_area);

	// insert the dialog into the world
    w->InsertTransient(&mfd,inter,x,y,TopCenter);

	if(mfd.Confirm())
	{
		// ok was hit
#if 0
		pc=strchr(var_area,'$');
		if( pc && pc[1]=='(' && strchr(pc,')') )
		{
			// a variable has been entered
			if(dbPutString(&assoc_fp,var_area))
				global_world_warning(inter,"update failed");
		}
		else if(dbPutForm(field,value))
#endif
		if(dbPutForm(field,value))
		{
			cerr << "Serious problem in devform() get_form\n";
			global_world_warning(inter,"Failure to save form!!");
			return;
		}

		if(pb->inout_se)
			pb->inout_se->Message(dbGetString(&assoc_fp));
	}
		// else cancel was hit

	dbFreeForm(field);
    w->Remove(&mfd);
}

// ---------class used to get value selected from a menu item-----
class my_menuitem: public MenuItem
{
public:
    my_menuitem(const char*,Interactor*,DBENTRY*,my_StringEditor* se,
		my_StringEditor* ase);

    virtual void Do();
	DBENTRY* GetField() { return curr_field; }
private:
	DBENTRY* curr_field;
	my_StringEditor* assoc_se;
	my_StringEditor* ass_se;
};
 
my_menuitem::my_menuitem(const char* it,Interactor* i,
                DBENTRY* fp,my_StringEditor* se,my_StringEditor* ase) 
				: MenuItem(it, i)
{
	curr_field=fp;
	assoc_se=se;
	ass_se=ase;
}

void my_menuitem::Do()
{
	if( strcmp(dbGetString(GetField()),GetInstance()) )
	{
    	if(dbPutString(GetField(),(char*)GetInstance()))
			global_world_warning(this,"Field update failed");
		else
		{
			if(assoc_se) assoc_se->Message(GetInstance());
		}
		if(ass_se) ass_se->RereadField();

	}
}

my_StringEditor::my_StringEditor(ButtonState* b, const char* s,const char* d)
		: StringEditor(b,s,d) { /* change_tab_group(true); */ }

void my_StringEditor::Update()
{
	StringEditor::Update();
}

void my_StringEditor::FieldUpdate()
{
	if(dbPutString(GetField(),(char*)Text()))
	{
		// should grab the cursor here so field can be updated
		global_world_warning(this,"Field update failed");
		Message("");
		Update();
	}
}
void my_StringEditor::RereadField()
{
	Message(dbGetString(GetField()));
	Update();
}

void my_StringEditor::Handle(Event& e)
{
	StringEditor::Handle(e);
	// time to update the field cause the guy left
	FieldUpdate();
}

class EL_StringBrowser : public StringBrowser
{
public:
	EL_StringBrowser(
		ButtonState* bs, int r, int c, boolean u);
};

EL_StringBrowser::EL_StringBrowser(
	ButtonState* bs, int r, int c, boolean u):StringBrowser(bs,r,c,u)
{
	input->Ignore(KeyEvent);
}


//------------------export list dialog------------------------
ExportList::ExportList() : 
	BasicDialog(new ButtonState,"Field Export List","Unexported Fields")
{
    const int space = round(.5*cm);

	browserAdd = new EL_StringBrowser(new ButtonState,10,12,false);
	browserRemove = new EL_StringBrowser(new ButtonState,10,12,false);
	Insert(
		new MarginFrame(
			new VBox(
				new HBox(_title,new HGlue),
				new VGlue(space,0),
				new HBox(
				    new VBox(
				    	new Banner("Add",NULL,NULL),
				    	new Frame( AddScroller(browserAdd) )
				    ),
				    new HGlue(space,0),
				    new VBox(
				    	new Banner("Remove",NULL,NULL),
				    	new Frame( AddScroller(browserRemove) )
				    )
				),
				new VGlue(space,0),
				new HBox(
					new VGlue(space,0),
					new HGlue,
					new PushButton(" Cancel ",state,2),
					new HGlue(space,0),
					new PushButton(" Apply ",state,1)
				)
			), space, space/2, 0
		)
	);
}

ExportList::~ExportList() { delete browserAdd; delete browserRemove; }

Interactor* ExportList::AddScroller(Interactor* i)
{
    return new HBox(
        new MarginFrame(i, 2),
        new VBorder,
        new VBox(
            new UpMover(i, 1),
            new HBorder,
            new VScroller("field_scroller",i),
            new HBorder,
            new DownMover(i, 1)
        )
    );
}

static int exp_comp(const void* i,const void* j)
{
	char* a=(char*)i;
	char* b=(char*)j;
	int rc;
	rc=strcmp(a,b);
	return rc;
}

void ExportList::my_go(DBENTRY* rp, RecComp *rec)
{
	DBENTRY dbe;
	long rc;
	int tot,x,y,i;
	char* f;
	UList *rec_list, *u;
	NodeComp *nodeComp;

	dbCopyEntryContents(rp,&dbe);
    browserAdd->Clear();

	tot=dbGetNFields(&dbe,0);
	f=new char[tot*5]; // 5 bytes for each field name
	memset(f,0,tot*5);

	for(x=0,rc=dbFirstField(&dbe,0);rc==0;rc=dbNextField(&dbe,0),x++)
		strcpy(&f[x*5],dbGetFieldName(&dbe));

	qsort(f,tot,5,exp_comp);
 
	for(y=0;y<x;y++) {
	    boolean appended = false;
	    for(i=0;i<3;i++) {
	    	switch(i) {
		    case 0:
		    rec_list = rec->GetInList();
		    break;
		    case 1:
		    rec_list = rec->GetOutList();
		    break;
		    case 2:
		    rec_list = rec->GetDataList();
		    break;
		}
	    	for(u = rec_list->First(); u!=rec_list->End(); u=u->Next())
	    	{
		    nodeComp = (NodeComp*)(*u)();
		    if( strcmp(nodeComp->GetFieldName(),&f[y*5])==0 ) {
		    	if ( !nodeComp->HasConnections() && i != 2 )
			    browserRemove->Append(&f[y*5]);
			else if ( i == 2 && !nodeComp->HasConnections() ) {
			    u=u->Next();
		    	    nodeComp = (NodeComp*)(*u)();
			    if ( !nodeComp->HasConnections() )
			    	browserRemove->Append(&f[y*5]);
			}
			appended = true;
			break;
		    }
	    	}
	    }
    	    if( !appended )
    	    	browserAdd->Append(&f[y*5]);
	}
		

    // insert field instances into list
	// for(rc=dbFirstField(&dbe,0);rc==0;rc=dbNextField(&dbe,0))
	//	browser->Append(dbGetFieldName(&dbe));

	delete [] f;
}

int ExportList::CountAdd() { return browserAdd->Selections(); }

char* ExportList::GetSelectAdd(int i)
{
	return browserAdd->String(browserAdd->Selection(i));
}

int ExportList::CountRemove() { return browserRemove->Selections(); }

char* ExportList::GetSelectRemove(int i)
{
	return browserRemove->String(browserRemove->Selection(i));
}

//------------------field edit dialog------------------------
FieldEditDialog::FieldEditDialog(DBENTRY* rp)
	: BasicDialog(new ButtonState,"edit fields",dbGetRecordName(rp))
{
    const int space = round(.5*cm);
	if(rp) dbCopyEntryContents(rp,&_rec);
	_edit_box = new VBox;
	Insert(
		new MarginFrame(
			new VBox(
				new HBox(_title,new HGlue),
				new HBox(_subtitle,new HGlue),
				new VGlue(space,0),
				new Frame( FEDinterior() ),
				new VGlue(space,0),
				new HBox(
					new VGlue(space,0),
					new HGlue,
					new PushButton(" Done ",state,1),
					new HGlue
				)
			),space, space/2, 0
		)
	);
	input = new Sensor(noEvents);
	input->Catch(KeyEvent);
}

FieldEditDialog::~FieldEditDialog() { delete _edit_box; }

void FieldEditDialog::GetData()
{
    Event e;
    int v = 0;
 
    state->SetValue(v);
    do {
        Read(e);
        if (e.eventType == KeyEvent) {
            state->SetValue(e.keystring[0]);
        } else {
            Forward(e);
        }
        state->GetValue(v);
    } while (v!=1);
}

Interactor* FieldEditDialog::AddScroller(Interactor* i)
{
    return new HBox(
        new MarginFrame(i, 2),
        new VBorder,
        new VBox(
            new UpMover(i, 1),
            new HBorder,
            new VScroller(i),
            new HBorder,
            new DownMover(i, 1)
        )
    );
}

Interactor* FieldEditDialog::FEDinterior()
{
	char curr_line[150];
	int menu_num,ftype,i,biggest;
	my_menuitem* mi;
	char** menu_list;
	char longest_mess[100];
	char* pc;
	DBENTRY ass_rec;
	my_PB* mpb;
	menu_ButtonState* mbs;
	Message* mess;
	my_PulldownMenu* fld_menu;
	my_StringEditor *fld_item,*fld_value,*fld_var;
	long rc;
 
	pc=NULL;
	mpb=NULL;

    // insert field instances into list
	for(rc=dbFirstField(&_rec,1);rc==0;rc=dbNextField(&_rec,1))
    {
		fld_menu=NULL;
		fld_item=NULL;

		switch( (ftype=dbGetFieldType(&_rec)) )
		{
			case DCT_MENUFORM:
				if(pc=dbGetRelatedField(&_rec))
				{
					dbCopyEntryContents(&_rec,&ass_rec);
					if(dbFindField(&ass_rec,pc)==0)
					{
						fld_item=new my_StringEditor(new ButtonState,
							"123456789012345678901234567890123456","\r\t\n");
						fld_item->Message(dbGetString(&ass_rec));
						fld_item->SetField(&ass_rec);
						// create the message part
						sprintf(curr_line,"%-25s %-4s %-17s",
							dbGetPrompt(&ass_rec),dbGetFieldName(&ass_rec),
							db_types[dbGetFieldType(&ass_rec)]);
						mess = new Message(curr_line,Left,2,1000000,0);
						entry_box = new HBox;
                		entry_box->Insert(mess);
                		entry_box->Insert(new HGlue);
                		entry_box->Insert(new Frame(fld_item));
                		entry_box->Insert(new HGlue);
						_edit_box->Insert(entry_box);
					}
				}
			case DCT_MENU:
				sprintf(curr_line,"%-25s %-4s %-17s",
					dbGetPrompt(&_rec),dbGetFieldName(&_rec),db_types[ftype]);
				mess=new Message(curr_line,Left,2,1000000,0);
				entry_box = new HBox;
                entry_box->Insert(new VBox(new VGlue,mess,new VGlue));

				fld_var=NULL;
				fld_value=new my_StringEditor(new ButtonState,
					"12345678901234567890","\r\t\n");
				fld_value->Message(dbGetString(&_rec));
				fld_value->SetField(&_rec);

                menu_list = dbGetMenuChoices(&_rec);
                menu_num = dbGetNMenuChoices(&_rec);
 
               	for(biggest=0,i=0;i<menu_num;i++)
               	{
					if(strlen(menu_list[i])>biggest)
						biggest=strlen(menu_list[i]);
               	}

				if(menu_num < 25)
				{
					// an option menu of less than 25 items is ok
					// this is dumb, find the longest menu string

					memset(longest_mess,' ',sizeof(longest_mess));
					strcpy(longest_mess,"Menu");
					longest_mess[biggest]='\0';

					/* the problem is appearing here in the device menu,
					when there are more than 25 items inserted */
  
                	fld_menu = new my_PulldownMenu("Device Menu","Menu",&_rec);

                	for(i=0;i<menu_num;i++)
                	{
                    	mi = new my_menuitem(menu_list[i],
                        	new Message(menu_list[i],Left,2,1000000,0),
							fld_menu->GetField(),fld_value,fld_item);

                    	fld_menu->Include(mi);
                	}
                	entry_box->Insert(new HGlue);
                	entry_box->Insert(fld_menu);
                }
				else
				{
					// use a popup type of scrolled list here
					mbs = new menu_ButtonState(0,fld_item);
					StringBrowser* sb =
						new StringBrowser("menu_sb",mbs,5,biggest,true);
					mbs->SetField(&_rec);
					mbs->SetInter(sb);
					sb->Clear();
                	for(i=0;i<menu_num;i++)
						sb->Append(menu_list[i]);
					int sel_inx = sb->Index(dbGetString(&_rec));
					if( sel_inx >= 0 && sel_inx < sb->Count() )
						sb->Select(sel_inx);
                	entry_box->Insert(new HGlue);
                	entry_box->Insert(AddScroller(sb));
				}

				if(ftype==DCT_MENUFORM)
				{
					mpb = new my_PB(&_rec);
					mpb->SetSE(fld_item);
					mpb->SetDial(mpb /*this*/);
                	entry_box->Insert(new HGlue);
					entry_box->Insert( mpb );
				}
        		entry_box->Insert(new HGlue);
        		entry_box->Insert(new Frame(fld_value));
                entry_box->Insert(new HGlue);
				_edit_box->Insert(entry_box);
				break;
			default:
				if(pc && strcmp(pc,dbGetFieldName(&_rec))==0) break;

				fld_value=new my_StringEditor(new ButtonState,
					"123456789012345678901234567890123456","\r\t\n");
				fld_value->Message(dbGetString(&_rec));
				fld_value->SetField(&_rec);

				// create the message part
				sprintf(curr_line,"%-25s %-4s %-17s",
					dbGetPrompt(&_rec),dbGetFieldName(&_rec),
					db_types[dbGetFieldType(&_rec)]);
				mess = new Message(curr_line,Left,2,1000000,0);
				entry_box = new HBox;
                entry_box->Insert(mess);
                entry_box->Insert(new HGlue);
                entry_box->Insert(new Frame(fld_value));
                entry_box->Insert(new HGlue);
				_edit_box->Insert(entry_box);
				break;
		}
    }

    World* w = unidraw->GetWorld();
	const char* dialogSize = w->GetAttribute("dialogSize");
	double pos;
	sscanf(dialogSize,"%lf",&pos);
	Shape* shap = _edit_box->GetShape();
	Shape new_shape = *shap;
	new_shape.vunits = 50;
	_edit_box->Reshape(new_shape);
	viewport = new Viewport("field_viewer", NULL, TopCenter);
	viewport->Insert(_edit_box);
	Coord x = (Coord)(w->Height()*pos);
	Shaper* s = new Shaper("Shaper",0,x,100000,0,100000,100000);
	Interactor* scr = AddScroller(viewport);
	s->Insert(scr);
	return s;
}

// -----------------------------pp/ms dialog box-----------------------
ppms_edit::ppms_edit(const char* t, const char* c_pp, const char* c_ms) : 
			BasicDialog(new ButtonState,t,"Set Attributes")
{
    const int space = round(.5*cm);
	int x=0;

	if(strcmp(c_pp,"PP")==0)
		x=0;
	else if(strcmp(c_pp,"NPP")==0)
		x=1;
	else if(strcmp(c_pp,"CA")==0)
		x=2;
	else if(strcmp(c_pp,"CP")==0)
		x=3;
	else if(strcmp(c_pp,"CPP")==0)
		x=4;

	_pp_bs = new ButtonState(x);
	_ms_bs = new ButtonState( strcmp(c_ms,"MS")==0?0:1 );

	_ca_rb = new RadioButton("ppms","CA",_pp_bs,2);
	_cp_rb = new RadioButton("ppms","CP",_pp_bs,3);
	_cpp_rb = new RadioButton("ppms","CPP",_pp_bs,4);
	_pp_rb = new RadioButton("ppms","PP",_pp_bs,0);
	_npp_rb = new RadioButton("ppms","NPP",_pp_bs,1);
	_ms_rb = new RadioButton("ppms","MS",_ms_bs,0);
	_nms_rb = new RadioButton("ppms","NMS",_ms_bs,1);

	Insert(
		new MarginFrame(
			new VBox(
				new HBox(_title,new HGlue),
				new HBox(_subtitle,new HGlue),
				new VGlue(space),
				new HBox(
					new HGlue,
					new Frame(new VBox(_pp_rb,_npp_rb,_ca_rb,_cp_rb,_cpp_rb)),
					new HGlue(space,0),
					new Frame( new VBox( _ms_rb, _nms_rb) ),
					new HGlue
				),
				new VGlue(space),
				new HBox(
					new HGlue,
					new PushButton(" Cancel ",state,2),
					new HGlue(space,0),
					new PushButton(" Ok ",state,1),
					new HGlue
				)
			), space, space/2, 0
		)
	);
}

ppms_edit::~ppms_edit()
{
	delete _ca_rb;
	delete _cp_rb;
	delete _cpp_rb;
	delete _pp_rb;
	delete _npp_rb;
	delete _nms_rb;
	delete _ms_rb;
	delete _pp_bs;
	delete _ms_bs;
}

char* ppms_edit::GetPP()
{
	char *rp_val;
	int p_val;

	_pp_bs->GetValue(p_val);
	switch(p_val)
	{
	case 0: rp_val="PP"; break;
	case 1: rp_val="NPP"; break;
	case 2: rp_val="CA"; break;
	case 3: rp_val="CP"; break;
	case 4: rp_val="CPP"; break;
	default: break;
	}
	return rp_val;
}

char* ppms_edit::GetMS()
{
	char *rm_val;
	int m_val;

	_ms_bs->GetValue(m_val);
	if(m_val==0) rm_val="MS"; else rm_val="NMS";
	return rm_val;
}

// -----------------------------external link dialog box---------------------
extlink_edit::extlink_edit(const char* t, const char* name, const char type) : 
			BasicDialog(new ButtonState,t,"Create external link")
{
    const int space = round(.5*cm);
	int bval;

	switch(type)
	{
	case 'i': bval=0; break;
	case 'o': bval=1; break;
	case 'b': bval=2; break;
	default: break;
	}

	_se=new StringEditor(new ButtonState,
			"123456789012345678901234567890123456","");
	_se->Message(name);

	_type_bs = new ButtonState(bval);

	_type_in = new RadioButton("extlink","Left",_type_bs,0);
	_type_out = new RadioButton("extlink","Right",_type_bs,1);
	_type_io = new RadioButton("extlink","Both",_type_bs,2);

	Insert(
		new MarginFrame(
			new VBox(
				new HBox(_title,new HGlue),
				new HBox(_subtitle,new HGlue),
				new VGlue(space),
				new HBox(
					new HGlue,
					new Frame(_se),
					new HGlue(space,0),
					new Frame( new VBox( _type_in, _type_out, _type_io) ),
					new HGlue
				),
				new VGlue(space),
				new HBox(
					new HGlue,
					new PushButton(" Cancel ",state,2),
					new HGlue(space,0),
					new PushButton(" Ok ",state,1),
					new HGlue
				)
			), space, space/2, 0
		)
	);
}

extlink_edit::~extlink_edit()
{
	delete _se;
	delete _type_in;
	delete _type_out;
	delete _type_io;
	delete _type_bs;
}

const char* extlink_edit::GetLinkString() { return _se->Text(); }

const char extlink_edit::GetLinkType()
{
	int val;
	char c;

	_type_bs->GetValue(val);

	switch(val)
	{
	case 0: c='i'; break;
	case 1: c='o'; break;
	default:
	case 2: c='b'; break;
	}

	return c;
}

//---------------class for private button-------------------
class my_ButtonState : public ButtonState
{
public:
	my_ButtonState(int);
	virtual void Notify();
	void Wundo();
	void SetInter(Interactor* i) { inter=i; }
	void SetField(DBENTRY* fp) { if(fp) dbCopyEntryContents(fp,&field); }
	DBENTRY* GetField() { return &field; }
private:
	Interactor* inter;
	DBENTRY field;
};

my_ButtonState::my_ButtonState(int i) : ButtonState(i) { }

void my_ButtonState::Notify()
{
	int v;
	ButtonState::Notify();

	GetValue(v);
	if(v)
	{
		SetValue(0);
		Wundo();
	}
}

void my_ButtonState::Wundo()
{
	devform(inter,GetField());
}

//------------------class for menu form string editor-------------------
class MFStringEditor : public StringEditor
{
public:
	MFStringEditor(ButtonState*, const char*,const char*);

	void SetField(char* fp) { field=fp; }
	char* GetField() { return field; }
	void FieldUpdate();

	virtual void Handle(Event&);
	virtual void Update();
private:
	char* field;
};

MFStringEditor::MFStringEditor(ButtonState* b, const char* s,const char* d)
		: StringEditor(b,s,d) { /* change_tab_group(true); */ }

void MFStringEditor::Update()
{
	StringEditor::Update();
	// find_focus(true);
}

void MFStringEditor::FieldUpdate() { strcpy(field,Text()); }

void MFStringEditor::Handle(Event& e)
{
	StringEditor::Handle(e);
	FieldUpdate();
	// next_focus();#endif
}

// -----------------------------menu form dialog ----------------------
MenuFormDialog::MenuFormDialog(char** value, char** prompt, int num,char* l):
			BasicDialog(new ButtonState,"Enter Data",l)
{
    const int space = round(.5*cm);
	VBox* vb = new VBox;
	HBox* hb;
	Message* mess;
	MFStringEditor* mfse;
	char curr_line[50];
	int i;

	for(i=0;i<num;i++)
	{
		// sprintf(curr_line,"%35s", prompt[i]);
		// mess = new Message(curr_line,Left,2,1000000,0);
		mess = new Message(prompt[i],Right,2,1000000,0);
		mfse = new MFStringEditor(new ButtonState,"123456789012345678901234567890","\r\t\n");
		mfse->Message(value[i]); // the old value
		mfse->SetField(value[i]); // place to put the data

		vb->Insert( new HBox(mess,new HGlue,new Frame(mfse),new HGlue) );
	}

#if 0
	mess = new Message("Enter Variable String:",Left,2,1000000,0);
	mfse = new MFStringEditor(new ButtonState,"1234567890123456789012345","\r\t\n");
	mfse->Message(l); // the old value
	mfse->SetField(l); // place to put the data
	hb = new HBox(new HGlue,mess,new HGlue,mfse,new HGlue(space,0),new HGlue);
#endif

	Insert(
		new MarginFrame(
			new VBox(
				new VBox(
					new HBox(_title,new HGlue),
					new HBox(_subtitle,new HGlue)
				),
#if 0
				new VGlue(space),
				new Frame(hb),
#endif
				new VGlue(space),
#if 0
				new HBox(
					new HGlue,
#endif
					/* new Frame( */ vb /*)*/,
#if 0
					new HGlue(space,0),
					new HGlue
				),
#endif
				new VGlue(space),
				new HBox(
					new HGlue,
					new PushButton(" Cancel ",state,2),
					new HGlue(space,0),
					new PushButton(" Ok ",state,1),
					new HGlue
				)
			), space, space/2, 0
		)
	);
}

MenuFormDialog::~MenuFormDialog() { }

boolean MenuFormDialog::Confirm()
{
    Event e;
    int v = 0;
 
    state->SetValue(v);
    do {
        Read(e);
        if (e.eventType == KeyEvent) {
            state->SetValue(e.keystring[0]);
        } else {
            Forward(e);
        }
        state->GetValue(v);
    } while (v!=1 && v!=2);

	if(v==1) return true;
	else return false;
}

// -------------------------my pushbutton ----------------------------

my_PB::~my_PB() {}

my_PB::my_PB(DBENTRY* fp): PushButton("my_pb","Form",new my_ButtonState(0),1)
{
	my_ButtonState* bs = (my_ButtonState*)subject;
	bs->SetField(fp);
	bs->SetInter(big_dial);
	SetField(fp);
	SetSE(NULL);
}

void my_PB::SetDial(Interactor* i)
{
	big_dial=i;
	my_ButtonState* bs = (my_ButtonState*)subject;
	bs->SetInter(big_dial);
}
