/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#ifndef dctdial_h
#define dctdial_h

#include <Unidraw/dialogs.h>
#include <InterViews/filechooser.h>
#include <InterViews/button.h>

#include "dctrec.h"

extern "C" {
#include "dbStaticLib.h"
};

class ButtonState;
class RadioButton;
class OptionMenu;
class HBox;
class VBox;
class Interactor;
class Viewport;
class my_StringEditor;

#ifdef DCTDIAL_SRC
char* db_types[] = {
	"STRING",
	"INTEGER",
	"REAL",
	"MENU",
	"MENUFORM",
	"INLINK",
	"OUTLINK",
	"FWDLINK",
	"NOACCESS"
};
#else
extern char* db_types[];
#endif

class ExportList: public BasicDialog
{
public:
	ExportList();
	~ExportList();

	Interactor* AddScroller(Interactor*);
	StringBrowser* GetBrowser() { return browserAdd; }
	void my_go(DBENTRY* rp, RecComp *rec);
	int CountAdd();
	char* GetSelectAdd(int);
	int CountRemove();
	char* GetSelectRemove(int);

private:
	StringBrowser* browserAdd, *browserRemove;
};

class FieldEditDialog: public BasicDialog
{
public:
	FieldEditDialog(DBENTRY*);
	~FieldEditDialog();

	Interactor* AddScroller(Interactor*);
	VBox* GetEditBox() { return _edit_box; }
	void GetData();

private:
	Interactor* FEDinterior();
	VBox* _edit_box;
	HBox* entry_box;
	Viewport* viewport;
	DBENTRY _rec;
};

class ppms_edit: public BasicDialog
{
public:
	ppms_edit(const char*, const char*, const char*);
	~ppms_edit();

	char* GetPP();
	char* GetMS();
private:
	ButtonState* _pp_bs;
	ButtonState* _ms_bs;
	RadioButton* _pp_rb;
	RadioButton* _npp_rb;
	RadioButton* _ms_rb;
	RadioButton* _nms_rb;
	RadioButton* _ca_rb;
	RadioButton* _cp_rb;
	RadioButton* _cpp_rb;
};

class extlink_edit: public BasicDialog
{
public:
	extlink_edit(const char* title, const char* name, const char type);
	~extlink_edit();

	const char* GetLinkString();
	const char GetLinkType();
private:
	StringEditor* _se;
	ButtonState* _type_bs;
	RadioButton* _type_in;
	RadioButton* _type_out;
	RadioButton* _type_io;
};

class MenuFormDialog: public BasicDialog
{
public:
	MenuFormDialog(char** value, char** prompt, int total,char* label);
	virtual ~MenuFormDialog();

	boolean Confirm();
};

class my_PB : public PushButton
{
public:
	my_PB(DBENTRY*);
	~my_PB();

	void SetDial(Interactor* i);
	DBENTRY* GetField() { return &field; }
	void SetField(DBENTRY* fp) { if(fp) dbCopyEntryContents(fp,&field); }
	void SetSE(my_StringEditor* se) { inout_se=se; }

	my_StringEditor* inout_se;
private:
	DBENTRY field;
	Interactor* big_dial;
};

#endif

