/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 * DctEditor implementation.
 */

#define DEBUG_POSITION 0
#define DEBUG_GEOMETRY 0

#if DEBUG_POSITION && !defined(TOOL_PALETTE_TOPRIGHT)
# define TOOL_PALETTE_TOPRIGHT
#endif

#include "dctclasses.h"
#include "dcted.h"
#include "dctdial.h"
#include "dctcomps.h"
#include "dctrec.h"
#include "dctext.h"
#include "dctmanip.h"
#include "dctcmds.h"
#include "dctcat.h"

#include <Unidraw/ctrlinfo.h>
#include <Unidraw/keymap.h>
#include <Unidraw/iterator.h>
#include <Unidraw/kybd.h>
#include <Unidraw/selection.h>
#include <Unidraw/clipboard.h>
#include <Unidraw/uctrls.h>
#include <Unidraw/upage.h>
#include <Unidraw/viewer.h>
#include <Unidraw/statevars.h>
#include <Unidraw/stateviews.h>
#include <Unidraw/grid.h>

#include <Unidraw/Commands/edit.h>
#include <Unidraw/Commands/catcmds.h>
#include <Unidraw/Commands/struct.h>
#include <Unidraw/Components/ellipse.h>
#include <Unidraw/Components/rect.h>
#include <Unidraw/Components/text.h>
#include <Unidraw/Graphic/ellipses.h>
#include <Unidraw/Graphic/polygons.h>
#include <Unidraw/Tools/move.h>
#include <Unidraw/Tools/scale.h>
#include <Unidraw/Tools/rotate.h>
#include <Unidraw/Tools/select.h>
#include <Unidraw/Tools/grcomptool.h>
#include <Unidraw/Tools/reshape.h>

#include <InterViews/window.h>
#include <InterViews/display.h>
#include <InterViews/border.h>
#include <InterViews/frame.h>
#include <InterViews/glue.h>
#include <InterViews/box.h>
#include <InterViews/menu.h>
#include <InterViews/tray.h>
#include <InterViews/panner.h>
#include <InterViews/message.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>

extern "C" {
#include "dbStaticLib.h"
};

static int verbose_save;

//----------------------------dct handler for delete-----------------------
DctHandler::DctHandler() { /* cerr << "in Dcthandler constructor\n"; */ }
DctHandler::~DctHandler() { /* cerr << "in dcthandler destructor\n"; */ }
boolean DctHandler::event(Event&) { return false; /*cerr << "in handler\n"; */ }

void DctUnidraw::SetIncPath(char* path)
{
	Uninode* x;
	char* pc;

	// clear old path
	ClearList(inc_list);
	inc_list_tail=NULL;
	tot_table=0;
	if(inc_path_copy) delete [] inc_path_copy;
	inc_path_copy=strdup(path);

	for(pc=strtok(inc_path_copy," :\t");pc;pc=strtok(NULL," :\t"))
	{
		++tot_table;
		x=inc_list_tail;
		inc_list_tail=new Uninode(pc);
		if(x) x->next=inc_list_tail;
		else  inc_list=inc_list_tail;
	}
}

void DctUnidraw::ClearList(Uninode*& list)
{
	Uninode *x,*n;

	for(x=list;x;)
	{
		n=x->next;
		delete [] x;
		x=n;
	}
	list=NULL;
}

char* DctUnidraw::ResolveName(char* fname)
{
	int i;
	int len;
	char* rc;
	Uninode* node;

	if(fname[0]=='/')
	{
		if(access(fname,R_OK)==0)
			rc=strdup(fname);
		else
			rc=NULL;
	}
	else if(inc_list)
	{
		len=strlen(inc_path)+strlen(fname)+1;
		rc=new char[len+1];

		for(node=inc_list;node;node=node->next)
		{
			sprintf(rc,"%s/%s",node->string,fname);
			if(access(rc,R_OK)==0) break;
		}
		if(node==NULL)
		{
			delete [] rc;
			rc=NULL;
		}
		// else
		// 	cerr << "resolve name " << rc << "\n";
	}
	else
		rc=NULL;

	return rc;
}

int DctUnidraw::SetDefaultDBDs(char** list,int total)
{
	int i,rc=1;

	ClearList(dbd_list);

	for(i=0;i<total;i++)
	{
		if(strstr(list[i],".dbd"))
			if(AddDBD(list[i])==0) rc=0;
	}

	if(rc)
	{
		// see if a dbd file exists in the current directory
		DIR* dirp=opendir(".");
		dirent* dp;
		while((dp=readdir(dirp)))
		{
			if(strstr(dp->d_name,".dbd"))
				if(AddDBD(dp->d_name)==0) rc=0;
		}
		closedir(dirp);
	}

	return rc;
}

int DctUnidraw::AddDBD(char* name)
{
	char* rname;
	int rc=0;
	Uninode* x=dbd_list;

	dbd_list=new Uninode(name,x);

	if((rname=ResolveName(dbd_list->string)))
	{
		if(dbReadDatabase(&epics_db,rname,NULL,NULL))
		{
			if(startup) cerr << "Read failed: dbd file " << rname << "\n";
			rc=1;
		}
		else
			if(startup) cerr << "Read dbd file: " << rname << "\n";

		delete [] rname;
	}
	else
		rc=1;

	return rc;
}

int DctUnidraw::ReadDefaultDBDs(DBBASE*& db)
{
	Uninode* x;
	char* rname;
	int rc=1;

	if(dbd_list)
	{
		for(x=dbd_list;x;x=x->next)
		{
			if((rname=ResolveName(x->string)))
			{
				if(dbReadDatabase(&db,rname,NULL,NULL))
				{
					if(startup)
						cerr << "Read failed: dbd file " << rname << "\n";
				}
				else
				{
					rc=0;
					if(startup)
						cerr << "Read dbd file: " << rname << "\n";
				}
				delete [] rname;
			}
		}
	}
	return rc;
}

//-----------------------------unidraw--------------------------------
DctUnidraw::DctUnidraw(DctCatalog* cat,int& argc,char** argv,
	OptionDesc* od, PropertyData* pd):Unidraw((Catalog*)cat,argc,argv,od,pd)
{
#if OSMajorVersion>4
	char* const* argvv = (char* const*)argv;
#else
	const char* const* argvv = (const char* const*)argv;
#endif
	int j,state,i,s,opt_num=1;
	char *rname,*pc;

	startup=1;
	db_name=NULL;
	inc_path=NULL;
	inc_path_copy=NULL;
	env_path=NULL;
    dbd_list=NULL;
    inc_list=NULL;
    inc_list_tail=NULL;
	epics_db=NULL;
	cat_db=NULL;
	tot_table=0;
	global_prefix[0]='\0';
	verbose_save=0; // global var

	if(argc>opt_num && strcmp(argvv[opt_num],"-v")==0)
	{
		++opt_num;
		verbose_save=1;
	}

	if((log_name=(char*)getenv("LOGNAME"))==NULL)
		log_name="xxx";

	if((inc_path=(char*)getenv("EPICS_DB_INCLUDE_PATH"))==NULL)
	{
		inc_path=strdup(".:./dbd:../dbd:../../dbd:../src");
		env_path=new char[strlen(inc_path)+23];
		sprintf(env_path,"EPICS_DB_INCLUDE_PATH=%s",inc_path);
		putenv(env_path);
	}

	SetIncPath(inc_path);
	cerr << "Using dbd include path " << inc_path << "\n";

	if(SetDefaultDBDs((char**)&argvv[opt_num],(argc>opt_num)?argc-opt_num:0))
	{
		// no dbds read
		// should search for .dbd files in current directory

		if((epics_db=dbAllocBase())==NULL)
		{
			cerr << "Cannot initialize an EPICS database\n";
			exit(1);
		}
	}
	startup=0;
}

DctUnidraw::~DctUnidraw()
{
	if(epics_db) dbFreeBase(epics_db);
	if(cat_db) dbFreeBase(cat_db);
}

void DctUnidraw::SetPrefix(char* new_prefix)
{
	 strcpy((char*)global_prefix,new_prefix);
}

void DctUnidraw::MakeCatDatabase()
{
	if(cat_db) dbFreeBase(cat_db);
	cat_db=NULL;

	if(ReadDefaultDBDs(cat_db))
	{
		// no dbds read
		if((cat_db=dbAllocBase())==NULL)
		{
			cerr << "Cannot initialize an EPICS database\n";
			exit(1);
		}
	}
}

void DctUnidraw::UseCatDatabase()
{
	if(epics_db) dbFreeBase(epics_db);
	epics_db=cat_db;
	cat_db=nil;
}

//--------------------------catalog--------------------------------
DctCatalog::DctCatalog(const char* c, Creator* cre, float version) 
	: Catalog(c,cre,version) { }

DctCatalog::~DctCatalog() {}

boolean DctCatalog::Retrieve(const char*,const char* name, Component*& comp)
{
	char err_msg[100];
	char *p,*db_name;
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetCatDatabase();
	boolean rc=true;

	db_name=new char[strlen(name)+4];
	sprintf(db_name,"%s.db",name);

	if(db)
	{
		if(Valid(name,comp))
			Forget(comp,name);

		if(dbReadDatabase(&db,db_name,NULL,NULL))
		{
			// database could not be read in!!!
			sprintf(err_msg,"db file %s could not be read",name);
			Iterator it;
			unidraw->First(it);
			Editor* ed = unidraw->GetEditor(it);
			global_warning(ed,err_msg);
			rc=false;
		}
		else
		{
			rc=Catalog::Retrieve(name,comp);

			if(rc==true)
				du->SetName(name);
		}
	}
	delete [] db_name;
	return rc;
}

boolean DctCatalog::Save(Component* comp, const char* name)
	{ return Save(comp,name,0); }

boolean DctCatalog::Save(Component* comp, const char* name, const char*)
{
	char err_msg[100];
	char *p,*db_name;
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();
	boolean rc=true;

	db_name=new char[strlen(name)+4];
	sprintf(db_name,"%s.db",name);

	if(dbWriteRecord(db,db_name,NULL,verbose_save))
	{
		// database could not be saved in!!!
		sprintf(err_msg,".db file %s could not be saved",db_name);
		Iterator it;
		unidraw->First(it);
		Editor* ed = unidraw->GetEditor(it);
		global_warning(ed,err_msg);
		return false;
	}
	else
		rc=Catalog::Save(comp,name);

	delete [] db_name;
	return rc;
}

/**************************dct graphiccomps***********************************/

DctGraphicComps::~DctGraphicComps()
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* epics_db = du->GetDatabase();
	DBENTRY dbe;
	long rc;
 
	// clear the nowhere stuff
	// put out warning and ask if the records should be deleted or not
	dbInitEntry(epics_db,&dbe);

	for(rc=dbFirstRecord(&dbe);rc==0;rc=dbNextRecord(&dbe))
	{
		if(!dbIsVisibleRecord(&dbe))
			dbDeleteRecord(&dbe);
	}
}

ClassId DctGraphicComps::GetClassId () { return DCTGRAPHIC_COMPS; }
boolean DctGraphicComps::IsA (ClassId id)
{ return DCTGRAPHIC_COMPS == id || GraphicComps::IsA(id); }

void DctGraphicComps::Interpret(Command* cmd)
{
	Editor* ed = cmd->GetEditor();
	Iterator i;
	Selection* s;
	GraphicView* view;

	if(cmd->IsA(DELETE_CMD)&&ed->GetComponent()==this)
	{
        s=ed->GetSelection();
        for(s->First(i); !s->Done(i); s->Next(i))
		{
            view=s->GetView(i);
			if(view->IsA(REC_VIEW))
			{
				RecView* recview=(RecView*)view;
				RecComp* reccomp=(RecComp*)recview->GetRecComp();

				if(reccomp->HasLinks())
				{
					global_warning(ed,"Record in Selection contains links - Cannot delete");
					return;
				}
			}
			else if(view->IsA(EXTLINK_VIEW))
			{
				ExtLinkView* extview=(ExtLinkView*)view;
				ExtLinkComp* extcomp=(ExtLinkComp*)extview->GetExtLinkComp();

				if(extcomp->HasLinks())
				{
					global_warning(ed,"Record in Selection contains links - Cannot delete");
					return;
				}
			}
        }
		GraphicComps::Interpret(cmd);
	}
	else if(cmd->IsA(DUP_CMD)&&ed->GetComponent()==this)
	{
        s=ed->GetSelection();
        for(s->First(i); !s->Done(i);)
		{
            view=s->GetView(i);
			if(view->IsA(WIRE_VIEW))
				s->Remove(i);
			else
				s->Next(i);
		}
		GraphicComps::Interpret(cmd);
	}
	else
		GraphicComps::Interpret(cmd);
}

Component* DctGraphicComps::Copy() { return GraphicComps::Copy(); }

void DctGraphicComps::Read(istream &in)
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DctCatalog* cat = (DctCatalog*)du->GetCatalog();
	char prefix[50];
	float file_ver = cat->FileVersion();
	float this_ver = cat->GetVersion();

	GraphicComps::Read(in);

	if(file_ver >= this_ver)
	{
		in >> prefix;
		if( strcmp(prefix,"---no-prefix---") ) du->SetPrefix(prefix);
	}
}

void DctGraphicComps::Write(ostream &out)
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	const char* prefix = du->GetPrefix();

	GraphicComps::Write(out);
	if(prefix && *prefix)
		out << du->GetPrefix() << " ";
	else
		out << "---no-prefix--- ";
}

ClassId DctGraphicViews::GetClassId () { return DCTGRAPHIC_VIEWS; }
boolean DctGraphicViews::IsA (ClassId id)
{ return DCTGRAPHIC_VIEWS == id || GraphicViews::IsA(id); }

/*****************************************************************************/

Dct::Dct(ToolPalette* tp, GraphicComp* comp)
{
    SetClassName("Dct");
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* epics_db = du->GetDatabase();
	DctCatalog* cat = (DctCatalog*)du->GetCatalog();

    _comp = (comp == nil) ? (new DctGraphicComps) : comp;
	_toolpal = tp;
	_prev_fg=nil;
	_prev_bg=nil;

	_keymap = new KeyMap;
	_keymap->Register(_toolpal->GetKeyMap());

	_name = new CompNameVar(_comp); // name of the container catalog
	_modifStatus = new ModifStatusVar(_comp); // modified
	_dbname = new NameVar(du->GetName()); // the database name
	_fontvar = new FontVar(unidraw->GetCatalog()->ReadFont("font",12));

	_color = new ColorVar(
		cat->ReadColor("fgcolor",1),cat->ReadColor("bgcolor",10)
	);

    InitViewer();
    Insert(Interior());
    GetKeyMap()->Execute(CODE_SELECT);

	container=epics_db;
}

Dct::~Dct()
{
	delete _keymap;
	if (_prev_fg != nil) _prev_fg->unref();
	if (_prev_bg != nil) _prev_bg->unref();
}

void Dct::Open()
{
	Editor::Open();
	ManagedWindow* mw = GetWindow();

	if(mw)
	{
		DctHandler* scumbag = new DctHandler();
		mw->wm_delete(scumbag);
	}
}

Component* Dct::GetComponent () { return _comp; }
Viewer* Dct::GetViewer (int id) { return (id == 0) ? _viewer : nil; }
KeyMap* Dct::GetKeyMap () { return _keymap; }
Selection* Dct::GetSelection () { return _toolpal->GetSelection(); }
Tool* Dct::GetCurTool () { return _toolpal->GetCurTool(); }

StateVar* Dct::GetState (const char* name)
{
    if (strcmp(name, "CompNameVar") == 0) return _name;
    else if (strcmp(name, "NameVar") == 0) return _dbname;
    else if (strcmp(name, "ModifStatusVar") == 0) return _modifStatus;
    else if (strcmp(name, "ColorVar") == 0) return _color;
    else if (strcmp(name, "FontVar") == 0) return _fontvar;
    else return _toolpal->GetState(name);
}

void Dct::SetComponent(Component* comp)
{
	Catalog* cat = unidraw->GetCatalog();

    if (comp == nil || comp->IsA(GRAPHIC_COMP))
	{
        _comp = (GraphicComp*) comp;

	    if(_prev_fg != nil && _prev_bg != nil)
		{
        	PSColor* tmp1 = _color->GetFgColor();
        	PSColor* tmp2 = _color->GetBgColor();
        	tmp1->ref();
        	tmp2->ref();
        	_color->SetColors(_prev_fg, _prev_bg);
        	_prev_fg->unref();
        	_prev_bg->unref();
        	_prev_fg = tmp1;
        	_prev_bg = tmp2;
    	}
		else
		{
        	Catalog* catalog = unidraw->GetCatalog();
        	_prev_fg = _color->GetFgColor();
        	_prev_bg = _color->GetBgColor();
        	_prev_fg->ref();
        	_prev_bg->ref();
        	_color->SetColors(
				cat->ReadColor("fgcolor",1),cat->ReadColor("bgcolor",10)
        	);
    	}
    }
}

void Dct::SetViewer (Viewer* v, int i) {
    if (i == 0) {
        _viewer = v;
    }
}

void Dct::SetKeyMap (KeyMap* k) { _keymap = k; }
void Dct::SetSelection (Selection* s) { _toolpal->SetSelection(s); }

Interactor* Dct::Interior()
{
	int gap = round(.1*cm);
	HBox* hb = new HBox( new HGlue(gap, 0, 0) );

	_tray = new Tray(
		new VBox(
			new HBorder,
			new HBox( Commands(), new HGlue),
			new HBorder,
			hb,
			new HBorder,
			_viewer
		)
	);

	HBox* vbg = new HBox(
		new ModifStatusVarView(_modifStatus),
		new CompNameVarView(_name, Left)
	);

	hb->Insert(vbg);
	hb->Insert(new HGlue);
/*
	hb->Insert(	new HGlue(gap, 0, 0) );
	hb->Insert(	new ModifStatusVarView(_modifStatus) );
	hb->Insert(	new CompNameVarView(_name, Left) );
	hb->Insert(	new CompNameVarView(_name, Left) );
	hb->Insert(	new NameVarView(_dbname, Right) );
*/
	_tray->Align(BottomRight, new Frame(new Panner(_viewer)));
	_tray->Propagate(false);

	return _tray;
}

Interactor* Dct::Commands () {
	HBox* comm_hbox = new HBox();
    MenuBar* commands = new MenuBar;
	HGlue* hglue = new HGlue("glue",1,1,1000000);
	const int space = round(.5*cm);

    commands->Include(FileMenu());
    commands->Include(EditMenu());
	comm_hbox->Insert(commands);

    return comm_hbox;
}


void Dct::InitViewer () {
#ifdef GDCT_STYLE
#else
#endif
    float w;
    float h;
    float pw;
    float ph;

	if(use_gdct_style_stuff)
	{
	    w =  8.5 * 3 * inches; // 8.5 *3* inches;
	    h =  11 * 3 * inches; // 11 *3* inches;
	    pw =  8.5 * inches; // 8.5 * inches;
	    ph =  11 * inches; // 11 * inches;
	}
	else
	{
	    w =  8.5 * inches; 
	    h =  11 * inches; 
	    pw =  8.5 * inches; 
	    ph =  11 * inches; 
	}
	page_width=w;
	page_height=h;
#if DEBUG_GEOMETRY
	printf("Dct::InitViewer: w=%g h=%g pw=%g ph=%g\n",
	  w,h,pw,ph);
	printf("  inches=%g points=%g\n",inches,points);
#endif

    GraphicView* view = (GraphicView*) _comp->Create(COMPONENT_VIEW);
    _comp->Attach(view);
    view->Update();

#ifdef GDCT_STYLE
#else
#endif
	if(use_gdct_style_stuff)
	    _viewer=new Viewer(this,view,new UPage(w,h),nil,round(pw),round(ph/1.5));
	else
	{
    	_viewer=new Viewer(this,view,new UPage(w,h),nil,round(pw/2),round(ph/2));
		_viewer->SetMagnification(3);
	}
}

void Dct::Include (Command* cmd, PulldownMenu* pdm) {
    ControlInfo* ctrlInfo = cmd->GetControlInfo();
    UControl* ctrl = new CommandControl(ctrlInfo);
    _keymap->Register(ctrl);
    pdm->Include(ctrl);
    cmd->SetEditor(this);
}

PulldownMenu* Dct::FileMenu () {
    PulldownMenu* pdm = new PulldownMenu("File");

    Include(
        new DctNewCompCmd(
            new ControlInfo("New", KLBL_NEWCOMP, CODE_NEWCOMP),
            new DctGraphicComps
        ),
        pdm
    );
    Include(
        new DctViewCompCmd(
            new ControlInfo("Open...", KLBL_VIEWCOMP, CODE_VIEWCOMP)
        ), pdm
    );
    Include(
        new DctSaveCompCmd(new ControlInfo("Save",KLBL_SAVECOMP,CODE_SAVECOMP)),
        pdm
    );
    Include(
        new DctSaveCompAsCmd(
            new ControlInfo("Save As...", KLBL_SAVECOMPAS, CODE_SAVECOMPAS)
        ), pdm
    );
    Include(
        new DctSaveCompAsVdctCmd(
#if 0
            new ControlInfo("Save As VDCT...", KLBL_SAVECOMPASVDCT, CODE_SAVECOMPASVDCT)
#else
	  // KE: These define the mneumonic on the menu.  Other menu
	  // items use KLBL_* from Unidraw kybd.c, not the ones
	  // defined in dct_classes.h.  However, there is no VCDT one
	  // in kybd.h so we have to use the dct_classes.h one.
            new ControlInfo("Save As VDCT...", KLBL_DCTSAVEASVDCTCMD, CODE_DCTSAVEASVDCTCMD)
#endif
        ), pdm
    );
    Include(
        new PrefixCmd(
            new ControlInfo("Prefix", KLBL_PREFIX, CODE_PREFIX)
        ), pdm
    );
#ifdef USE_APPL_NAME_STUFF
    Include(
        new ApplNameCmd(
            new ControlInfo("Name Application", KLBL_APPLNAME, CODE_APPLNAME)
        ), pdm
    );
#endif
#if 0
    Include(
        new ImportRecCmd(
            new ControlInfo("Import Records", KLBL_IMPORTREC, CODE_IMPORTREC)
        ), pdm
    );
#endif
    Include(
        new NowhereEditCmd(
            new ControlInfo("Hidden Record Edit", KLBL_NOWHEREEDIT, CODE_NOWHEREEDIT)
        ), pdm
    );
    Include(
        new QuitCmd(
            new ControlInfo("Quit", KLBL_QUIT, CODE_QUIT)
        ), pdm
    );

    return pdm;
}

PulldownMenu* Dct::EditMenu () {
    PulldownMenu* pdm = new PulldownMenu("Edit");
/*
    Include(
        new UndoCmd(
            new ControlInfo("Undo", KLBL_UNDO, CODE_UNDO)
        ), pdm
    );
    Include(
        new RedoCmd(
            new ControlInfo("Redo", KLBL_REDO, CODE_REDO)
        ), pdm
    );
*/
    Include(
        new DeleteCmd(
            new ControlInfo("Delete", KLBL_DEL, CODE_DEL)
        ), pdm
    );
    Include(
        new DupCmd(
            new ControlInfo("Duplicate", KLBL_DUP, CODE_DUP)
        ), pdm
    );
	/*
    Include(
        new CutCmd(
            new ControlInfo("Cut", KLBL_CUT, CODE_CUT)
        ), pdm
    );
    Include(
        new CopyCmd(
            new ControlInfo("Copy", KLBL_COPY, CODE_COPY)
        ), pdm
    );
    Include(
        new PasteCmd(
            new ControlInfo("Paste", KLBL_PASTE, CODE_PASTE)
        ), pdm
    );
	*/
    Include(
        new BackCmd(
            new ControlInfo("Back", KLBL_BACK, CODE_BACK)
        ), pdm
    );
    Include(
        new FrontCmd(
            new ControlInfo("Front", KLBL_FRONT, CODE_FRONT)
        ), pdm
    );
    Include(
        new GroupCmd(
            new ControlInfo("Group", KLBL_GROUP, CODE_GROUP)
        ), pdm
    );
    Include(
        new UngroupCmd(
            new ControlInfo("Ungroup", KLBL_UNGROUP, CODE_UNGROUP)
        ), pdm
    );

    return pdm;
}

// ---------------------------tool palatte follows--------------------

ToolPalette::ToolPalette()
{
	SetClassName("ToolPalette");
	ManagedWindow* window = new ApplicationWindow(this);
	SetWindow(window);

	_curCtrl = new ControlState;
	_selection = new Selection;
	_keymap = new KeyMap;

	Insert(new Frame(Interior()));
}

ToolPalette::~ToolPalette()
{
	delete _keymap;
	delete _selection;
}

KeyMap* ToolPalette::GetKeyMap() { return _keymap; }
Selection* ToolPalette::GetSelection () { return _selection; }
 
Tool* ToolPalette::GetCurTool()
{
    UControl* c = (UControl*)_curCtrl->Selection();
    return (Tool*) c->GetControlInfo()->GetOwner();
}
 
void ToolPalette::SetKeyMap(KeyMap* k) { _keymap = k; }
void ToolPalette::SetSelection (Selection* s) { _selection = s; }
Interactor* ToolPalette::Interior() { return Tools(); }

static const int unit = 15;
static Coord xWire[] = { 0, unit/2, unit/2, unit };
static Coord yWire[] = { 0, 0, unit, unit };
static const int nWire = 4;

Interactor* ToolPalette::Tools () {
    VBox* tools = new VBox;
	tools->Propagate(false);
    const Coord tunit = round(0.5*cm);

    Include(
        new SelectTool(new ControlInfo("Select", KLBL_SELECT, CODE_SELECT)),
        tools
    );
    Include(
        new MoveTool(new ControlInfo("Move", KLBL_MOVE, CODE_MOVE)),
        tools
    );
    Include(
        new ScaleTool(new ControlInfo("Scale", KLBL_SCALE, CODE_SCALE)),
        tools
    );
    Include(
        new RotateTool(new ControlInfo("Rotate", KLBL_ROTATE, CODE_ROTATE)),
        tools
    );
    Include(
        new ExamineTool(new ControlInfo("Examine", KLBL_EXAMINE, CODE_EXAMINE)),
		tools
    );

	// linking
    WireComp* wireComp = new WireComp(xWire, yWire, nWire, stdgraphic);
    Include(
        new GraphicCompTool(
            new ControlInfo(wireComp, KLBL_LINK, CODE_LINK), wireComp
        ), tools
    );

	// rectangles
    SF_Rect* rect = new SF_Rect(0, 0, tunit*2, tunit, stdgraphic);
    rect->SetPattern(psnonepat);
    RectComp* rectComp = new RectComp(rect);
    Include(
        new GraphicCompTool(
            new ControlInfo(rectComp, KLBL_RECT, CODE_RECT), rectComp
        ), tools
    );

	// ellipses
    SF_Ellipse* ellipse = new SF_Ellipse(0, 0, tunit, tunit/2, stdgraphic);
    ellipse->SetPattern(psnonepat);
    EllipseComp* ellipseComp = new EllipseComp(ellipse);
    Include(
        new GraphicCompTool(
            new ControlInfo(ellipseComp, KLBL_ELLIPSE, CODE_ELLIPSE),
            ellipseComp
        ), tools
    );

	// text
    TextGraphic* text = new TextGraphic("Text", stdgraphic);
    TextComp* textComp = new TextComp(text);
    Include(
        new GraphicCompTool(
            new ControlInfo(textComp, KLBL_TEXT, CODE_TEXT), textComp
        ), tools
    );

	// edit text
    TextGraphic* text1 = new TextGraphic("Edit Text", stdgraphic);
    TextComp* textComp1 = new TextComp(text1);
    Include(
        new ReshapeTool(
            new ControlInfo(textComp1, KLBL_EDITTEXT, CODE_EDITTEXT)
        ), tools
    );

	// database records
	RecComp* recComp = new RecComp();
	ControlInfo* ci = new ControlInfo(recComp, KLBL_REC, CODE_REC);
	GraphicCompTool* gct = new GraphicCompTool( ci, recComp);
	Include(gct,tools);

	// external link
	ExtLinkComp* extlinkComp = new ExtLinkComp();
	ControlInfo* ext_ci = 
		new ControlInfo(extlinkComp, KLBL_EXTLINK, CODE_EXTLINK);
	GraphicCompTool* ext_gct = new GraphicCompTool( ext_ci, extlinkComp);
	Include(ext_gct,tools);

    return tools;
}

void ToolPalette::Include (Tool* tool, Box* box)
{
    ControlInfo* ctrlInfo = tool->GetControlInfo();
    UControl* ctrl = new VPanelControl(ctrlInfo, _curCtrl);
    _keymap->Register(ctrl);
    box->Insert(ctrl);
}

void ToolPalette::Open()
{
	Editor::Open();
	ManagedWindow* mw = GetWindow();

	if(mw)
	{
	  // Add WM_DELETE handler
		DctHandler* scumbag = new DctHandler();
		mw->wm_delete(scumbag);

#ifdef TOOL_PALETTE_TOPRIGHT
	      // Move palette to upper right of screen
		Canvas *c=mw->canvas();
		Display *d=mw->display();

#if DEBUG_POSITION
		printf("ToolPalette: left=%d bottom=%d width=%d height=%d\n",
		  mw->left(),mw->bottom(),mw->width(),mw->height());
		printf("  IsMapped: %s\n",mw->is_mapped()?"True":"False");
		printf("  Canvas: %g x %g [%d x %d]\n",
		  c->width(),c->height(),c->pwidth(),c->pheight());
		printf("  Display: %g x %g [%d x %d]\n",
		  d->width(),d->height(),d->pwidth(),d->pheight());
#endif

	      // move uses Coord.  Want x=displayWidth-canvasWidth-dx,
	      // y=dy=displayHeight-canvasHeight-bottom ->
	      // bottom=displayHeight-canvasHeight-dy, probably
	      // ignoring borders.  It would be better to use
	      // resources, but couldn't figure out how.
		int dx=15,dy=10;
		mw->move(d->width()-c->width()-d->to_coord(dx),
		  d->height()-c->height()-d->to_coord(dy));

#if DEBUG_POSITION
		printf("ToolPalette: left=%d bottom=%d width=%d height=%d\n",
		  mw->left(),mw->bottom(),mw->width(),mw->height());
		printf("  IsMapped: %s\n",mw->is_mapped()?"True":"False");
#endif
#endif
	}
}

