/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#ifndef dct_h
#define dct_h

#if 0
extern "C" {
#include "dbStaticLib.h"
}
#endif
#include <string.h>

#ifndef DEFAULT_DCTSDR
#define DEFAULT_DCTSDR "./default.dbd"
#endif
#ifndef DCTSDRFILE
#define DCTSDRFILE "dbd/default.dbd"
#endif

#define DCT_VERSION 1.2

#include <Unidraw/unidraw.h>
#include <Unidraw/catalog.h>
#include <Unidraw/editor.h>
#include <Unidraw/Components/grcomp.h>
#include <Unidraw/Components/grview.h>
#include <InterViews/handler.h>

typedef struct dbBase DBBASE;

class ControlState;
class ToolPalette;
class Catalog;
class OptionDesc;
class PropertyData;
class DctCatalog;
class ModifStatusVar;
class CompNameVar;
class NameVar;
class FontVar;
class ColorVar;
class Viewer;
class PullDownMenu;
class Event;

class Uninode
{
public:
	Uninode()			{ string=NULL; next=NULL; }
	Uninode(char* name)	{ string=name; next=NULL; }
	Uninode(char* name,Uninode* x)	{ string=strdup(name); next=x; }
	~Uninode()			{ if(string) delete [] string; }

	char* string;
	struct Uninode* next;
};

//----------------------delete handler----------------------
class DctHandler : public Handler
{
public:
    DctHandler();
    virtual ~DctHandler();
 
    virtual boolean event(Event&);
};

//-------------------------unidraw-------------------------------
class DctUnidraw: public Unidraw
{
public:
	DctUnidraw(DctCatalog*,int& argc, char** argv,
				OptionDesc* = nil, PropertyData* = nil);
	virtual ~DctUnidraw();

	DBBASE* GetDatabase();
	DBBASE* GetCatDatabase();
	void SetDatabase(DBBASE*);
	void SetName(const char* name);
	char* GetName();
	void MakeCatDatabase();
	void UseCatDatabase();
	const char* GetPrefix();
	void SetPrefix(char*);
	char* GetLogName();

	const Uninode *GetDBDList(void) const;
	void ClearList(Uninode*& list);
	void SetIncPath(char* path);
	char* GetIncPath();
	char* ResolveName(char* file);
	int AddDBD(char* name);
	int SetDefaultDBDs(char** list,int total);
	int ReadDefaultDBDs(DBBASE*& db);

private:
	DBBASE *epics_db,*cat_db;
	char *inc_path,*inc_path_copy,*env_path;
	char *db_name,*log_name;
	Uninode* dbd_list;
	Uninode *inc_list,*inc_list_tail;
	int tot_table,startup;
	char global_prefix[50];
};

inline const Uninode *DctUnidraw::GetDBDList(void) const {return dbd_list; }
inline DBBASE* DctUnidraw::GetDatabase() { return epics_db; }
inline DBBASE* DctUnidraw::GetCatDatabase() { return cat_db; }
inline void DctUnidraw::SetDatabase(DBBASE* db) { epics_db=db; }
inline const char* DctUnidraw::GetPrefix() {return (const char*)global_prefix;}
inline char* DctUnidraw::GetLogName() { return log_name; }
inline char* DctUnidraw::GetIncPath() { return inc_path; }

inline char* DctUnidraw::GetName() { return db_name; }
inline void DctUnidraw::SetName(const char* name)
{
	if(db_name) delete [] db_name;
	db_name=new char[strlen(name)+1];
	strcpy(db_name,name);
}

//---------------------------catalog-----------------------------
class DctCatalog: public Catalog
{
public:
	DctCatalog(const char*,Creator*,float version = UV_LATEST);
	virtual ~DctCatalog();

    virtual boolean Save(EditorInfo* e, const char* c)
		{ return Catalog::Save(e,c); }
    virtual boolean Save(Command* e, const char* c)
		{ return Catalog::Save(e,c); }
    virtual boolean Save(Tool* e, const char* c)
		{ return Catalog::Save(e,c); }
    // virtual boolean Save(Component* e, const char* c)
	// 	{ return Catalog::Save(e,c); }
 
    virtual boolean Retrieve(const char* c, EditorInfo*& e)
		{ return Catalog::Retrieve(c,e); }
    virtual boolean Retrieve(const char* c, Component*& e)
		{ return Catalog::Retrieve(c,e); }
    virtual boolean Retrieve(const char* c, Command*& e)
		{ return Catalog::Retrieve(c,e); }
    virtual boolean Retrieve(const char* c, Tool*& e)
		{ return Catalog::Retrieve(c,e); }

	virtual boolean Retrieve(const char*,const char*, Component*&);
	virtual boolean Save(Component*, const char*,const char*);
    virtual boolean Save(Component*, const char*);
private:
};

//---------------------the editor component--------------------
class DctGraphicComps : public GraphicComps
{
public:
	virtual ~DctGraphicComps();

	virtual void Interpret(Command*);
	virtual Component* Copy();

	virtual void Read(istream &in);
	virtual void Write(ostream &out);

	virtual ClassId GetClassId();
	virtual boolean IsA(ClassId);
};

class DctGraphicViews : public GraphicViews
{
public:
	virtual ClassId GetClassId();
	virtual boolean IsA(ClassId);
};

//--------------main editor-------------------
class Dct : public Editor
{
public:
    Dct(ToolPalette*, GraphicComp* = nil);
    virtual ~Dct();

    virtual Component* GetComponent();
    virtual Viewer* GetViewer(int = 0);
    virtual KeyMap* GetKeyMap();
    virtual Tool* GetCurTool();
    virtual Selection* GetSelection();
	virtual StateVar* GetState(const char*);
	virtual void Open();

    virtual void SetComponent(Component*);
    virtual void SetViewer(Viewer*, int = 0);
    virtual void SetKeyMap(KeyMap*);
    virtual void SetSelection(Selection*);

    ToolPalette* GetToolPalette() { return _toolpal; }
    DBBASE* GetContainer() { return container; }
    void SetContainer(DBBASE* c) { container=c; }
    void GetPageSize(float &width, float &height);

private:
    Interactor* Interior();
    void InitViewer();

    Interactor* Commands();
    Interactor* Tools();

    void Include(class Command*, class PulldownMenu*);
    void Include(Tool*, class Box*);

    PulldownMenu* FileMenu();
    PulldownMenu* EditMenu();

private:
    GraphicComp* _comp;
    KeyMap* _keymap;
    ToolPalette* _toolpal;
	class Tray* _tray;
    Viewer* _viewer;

	CompNameVar* _name;
	ModifStatusVar* _modifStatus;
	NameVar* _dbname;
	ColorVar* _color;
	FontVar* _fontvar;

	DBBASE* container;

	PSColor* _prev_fg;
	PSColor* _prev_bg;
};

class ToolPalette : public Editor
{
public:
    ToolPalette();
    virtual ~ToolPalette();
 
    virtual KeyMap* GetKeyMap();
    virtual Tool* GetCurTool();
    virtual Selection* GetSelection();
	virtual void Open();
 
    virtual void SetKeyMap(KeyMap*);
    virtual void SetSelection(Selection*);
	ControlState* GetControlState();
 
private:
    Interactor* Interior();
    Interactor* Tools();
    void Include(Tool*,Box*);

    KeyMap* _keymap;
    ControlState* _curCtrl;
    Selection* _selection;
    class VBox* _tools;
};
 
inline ControlState* ToolPalette::GetControlState () { return _curCtrl; }

#endif
