/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#define VDCT_TEXT_WIDTH  260
#define VDCT_TEXT_HEIGHT 20

#include <InterViews/transformer.h>
#include <Unidraw/unidraw.h>
#include <Unidraw/viewer.h>
#include <Unidraw/manips.h>
#include <Unidraw/Tools/tool.h>
#include <Unidraw/Commands/command.h>
#include <Unidraw/Components/pin.h>

#include "dctcreat.h"
#include "dctclasses.h"
#include "dctcomps.h"
#include "dctvars.h"
#include "dctrec.h"
#include "dctext.h"
#include "dcted.h"
#include "dctdial.h"

#include <stream.h>
#include <string.h>
#include <stdlib.h>
#include <memory.h>
#include <stdio.h>

extern "C" {
#include "dbStaticLib.h"
};

// ------------------------external link component stuff--------------------

int ExtLinkComp::vdctid0=0;

ExtLinkComp::ExtLinkComp()
{
	vdctid=++vdctid0;
	Coord x=0, y=0;
	Picture *main_pict = new Picture;

	// the job here is to make an empty extlinkcomp for the menu palette
	// this is the prototype thing

	name_graphic=nil;
	strcpy(link_name,"External Link");
	link_type='o';
	a_new_link_name=nil;
	a_new_link_type=nil;
	name_pincomp_left=nil;

	name_graphic = new DataGraphic(link_name);

	// the main pin for the link
	PinGraphic* pin_g = new PinGraphic(x,y,stdgraphic);
	name_pincomp_right = new NodeComp(pin_g);
	name_pincomp_right->SetNodeType(ExtNode);
	name_pincomp_right->SetMobility(Fixed);
	name_pincomp_right->SetTransMethod(Out);
	Graphic* pin_graphic = name_pincomp_right->GetPin();
	Graphic* pg = name_pincomp_right->GetGraphic();
	name_graphic->Align(CenterRight,pg,CenterLeft);

	main_pict->Append(pin_graphic, name_graphic);
	SetGraphic(main_pict);
}

ExtLinkComp::ExtLinkComp(Coord x, Coord y, char* l_name, char l_type)
{
	name_graphic=nil;
	strcpy(link_name,l_name);
	link_type=l_type;
	a_new_link_name=nil;
	a_new_link_type=nil;
	name_pincomp_right=nil;
	name_pincomp_left=nil;

	char* plink_name = strdup(l_name);
	char* rec = strtok(plink_name,".");
	char* fld = strtok(NULL,".");
	
	vdctid=++vdctid0;

	if(fld==nil) fld="VAL ";

	Picture *main_pict = new Picture;

	// create the name of the link
	name_graphic = new DataGraphic(link_name);

	// the main pin for the record - a lot of code or what?
	if(link_type=='o' || link_type=='b')
	{
		PinGraphic* pin_g_right = new PinGraphic(x,y,stdgraphic);
		name_pincomp_right = new NodeComp(pin_g_right);
		name_pincomp_right->SetNodeType(ExtNode);
		NodeVar* nv = name_pincomp_right->GetNodeVar();
		nv->SetRecField(fld,rec);
		name_pincomp_right->SetSide('l');
		name_pincomp_right->SetMobility(Fixed);
		name_pincomp_right->SetTransMethod(Out);
		Graphic* pin_graphic_right = name_pincomp_right->GetPin();
		Graphic* pg = name_pincomp_right->GetGraphic();
		name_graphic->Align(CenterRight,pg,CenterLeft);
		main_pict->Append(pin_graphic_right);
	}
	if(link_type=='i' || link_type=='b')
	{
		PinGraphic* pin_g_left = new PinGraphic(x,y,stdgraphic);
		name_pincomp_left = new NodeComp(pin_g_left);
		name_pincomp_left->SetNodeType(ExtNode);
		NodeVar* nv = name_pincomp_left->GetNodeVar();
		nv->SetRecField(fld,rec);
		name_pincomp_left->SetSide('l');
		name_pincomp_left->SetMobility(Fixed);
		name_pincomp_left->SetTransMethod(Out);
		Graphic* pin_graphic_left = name_pincomp_left->GetPin();
		Graphic* pg = name_pincomp_left->GetGraphic();
		name_graphic->Align(CenterLeft,pg,CenterRight);
		main_pict->Append(pin_graphic_left);
	}

	// set up the entire thing
	main_pict->Append(name_graphic);
	SetGraphic(main_pict);
}

ExtLinkComp::~ExtLinkComp()
{
	Graphic* parent = GetGraphic();

	if(name_pincomp_right)
	{
		parent->Remove(name_pincomp_right->GetGraphic());
		delete name_pincomp_right;
	}

	if(name_pincomp_left)
	{
		parent->Remove(name_pincomp_left->GetGraphic());
		delete name_pincomp_left;
	}

	if(a_new_link_name) delete a_new_link_name;
}

void ExtLinkComp::First(Iterator& i)
{
	if(name_pincomp_left) i.SetValue(name_pincomp_left);
	else i.SetValue(name_pincomp_right);
}

void ExtLinkComp::Last(Iterator& i)
{
	if(name_pincomp_right) i.SetValue(name_pincomp_right);
	else i.SetValue(name_pincomp_left);
}

void ExtLinkComp::Next(Iterator& i)
{
    NodeComp* v = (NodeComp*)i.GetValue();

	if(v==name_pincomp_left) i.SetValue(name_pincomp_right);
	else if(v==nil) i.SetValue(name_pincomp_left);
	else i.SetValue(nil);
}

void ExtLinkComp::Prev(Iterator& i)
{
    NodeComp* v = (NodeComp*)i.GetValue();

	if(v==name_pincomp_right) i.SetValue(name_pincomp_left);
	else if(v==nil) i.SetValue(name_pincomp_right);
	else i.SetValue(nil);
}

boolean ExtLinkComp::Done(Iterator i) { return i.GetValue()==nil; }

GraphicComp* ExtLinkComp::GetComp(Iterator i)
{ return (GraphicComp*)i.GetValue(); }

Graphic *ExtLinkComp::GetGraphic()
{
	Graphic *graphic= GraphicComp::GetGraphic();
	return graphic;
}

void ExtLinkComp::Update()
{
	Notify();
	ConfigChanged(0);
}


int ExtLinkComp::RenameLink(const char *new_name) {

    SetNewRecFldName(new_name);
    SetRecFldName(new_name);
    
    Graphic *ng = new DataGraphic(link_name);
    Picture *main_pict = (Picture *)GraphicComp::GetGraphic();
    main_pict->Remove(name_graphic);
    delete name_graphic;
    name_graphic = ng;
    main_pict->Append(name_graphic);
    
    char *fname;
    
    if(name_pincomp_right)
    {
	NodeVar* node_var = name_pincomp_right->GetNodeVar();
	
	if((fname=node_var->GetField())==NULL)
	    fname="VAL";
	else if(fname[0]=='\0')
	    fname="VAL";

	node_var->SetRecField(fname,link_name);
	name_pincomp_right->Transmit();
    }

    if(name_pincomp_left)
    {
	NodeVar* node_var = name_pincomp_left->GetNodeVar();
	
	if((fname=node_var->GetField())==NULL)
	    fname="VAL";
	else if(fname[0]=='\0')
	    fname="VAL";

	node_var->SetRecField(fname,link_name);
	name_pincomp_left->Transmit();
    }

    Update();
    return 0;
}

void ExtLinkComp::Read (istream& in)
{
	GraphicComp::Read(in);
	Catalog* catalog = unidraw->GetCatalog();
	DctUnidraw* du = (DctUnidraw*)unidraw;

	in >> link_name >> link_type;
	// cerr << "just extlinkcomp::read() link_name=(" << link_name << ")\n";

	a_new_link_name = nil;
	a_new_link_type = nil;
	name_pincomp_right = nil;
	name_pincomp_left = nil;

	Picture *main_pict = new Picture;

	// create the name of the record
	name_graphic = new DataGraphic(link_name);

	if(link_type=='o' || link_type=='b')
	{
		name_pincomp_right = (NodeComp*)catalog->ReadComponent(in);
		Graphic* pin_graphic_right = name_pincomp_right->GetPin();
		Graphic* pg_right = name_pincomp_right->GetGraphic();
		name_graphic->Align(CenterRight,pg_right,CenterLeft);
		main_pict->Append(pin_graphic_right);
	}
	if(link_type=='i' || link_type=='b')
	{
		name_pincomp_left = (NodeComp*)catalog->ReadComponent(in);
		Graphic* pin_graphic_left = name_pincomp_left->GetPin();
		Graphic* pg_left = name_pincomp_left->GetGraphic();
		name_graphic->Align(CenterLeft,pg_left,CenterRight);
		main_pict->Append(pin_graphic_left);
	}

	Transformer* t = catalog->ReadTransformer(in);
	main_pict->SetTransformer(t);
	main_pict->Append(name_graphic);
	SetGraphic(main_pict);
}

void ExtLinkComp::Write (ostream& out)
{
	GraphicComp::Write(out);
	Catalog* catalog = unidraw->GetCatalog();

	out << link_name << " " << link_type << " ";

	if(link_type=='o' || link_type=='b')
		catalog->WriteComponent(name_pincomp_right, out);
	if(link_type=='i' || link_type=='b')
		catalog->WriteComponent(name_pincomp_left, out);

	catalog->WriteTransformer(GetGraphic()->GetTransformer(), out);
}

void ExtLinkComp::WriteVDCT(FILE *fp)
{
	float left=0,bottom=0,right=0,top=0;
	int lvdct=0,bvdct=0,rvdct=0,tvdct=0;

	// No VDCT equivalent, make a text box
	GetGraphic()->GetBounds(left,bottom,right,top);
	PageToVDCT(left,top,lvdct,tvdct);
    fprintf(fp,"\n\n## Ext Link has no VDCT equivalent.  Using TextBox...\n");
	fprintf(fp,"#! TextBox(TBExtLink%d,",vdctid);
	PageToVDCT(left,bottom,lvdct,bvdct);
	PageToVDCT(right,top,rvdct,tvdct);
	bvdct=tvdct-VDCT_TEXT_HEIGHT;
	rvdct=lvdct+VDCT_TEXT_WIDTH;;
	fprintf(fp,"%d,%d,%d,%d,",lvdct,bvdct,rvdct,tvdct);
	fprintf(fp,"0,\"Helvetica\",14,0,16777215,");
	fprintf(fp,"\"ExtLink %s\"\n",link_name);
	fprintf(fp,"\n");    // Extra CR to make it stand out
}

boolean ExtLinkComp::HasLinks()
{
	if(name_pincomp_right!=nil && name_pincomp_right->HasConnections())
		return true;
	if(name_pincomp_left!=nil && name_pincomp_left->HasConnections())
		return true;

	return false;
}

void ExtLinkComp::Interpret(Command* cmd) { GraphicComp::Interpret(cmd); }
void ExtLinkComp::Uninterpret(Command* cmd) { GraphicComp::Uninterpret(cmd); }

ClassId ExtLinkComp::GetClassId () { return EXTLINK_COMP; }
boolean ExtLinkComp::IsA(ClassId id)
	{return EXTLINK_COMP==id||GraphicComp::IsA(id);}

void ExtLinkComp::SetNewRecFldName(const char* string)
{
	if(a_new_link_name) delete a_new_link_name;
	a_new_link_name = strdup(string);
	char* t = a_new_link_name;
	while( t=strchr(t,' ') ) *t++='_';
}

void ExtLinkComp::SetNewRecFldType(const char c) { a_new_link_type = c; }

void ExtLinkComp::SetRecFldName(const char* string)
{
	strcpy(link_name, string);
	char* t = link_name;
	while( t=strchr(t,' ') ) *t++='_';
}

Component* ExtLinkComp::Copy()
{
	Coord x,y;

	if(link_type=='i' || link_type=='b')
	{
		PinGraphic* pin_left = name_pincomp_left->GetPin();
		pin_left->GetOriginal(x,y);
	}
	if(link_type=='o' || link_type=='b')
	{
		PinGraphic* pin_right = name_pincomp_right->GetPin();
		pin_right->GetOriginal(x,y);
	}

	ExtLinkComp* copy;

	if(a_new_link_name)
		copy = new ExtLinkComp(x,y,a_new_link_name,a_new_link_type);
	else
		copy = new ExtLinkComp(x,y,link_name,link_type);

	Transformer* t = GetGraphic()->GetTransformer();
	copy->GetGraphic()->SetTransformer(new Transformer(t));
	return copy;
}

// ------------------------rec view stuff--------------------
ExtLinkView::ExtLinkView(ExtLinkComp* subj) : GraphicView(subj)
{
	link_pinview_left=nil;
	link_pinview_right=nil;
}
 
ExtLinkView::~ExtLinkView()
{
	Graphic* gr;

	if(link_pinview_right)
	{
		gr=GetGraphic();
		gr->Remove(link_pinview_right->GetGraphic());
		delete link_pinview_right;
	}

	if(link_pinview_left)
	{
		gr=GetGraphic();
		gr->Remove(link_pinview_left->GetGraphic());
		delete link_pinview_left;
	}
}

// make sure that these iterate through all three lists, data in out

void ExtLinkView::First(Iterator& i)
{ 
	if(link_pinview_left) i.SetValue(link_pinview_left);
	else i.SetValue(link_pinview_right);
}
void ExtLinkView::Last(Iterator& i)
{
	if(link_pinview_right) i.SetValue(link_pinview_right);
	else i.SetValue(link_pinview_left);
}
void ExtLinkView::Next(Iterator& i)
{
    NodeView* v = (NodeView*)i.GetValue();

	if(v==link_pinview_left) i.SetValue(link_pinview_right);
	else if(v==nil) i.SetValue(link_pinview_left);
	else i.SetValue(nil);
}
void ExtLinkView::Prev(Iterator& i)
{
    NodeView* v = (NodeView*)i.GetValue();

	if(v==link_pinview_right) i.SetValue(link_pinview_left);
	else if(v==nil) i.SetValue(link_pinview_right);
	else i.SetValue(nil);
}

boolean ExtLinkView::Done(Iterator i) { return i.GetValue()==nil; }
 
void ExtLinkView::SetView(GraphicView* gv,Iterator& i) { i.SetValue(gv); }
GraphicView* ExtLinkView::GetView(Iterator i)
	{ return (GraphicView*)i.GetValue(); }

ExtLinkComp* ExtLinkView::GetExtLinkComp()
	{ return (ExtLinkComp*)GetSubject(); }

ClassId ExtLinkView::GetClassId () { return EXTLINK_VIEW; }
 
boolean ExtLinkView::IsA (ClassId id)
{ return EXTLINK_VIEW == id || GraphicView::IsA(id); }

void ExtLinkView::Interpret(Command* cmd) { GraphicView::Interpret(cmd); }
void ExtLinkView::Uninterpret(Command* cmd) { GraphicView::Uninterpret(cmd); }

void ExtLinkView::Update()
{
	ExtLinkComp* comp = GetExtLinkComp();

	Graphic* graphic = GetGraphic();
	IncurDamage(graphic);

	DataGraphic* ng = (DataGraphic*)comp->GetNameGraphic();
	DataGraphic* ng1 = (DataGraphic*)name_graphic;

	if( (*ng)!=(*ng1) )
	{
		//----------------added from rename-------------------
		if(name_graphic)
		{
			char *name = comp->GetLinkName();
			Graphic* tg = new DataGraphic(name);
			Picture *gp = (Picture *)graphic;
			gp->Remove(name_graphic);
			delete name_graphic;
			name_graphic = tg;

   	    	        if (link_pinview_right) {
	    	    	    Graphic* pg = link_pinview_right->GetGraphic();
	    	    	    tg->Align(CenterRight,pg,CenterLeft);
   	    	    	}
    
    	    	    	if (link_pinview_left) {
	    	    	    Graphic* pg = link_pinview_left->GetGraphic();
	    	    	    tg->Align(CenterLeft,pg,CenterRight);
   	    	    	}
    	
			gp->Append(name_graphic);
 		}
		// ------------------------------------------------------
				
		Dct* dct_ed = (Dct*)GetViewer()->GetEditor();
		ModifStatusVar* nv=(ModifStatusVar*)dct_ed->GetState("ModifStatusVar");
		nv->SetModifStatus(true);
		unidraw->Update();
	}

	*graphic = *comp->GetGraphic();
	IncurDamage(graphic);
	if(link_pinview_right) link_pinview_right->Update();
	if(link_pinview_left) link_pinview_left->Update();
	EraseHandles();
}

Manipulator *ExtLinkView::CreateManipulator(Viewer* v, Event& e, Transformer* t, Tool* tt)
{
	Manipulator* manip;

	manip = GraphicView::CreateManipulator(v, e, t, tt);
	return manip;
}
 
Command *ExtLinkView::InterpretManipulator(Manipulator* m)
{
	DragManip* dm = (DragManip*) m;
	Dct* ed = (Dct*)dm->GetViewer()->GetEditor();
	Tool* tool = dm->GetTool();
	Command* comm = nil;
	ExtLinkComp* link_comp = GetExtLinkComp();
	DctUnidraw* du = (DctUnidraw*)unidraw;

	if (tool->IsA(GRAPHIC_COMP_TOOL))
	{
	    // populate the chooser with all record types
		char* link_name = "";
		if( strcmp(link_comp->GetLinkName(),"External Link") )
			link_name = link_comp->GetLinkName();

		extlink_edit exl("Enter External Link and Type",
			link_name,link_comp->GetLinkType());
	 
		ed->InsertDialog(&exl);
		global_dialog_wn_delete(&exl);

		if(exl.Accept())
		{
			link_comp->SetNewRecFldName(exl.GetLinkString());
			link_comp->SetNewRecFldType(exl.GetLinkType());
			ed->RemoveDialog(&exl);
		}
		else
		{
			ed->RemoveDialog(&exl);
			return nil;
		}
	}

	comm = GraphicView::InterpretManipulator(m);
	return comm;
}

Graphic* ExtLinkView::GetGraphic()
{
	Graphic* graphic = GraphicView::GetGraphic();
	char *l_name;
	char l_type;

	if(graphic==nil)
	{
		ExtLinkComp* link_comp = GetExtLinkComp();
		if(link_comp==nil) { cerr << "nil link_comp"; exit(0); }

		l_type=link_comp->GetLinkType();

		NodeComp* link_pincomp_left = link_comp->GetLeftNodeComp();
		NodeComp* link_pincomp_right = link_comp->GetRightNodeComp();

		if(link_pincomp_right==nil && link_pincomp_left==nil)
			{ cerr << "nil link_pincomp"; exit(0); }

		l_name=link_comp->GetLinkName();
		// cerr << "extlinkview::getgraphic link_name=(" << l_name << ")\n";
		name_graphic = new DataGraphic(l_name);
		graphic = new Picture;

		if(l_type=='o'||l_type=='b')
		{
			link_pinview_right=
				(NodeView*)link_pincomp_right->Create(COMPONENT_VIEW);
			link_pincomp_right->Attach(link_pinview_right);
			Graphic* pin_g_view_right = link_pinview_right->GetGraphic();
			name_graphic->Align(CenterRight,pin_g_view_right,CenterLeft);
			graphic->Append(pin_g_view_right);
		}

		if(l_type=='i'||l_type=='b')
		{
			link_pinview_left=
				(NodeView*)link_pincomp_left->Create(COMPONENT_VIEW);
			link_pincomp_left->Attach(link_pinview_left);
			Graphic* pin_g_view_left = link_pinview_left->GetGraphic();
			name_graphic->Align(CenterLeft,pin_g_view_left,CenterRight);
			graphic->Append(pin_g_view_left);
		}

		graphic->Append(name_graphic);
		SetGraphic(graphic);
	}
	return graphic;
}

/* **************************** Emacs Editing Sequences ***************** */
/* Local Variables: */
/* tab-width: 4 */
/* c-basic-offset: 4 */
/* c-comment-only-line-offset: 0 */
/* c-file-offsets: ((substatement-open . 0) (label . 0)) */
/* End: */
