/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#ifndef __dctextlink_H__
#define __dctextlink_H__

#include <stream.h>
#include <Unidraw/Components/grcomp.h>
#include <Unidraw/Components/grview.h>
#include <Unidraw/Components/psview.h>
#include <Unidraw/Graphic/picture.h>
#include <Unidraw/iterator.h>

class Command;
class NodeView;
class NodeComp;

// ---------------------the external link field component thing--------------
class ExtLinkComp: public GraphicComp
{
public:
	ExtLinkComp();
	ExtLinkComp(Coord x, Coord y, char* link_name, char link_type);
	virtual ~ExtLinkComp();

	virtual void First(Iterator&);
	virtual void Last(Iterator&);
	virtual void Next(Iterator&);
	virtual void Prev(Iterator&);
	virtual boolean Done(Iterator);
	virtual GraphicComp* GetComp(Iterator);

	virtual Graphic* GetGraphic();

	virtual void Interpret(Command*);
	virtual void Uninterpret(Command*);
	virtual void Update();
	virtual Component *Copy();

	virtual void Read(istream&);
	virtual void Write(ostream&);
	void WriteVDCT(FILE *fp);

	virtual ClassId GetClassId();
	virtual boolean IsA(ClassId);

	ExtLinkComp* GetExtLinkComp();

	NodeComp* GetRightNodeComp();
	NodeComp* GetLeftNodeComp();
	char GetLinkType();
	char* GetLinkName();
	Graphic* GetNameGraphic();
	boolean HasLinks();
	int ConfigChanged() { return cfgc; }
	void ConfigChanged(int i) { cfgc=i; }

	void SetNewRecFldName(const char*);
	void SetNewRecFldType(const char);

	void SetRecFldName(const char*);

    	int RenameLink(const char *new_name);

private:
	Graphic* name_graphic;
	NodeComp* name_pincomp_right;
	NodeComp* name_pincomp_left;
	
	int cfgc;
	char link_name[50];
	char link_type;
	char* a_new_link_name;
	char a_new_link_type;

	int vdctid;
	static int vdctid0;
};

inline char ExtLinkComp::GetLinkType() { return link_type; }
inline NodeComp* ExtLinkComp::GetRightNodeComp() { return name_pincomp_right; }
inline NodeComp* ExtLinkComp::GetLeftNodeComp() { return name_pincomp_left; }
inline Graphic* ExtLinkComp::GetNameGraphic() { return name_graphic; }
inline char* ExtLinkComp::GetLinkName() { return link_name; }

// ---------------------the external link view thing--------------------
class ExtLinkView: public GraphicView
{
public:
	ExtLinkView(ExtLinkComp* = nil);
	virtual ~ExtLinkView();

	virtual void First(Iterator&);
	virtual void Last(Iterator&);
	virtual void Next(Iterator&);
	virtual void Prev(Iterator&);
	virtual boolean Done(Iterator);

	void SetView (GraphicView*, Iterator&);
	GraphicView* GetView (Iterator);

	virtual Graphic* GetGraphic();

	virtual void Interpret(Command*);
	virtual void Uninterpret(Command*);
	virtual void Update();

	virtual Manipulator *CreateManipulator(Viewer*,Event&,Transformer*,Tool*);
	virtual Command* InterpretManipulator(Manipulator*);
		 
	virtual ClassId GetClassId();
	virtual boolean IsA(ClassId);

	ExtLinkComp* GetExtLinkComp();

private:
	Graphic* name_graphic;
	NodeView* link_pinview_right;
	NodeView* link_pinview_left;
};

#endif
