/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#include <InterViews/filebrowser.h>

#include "dcted.h"
#include "dctclasses.h"
#include "dctfile.h"
 
#include <stream.h>
#include <memory.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>

// ---------------------------new record creation dialog box------------------
FileDialog::FileDialog(const char* t, const char* s) :
	FileChooser(t,s,loc,10,24,s)
{
	FileBrowser* fb = browser();
	fb->SetTextFilter(".*\\.dbd*$");
	fb->Update();
}

void FileDialog::SetTextFilter(const char* f)
{
	FileBrowser* fb = browser();
	fb->SetTextFilter(f);
}

void FileDialog::Update(void)
{
	FileBrowser* fb = browser();
	fb->Update();
}

FileDialog::~FileDialog()
{
	FileBrowser* fb = browser();
	if(loc_mod) delete [] loc;
	loc = new char[strlen(fb->GetDirectory())+1];
	strcpy(loc,fb->GetDirectory());
	loc_mod=1;
}

int FileDialog::GetSaveName(char*& rname)
{
	int rc,len;

	do
	{
		if(Accept()==true)
		{
			// ok
			const char* n=Choice();
			len=strlen(n);
	
			if(strncmp(&n[len-3],".db",3)==0)
			{
				// selected .db file exists
				rname=new char[len+1];
				strcpy(rname,n);
				rname[len-3]='\0';
				rc=FDIAL_NORMAL;
			}
			else if(strncmp(&n[len-4],".dbd",4)==0)
			{
				// selected .dbd file exists
				rname=new char[len];
				strcpy(rname,n);
				rc=FDIAL_DBD;
			}
			else if(strncmp(&n[len-4],".vdb",4)==0)
			{
				// selected .vbd file exists
				rname=new char[len];
				strcpy(rname,n);
				rc=FDIAL_VDB;
			}
			else
				rc=FDIAL_INVALID;
		}
		else
		{
			// cancel
			rname=NULL;
			rc=FDIAL_CANCEL;
		}

		if(rc==FDIAL_BAD)
		{
			rname=NULL;
		      // KE: This doesn't do anything
			SetSubtitle("Invalid file name!");
			Update();
		}
	}
	while(rc==FDIAL_BAD);

	return rc;
}

int FileDialog::GetOpenName(char*& rname)
{
	int rc,len;

	do
	{
		if(Accept()==true)
		{
			// ok
			const char* n=Choice();
			len=strlen(n);
	
			if(strncmp(&n[len-3],".db",3)==0)
			{
				if(access(n,F_OK)==0)
				{
					// selected .db file exists
					rname=new char[len+1];
					strcpy(rname,n);
					rname[len-3]='\0';

					if(access(rname,F_OK)==0)
						rc=FDIAL_NORMAL; // graphic file exists
					else
					{
						// .db file only
						rname[len-3]='.';
						rc=FDIAL_DB;
					}
				}
				else
					rc=FDIAL_BAD;
			}
			else if(strncmp(&n[len-4],".dbd",4)==0)
			{
				if(access(n,F_OK)==0)
				{
					// selected .dbd file exists
					rname=new char[len];
					strcpy(rname,n);
					rc=FDIAL_DBD;
				}
				else
					rc=FDIAL_BAD;
			}
		}
		else
		{
			// cancel
			rname=NULL;
			rc=FDIAL_CANCEL;
		}

		if(rc==FDIAL_BAD)
		{
			rname=NULL;
			SetSubtitle("File selected does not exist!");
			Update();
		}
	}
	while(rc==FDIAL_BAD);

	return rc;
}

char* FileDialog::loc = "./";
int FileDialog::loc_mod = 0;

