/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#include <InterViews/interactor.h>
#include <InterViews/window.h>
#include <InterViews/button.h>
#include <InterViews/streditor.h>
#include <InterViews/frame.h>
#include <InterViews/world.h>
#include <InterViews/glue.h>
#include <InterViews/box.h>
#include <InterViews/event.h>
#include <InterViews/sensor.h>
#include <Unidraw/dialogs.h>
#include <Unidraw/editor.h>
#include <Unidraw/unidraw.h>

#include <stream.h>
#include <string.h>

#include "dcted.h"

// ----------------------------key board stuff------------------------
const char* KLBL_REC = "r",		*CODE_REC = "r";
const char* KLBL_EXTLINK = "X",	*CODE_EXTLINK = "X";
// const char* KLBL_CONT = "C",	*CODE_CONT = "C";
const char* KLBL_RECPIN = "R",	*CODE_RECPIN = "R";
const char* KLBL_FLDPIN = "F",	*CODE_FLDPIN = "F";
const char* KLBL_MYLINK = "L",	*CODE_MYLINK = "L";
const char* KLBL_EXAMINE = "E",	*CODE_EXAMINE = "E";
const char* KLBL_FIELDEDIT = "e",	*CODE_FIELDEDIT = "e";
const char* KLBL_FIELDEXPORT = "x",	*CODE_FIELDEXPORT = "x";
const char* KLBL_MAKEDB = "y",	*CODE_MAKEDB = "y";
const char* KLBL_RENAMEDB = "z",	*CODE_RENAMEDB = "z";
const char* KLBL_RENAMEREC = "u",	*CODE_RENAMEREC = "u";
const char* KLBL_LINKFIELD = "i",	*CODE_LINKFIELD = "i";
// const char* KLBL_CONTCMD = "x",	*CODE_CONTCMD = "x";
const char* KLBL_WIRE = "w",	*CODE_WIRE = "w";
const char* KLBL_IMPORTREC = "t",	*CODE_IMPORTREC = "t";
const char* KLBL_NOWHEREEDIT = "K",	*CODE_NOWHEREEDIT = "t";
const char* KLBL_DCTNEWCMD = "n",	*CODE_DCTNEWCMD = "n";
const char* KLBL_DCTSAVECMD = "S",	*CODE_DCTSAVECMD = "S";
const char* KLBL_DCTSAVEASCMD = "s",	*CODE_DCTSAVEASCMD = "s";
const char* KLBL_DCTSAVEASVDCTCMD = "v",	*CODE_DCTSAVEASVDCTCMD = "v";
const char* KLBL_DCTVIEWCMD = "o",	*CODE_DCTVIEWCMD = "o";
const char* KLBL_PREFIX = "k",	*CODE_PREFIX = "k";
const char* KLBL_EDITTEXT = "d", 	*CODE_EDITTEXT = "d";

void global_dialog_wn_delete(Interactor* i)
{
	ManagedWindow* mw = i->GetTopLevelWindow();
	if(mw)
	{
		DctHandler* scumbag = new DctHandler();
		mw->wm_delete(scumbag);
	}
}

//---------------------global warning, message, and error dialogs----------

void global_world_warning(Interactor* i, const char* warning)
{
	World* w = unidraw->GetWorld();
	Coord x,y;
    AcknowledgeDialog dialog("WARNING!:",warning);
	Alignment al = Center;

    i->GetPosition(x,y);
    w->InsertTransient(&dialog,i,x,y,TopLeft);
    dialog.Acknowledge();
    w->Remove(&dialog);
}

void global_warning(Editor* ed, const char* warning)
{
    AcknowledgeDialog dialog("WARNING!:",warning);
    ed->InsertDialog(&dialog);
    dialog.Acknowledge();
    ed->RemoveDialog(&dialog);
}

char global_confirm(Editor* ed, const char* confirm)
{
	char c;
    ConfirmDialog dialog("CONFIRM:",confirm);
    ed->InsertDialog(&dialog);
    c=dialog.Confirm();
    ed->RemoveDialog(&dialog);
	return c; // 'y' = yes, 'n' = no, '^G' = cancel
}

// --------------------------string get dialog box---------------------

class getastring: public BasicDialog
{
public:
    getastring(const char* title, const char* sub, const char* name);
    ~getastring();
    const char* GetTheString();
	virtual boolean Accept();
private:
    StringEditor* _se;
	ButtonState* _bs;
};

getastring::getastring(const char* t, const char* sub,const char* name) :
            BasicDialog(new ButtonState,t,sub)
{
    const int space = round(.5*cm);
 
	_bs=new ButtonState;
    _se=new StringEditor(state, "123456789012345678901234567890123456","\r");
    _se->Message(name);
 
    Insert(
        new MarginFrame(
            new VBox(
                new HBox(_title,new HGlue),
                new HBox(_subtitle,new HGlue),
                new VGlue(space),
                new HBox(
                    new HGlue,
                    new Frame(_se),
                    new HGlue
                ),
                new VGlue(space),
                new HBox(
                    new HGlue,
                    new PushButton(" Cancel ",state,2),
                    new HGlue(space,0),
                    new PushButton(" Ok ",state,'\r'), // 1),
                    new HGlue
                )
            ), space, space/2, 0
        )
    );

	// input = new Sensor(noEvents);
	// input->Catch(KeyEvent);
}

boolean getastring::Accept()
{
    Event e;
    int v=0;
 
    state->SetValue(v);
    do
	{
		Read(e);
		// e.target->Handle(e);
		Forward(e);
		state->GetValue(v);
    }
	while (v == 0 && e.target != nil);

    return v == '\r' || e.target == nil;
}
 
getastring::~getastring() { delete _se; delete _bs; }
const char* getastring::GetTheString() { return _se->Text(); }
 
char* global_getastring(Editor* ed, const char* t, 
		const char* s, const char* n)
{
	char* mess;
	getastring dialog(t,s,n);
    ed->InsertDialog(&dialog);

    if(dialog.Accept())
	{
		// cerr << "getastring <" << dialog.GetTheString() << ">\n";
		mess=strdup(dialog.GetTheString());
	}
	else
		mess=strdup("\0");

    ed->RemoveDialog(&dialog);
	return mess; // mess="\0" is empty string, mess=nil is cancel
}

