/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#ifndef __dctmanip_h__
#define __dctmanip_h__

#include <stream.h>
#include <InterViews/rubline.h>
#include <Unidraw/Tools/tool.h>
#include <Unidraw/manip.h>
#include <Unidraw/manips.h>

class PopupMenu;
class ConnectorView;
class RubberWire;

//---------------------------examine tool-------------------------------
class ExamineTool : public Tool {
public:
    ExamineTool(ControlInfo* = nil);
    virtual ~ExamineTool();
 
    virtual Manipulator* CreateManipulator(Viewer*, Event&, Transformer* =nil);
 
    virtual Tool* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    void InitCmds(Viewer*, class GraphicComp*);
    Manipulator* CreatePopupManip(Viewer*,GraphicComp*);
private:
	class FieldEditCmd* field_edit_cmd;
	class FieldExportCmd* field_export_cmd;
	class RenameRecCmd* rename_rec_cmd;
	// class ContCmd* cont_cmd;
};

//-----------------------popup manipulator-----------------------
class PopupManip : public Manipulator
{
public:
    PopupManip(Viewer*, PopupMenu*, Tool* = nil);
    virtual ~PopupManip();
 
    virtual void Grasp(Event&);
    virtual boolean Manipulating(Event&);
    virtual void Effect(Event&);
 
    virtual void SetViewer(Viewer*);
    virtual void SetTool(Tool*);
 
    virtual Viewer* GetViewer();
    virtual Tool* GetTool();
private:
    Viewer* _viewer;
    PopupMenu* _popup;
    Tool* _tool;
};

//-----------------------Wire manipulator--------------------------

class WireManip : public VertexManip
{
public:
    WireManip(Viewer*, RubberWire*, Transformer* = nil, Tool* = nil);
 
    virtual boolean Manipulating(Event&);
    ConnectorView* GetTarget();
    RubberWire* GetRubberWire();
protected:
    virtual void Constrain(Event&);
private:
    ConnectorView* _target;
};
 
inline ConnectorView* WireManip::GetTarget () { return _target; }

#endif
