/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#include <InterViews/adjuster.h>
#include <InterViews/frame.h>
#include <InterViews/glue.h>
#include <InterViews/box.h>
#include <InterViews/scroller.h>
#include <InterViews/message.h>
#include <InterViews/strbrowser.h>
#include <InterViews/streditor.h>
#include <InterViews/event.h> // for event stuff
#include <InterViews/border.h>

#include "dcted.h"
#include "dctclasses.h"
#include "dctnewrec.h"
 
#include <stream.h>
#include <memory.h>
#include <string.h>
#include <stdio.h>

extern "C" {
#include "dbStaticLib.h"
};
 
#define NR_BUTT_DONE 1
#define NR_BUTT_CREATE 2
#define NR_BUTT_SHOW 3

static int rec_counter = 0;

//----------------private buttonstate thing-----------------------

NewRecBS::NewRecBS(int i) : ButtonState(i) {}

void NewRecBS::Notify()
{
	int v;
	ButtonState::Notify();

	GetValue(v);
	if(v)
	{
		SetValue(0);
		newrec_dialog->perform_update(v);
	}
}

// ---------------------------new record creation dialog box------------------
NewRecDialog::NewRecDialog(const char* t) : BasicDialog(new ButtonState(0),t,"")
{
    const int space = round(.5*cm);
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();
	DBENTRY dbe;
	char new_name[30];
	char* rd_name;
	long rc;

	main_bs = new NewRecBS(0);
	main_bs->SetEdit(this);

	show_pb = new PushButton("Show",main_bs,NR_BUTT_SHOW);
	create_pb = new PushButton("Create",state,NR_BUTT_CREATE);
	done_pb = new PushButton("Done",state,NR_BUTT_DONE);
	rec_se = new StringEditor(new ButtonState,
		"123456789012345678901234567890","\r\t\n");
	rec_sb = new StringBrowser(new ButtonState,10,24,true);
	type_sb = new StringBrowser(new ButtonState,10,24,true);
	sprintf(new_name,"%s%s_%d",du->GetPrefix(),du->GetLogName(),rec_counter++);
	rec_se->Message(new_name);
	rec_sb->Clear();
	type_sb->Clear();

	// populate the browser with all record types
	dbInitEntry(db,&dbe);
	for(rc=dbFirstRecordType(&dbe);rc==0;rc=dbNextRecordType(&dbe))
	{
    	rd_name=new char[strlen(dbGetRecordTypeName(&dbe))+1];
    	strcpy(rd_name,dbGetRecordTypeName(&dbe));
    	type_sb->Append(rd_name);
    	delete [] rd_name;
	}

	VBox* vb = new VBox;
	vb->Insert(show_pb);
	vb->Insert(create_pb);
	vb->Insert(done_pb);

	Insert(
		new MarginFrame(
			new VBox(
				new HBox(_title,new HGlue),
				new HBox(_subtitle,new HGlue),
				new VGlue(space),
				new HBox(
					new HGlue,
					vb,
					new HGlue(space,0),
					new Frame( AddScroller(type_sb) ),
					new HGlue(space,0),
					new VBox(
						new Frame( AddScroller(rec_sb) ),
						new VGlue(space),
						new Frame(
							new HBox(
								new Message("mess","New Name:",Left,0,0,0),
								rec_se
							)
						)
					),
					new HGlue
				)
				// new VGlue(space),
			), space, space/2, 0
		)
	);
}

NewRecDialog::~NewRecDialog() { }

void NewRecDialog::perform_update(int v)
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();
	DBENTRY dbe;
	char* rd_name;
	long rc;

	switch(v)
	{
	case NR_BUTT_SHOW: // show
		rec_sb->Clear();
		if(type_sb->Selections()!=0)
		{
			dbInitEntry(db,&dbe);
			dbFindRecordType(&dbe,type_sb->String(type_sb->Selection(0)));
			for(rc=dbFirstRecord(&dbe);rc==0;rc=dbNextRecord(&dbe))
			{
				if(!dbIsVisibleRecord(&dbe))
				{
    				rd_name=new char[strlen(dbGetRecordName(&dbe))+1];
    				strcpy(rd_name,dbGetRecordName(&dbe));
    				rec_sb->Append(rd_name);
    				delete [] rd_name;
    			}
			}
		}
		break;
	default: break;
	}
}

boolean NewRecDialog::GetEntry(char*& rectype, char*& recname)
{
	boolean rc=true;

	if( type_sb->Selections()!=0 ) 
	{
		rectype=type_sb->String(type_sb->Selection(0));

		if( rec_sb->Selections()!=0 )
		{
			// a nowhere record was chosen
			recname=rec_sb->String(rec_sb->Selection(0));
		}
		else
		{
			// a new record name was entered
			if( strcmp(rec_se->Text(),"")!=0 )
				recname=(char*)rec_se->Text();
			else
			{
				recname=nil;
				rc = false;
			}
		}
	}
	else
	{
		rectype=nil;
		recname=nil;
		rc = false; // error condition
	}

	return rc; // everything ok
}

boolean NewRecDialog::GetData()
{
    Event e;
    int v = 0;
 
    state->SetValue(v);
    do
	{
        Read(e);
        if (e.eventType == KeyEvent)
		{
            state->SetValue(e.keystring[0]);
        }
		else
		{
            Forward(e);
        }
        state->GetValue(v);
    } while (v!=0);

	if(v==NR_BUTT_CREATE) return true;
	else return false;
}

Interactor* NewRecDialog::AddScroller(Interactor* i)
{
    return new HBox(
        new MarginFrame(i, 2),
        new VBorder,
        new VBox(
            new UpMover(i, 1),
            new HBorder,
            new VScroller("field_scroller",i),
            new HBorder,
            new DownMover(i, 1)
        )
    );
}

