/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#include <InterViews/world.h>
#include <InterViews/box.h>
#include <InterViews/glue.h>
#include <InterViews/frame.h>
#include <InterViews/message.h>
#include <InterViews/strbrowser.h>
#include <InterViews/streditor.h>
#include <InterViews/border.h>
#include <InterViews/adjuster.h>
#include <InterViews/scroller.h>
#include <InterViews/event.h> // for event stuff

#include <Unidraw/dialogs.h>

#include "dcted.h"
#include "dctdial.h"
#include "dctclasses.h"
#include "dctnowhere.h"
 
#include <stream.h>
#include <memory.h>
#include <string.h>
#include <stdio.h>
 
extern "C" {
#include "dbStaticLib.h"
};

#define BUTT_DONE 1
#define BUTT_SHOW 2
#define BUTT_RENAME 3
#define BUTT_COPY 4
#define BUTT_EDIT 5
#define BUTT_DELETE 6

static void problem_warning(Interactor* i, Coord x,Coord y,const char* warning)
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
    World* w = du->GetWorld();
    AcknowledgeDialog dial("WARNING!:",warning);

    w->InsertTransient(&dial,i,x,y,BottomLeft);
	global_dialog_wn_delete(&dial);
    dial.Acknowledge();
    w->Remove(&dial);
}

//----------------private buttonstate thing-----------------------

NowhereBS::NowhereBS(int i) : ButtonState(i) {}

void NowhereBS::Notify()
{
	int v;
	ButtonState::Notify();

	GetValue(v);
	if(v)
	{
		SetValue(0);
		nowhere_edit->perform_update(v);
	}
}

// -----------------------------nowhere record dialog box---------------------
NowhereEdit::NowhereEdit(const char* t) : BasicDialog(new ButtonState(0),t,"")
{
    const int space = round(.5*cm);
	char* rd_name;
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();
	DBENTRY dbe;
	long rc;

	main_bs = new NowhereBS(0);
	main_bs->SetEdit(this);
	rename_pb = new PushButton("Rename",main_bs,BUTT_RENAME);
	delete_pb = new PushButton("Delete",main_bs,BUTT_DELETE);
	copy_pb = new PushButton("Copy",main_bs,BUTT_COPY);
	edit_pb = new PushButton("Edit",main_bs,BUTT_EDIT);
	show_pb = new PushButton("Show",main_bs,BUTT_SHOW);
	done_pb = new PushButton("Done",state,BUTT_DONE);

	rec_se = new StringEditor(new ButtonState,
		"123456789012345678901234567890","\r\t\n");
	rec_sb = new StringBrowser(new ButtonState,10,24,true);
	type_sb = new StringBrowser(new ButtonState,10,24,true);
	rec_se->Message("");
	rec_sb->Clear();
	type_sb->Clear();

	dbInitEntry(db,&dbe);

	// populate the browser with all record types
	for(rc=dbFirstRecordType(&dbe);rc==0;rc=dbNextRecordType(&dbe))
	{
    	rd_name=new char[strlen(dbGetRecordTypeName(&dbe))];
		strcpy(rd_name,dbGetRecordTypeName(&dbe));
    	type_sb->Append(rd_name);
    	delete [] rd_name;
	}

	VBox* vb = new VBox;
	vb->Insert(show_pb);
	vb->Insert(rename_pb);
	vb->Insert(delete_pb);
	vb->Insert(copy_pb);
	vb->Insert(edit_pb);
	vb->Insert(done_pb);

	Insert (
		new MarginFrame (
			new VBox (
				new HBox(_title,new HGlue),
				new HBox(_subtitle,new HGlue),
				new VGlue(space),
				new HBox (
					new HGlue,
					vb,
					new HGlue(space,0),
					new Frame( AddScroller(type_sb) ),
					new HGlue(space,0),
					new VBox (
						new Frame( AddScroller(rec_sb) ),
						new VGlue(space),
						new Frame (
							new HBox(
								new Message("mess","New Name:",Left,0,0,0),
								rec_se
							)
						)
					),
					new HGlue
				)
				// new VGlue(space),
			), space, space/2, 0
		)
	);
}

NowhereEdit::~NowhereEdit() { }

void NowhereEdit::perform_update(int v)
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();
	DBENTRY dbe,dben;
    World* w = du->GetWorld();
	int sel;
    Coord x,y;
	long rc;
	char* rd_name;

    GetRelative(x,y,w);
	dbInitEntry(db,&dbe);

	switch(v)
	{
	case BUTT_RENAME: // rename
		if( type_sb->Selections()!=0 && rec_sb->Selections()!=0 &&
			strcmp(rec_se->Text(),"")!=0 )
		{
			// get the index of the current string for replace later
			sel = rec_sb->Selection(0);
			if(dbFindRecord(&dbe,rec_sb->String(sel))==0)
			{
				if(dbRenameRecord(&dbe,(char*)rec_se->Text()))
					problem_warning(this,x,y,"Record could not be renamed");
				else
				{
					dbFindRecord(&dbe,(char*)rec_se->Text());
					rec_sb->Replace(rec_se->Text(),sel);
					rec_sb->UnselectAll();
					rec_se->Message("");
				}
			}
			else
				problem_warning(this,x,y,"Record to be renamed does not exist");
		}
		break;
	case BUTT_DELETE: // delete
		if(type_sb->Selections()!=0 && rec_sb->Selections()!=0)
		{
			// get the index of the current string for removal later
			sel = rec_sb->Selection(0);
			if(dbFindRecord(&dbe,rec_sb->String(sel))==0)
			{
				if(dbDeleteRecord(&dbe)==0)
				{
					rec_sb->Remove(sel);
					rec_sb->UnselectAll();
				}
				else
					problem_warning(this,x,y,"Record could not be deleted");
			}
			else
				problem_warning(this,x,y,"Record could not be deleted");
				
		}
		break;
	case BUTT_COPY: // copy
	{
		// must get the Text from the se and see if it points to nothing
		char* new_name = (char*)rec_se->Text();

		if(	type_sb->Selections()!=0 && rec_sb->Selections()!=0 &&
			strcmp(new_name,"")!=0 )
		{
			sel = rec_sb->Selection(0);
			if(dbFindRecord(&dbe,rec_sb->String(sel))==0)
			{
				dbInitEntry(db,&dben);
				if(dbFindRecord(&dben,new_name)==0)
				{
					// problem here, new record name already exists
					problem_warning(this,x,y,
						"Record cannot be copied - New name already exists");
				}
				else
				{
					dbCopyEntryContents(&dbe,&dben);
					if(dbCopyRecord(&dben,new_name,0))
						problem_warning(this,x,y,"Record could not be copied");
					else
					{
						dbFindRecord(&dben,new_name);
						rec_sb->Append(dbGetRecordName(&dben));
						rec_sb->UnselectAll();
						rec_se->Message("");
					}
				}
			}
		}
		break;
	}
	case BUTT_EDIT: // edit
		if(type_sb->Selections()!=0)
		{
			if(rec_sb->Selections()!=0)
				rc=dbFindRecord(&dbe,rec_sb->String(rec_sb->Selection(0)));
			else
			{
				const char* nrec = rec_se->Text();
				if(nrec && nrec[0])
				{
					dbFindRecordType(&dbe,
						type_sb->String(type_sb->Selection(0)));
					rc=dbCreateRecord(&dbe,(char*)nrec);
	
					if(rc)
						problem_warning(this,x,y,"Record could not be created");
					else
					{
						rec_sb->Append(dbGetRecordName(&dbe));
						rec_se->Message("");
					}
				}
			}

			if(rc==0)
			{
				FieldEditDialog fld_edit(&dbe);
    			w->InsertTransient(&fld_edit,this);
				// global_dialog_wn_delete(&fld_edit);
    			// fld_edit.GetData();
    			fld_edit.Accept();
    			w->Remove(&fld_edit);
				rec_sb->UnselectAll();
			}
		}
		break;
	case BUTT_SHOW: // show
		rec_sb->Clear();
		if(type_sb->Selections()!=0)
		{
			dbFindRecordType(&dbe,type_sb->String(type_sb->Selection(0)));

			for(rc=dbFirstRecord(&dbe);rc==0;rc=dbNextRecord(&dbe))
    			rec_sb->Append(dbGetRecordName(&dbe));
		}
		break;
	default: break;
	}
}

void NowhereEdit::GetData()
{
    Event e;
    int v = 0;
 
    state->SetValue(v);
    do
	{
        Read(e);
        if(e.eventType == KeyEvent)
            state->SetValue(e.keystring[0]);
		else
            Forward(e);

        state->GetValue(v);
    } while(v!=BUTT_DONE);
}

Interactor* NowhereEdit::AddScroller(Interactor* i)
{
    return new HBox(
        new MarginFrame(i, 2),
        new VBorder,
        new VBox(
            new UpMover(i, 1),
            new HBorder,
            new VScroller("field_scroller",i),
            new HBorder,
            new DownMover(i, 1)
        )
    );
}

