/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#define DEBUG_VDCT 0

#include <InterViews/transformer.h>
#include <InterViews/rubrect.h>

#include <Unidraw/ulist.h>
#include <Unidraw/viewer.h>
#include <Unidraw/clipboard.h>
#include <Unidraw/unidraw.h>
#include <Unidraw/catalog.h>
#include <Unidraw/manips.h>
#include <Unidraw/Graphic/lines.h>
#include <Unidraw/Graphic/pspaint.h>
#include <Unidraw/Graphic/polygons.h>
#include <Unidraw/Commands/edit.h>
#include <Unidraw/Components/text.h>
#include <Unidraw/Tools/tool.h>

#include "dctcreat.h"
#include "dctclasses.h"
#include "dctcomps.h"
#include "dctvars.h"
#include "dctrec.h"
#include "dcted.h"
#include "dctnewrec.h"

#include <stream.h>
#include <string.h>
#include <stdlib.h>

extern "C" {
#include "dbStaticLib.h"
};

// ----------------------------rec graphic stuff----------------------
RecGraphic::RecGraphic(const char* t, const char* c)
{
	Catalog* cat = unidraw->GetCatalog();
	Coord left, bottom, right, top;
	PSColor* fgblack = cat->ReadColor("fgcolor",1);
	PSColor* fgblue = cat->ReadColor("fgcolor",7);
	PSColor* bgwhite = cat->ReadColor("bgcolor",10);
	PSFont* helv14;
	PSFont* helv12;
	if(use_gdct_style_stuff)
	{
		helv14 = cat->ReadFont("font",5);
		helv12 = cat->ReadFont("font",4);
	}
	else
	{
		helv14 = cat->ReadFont("font",12);
		helv12 = cat->ReadFont("font",12);
	}

	// record name part
	rec_name = new TextGraphic(c,14);
	rec_name->FillBg(1);
	rec_name->SetColors(fgblue,bgwhite);
	rec_name->SetFont(helv14);
	rec_name->SetPattern(new PSPattern(0, (float)1));
	rec_name->SetTransformer(new Transformer(1, 0, 0, 1, 588.672, 482.704));

	// record type part
	rec_type = new TextGraphic(t,14);
	rec_type->FillBg(1);
	rec_type->SetColors(fgblue,bgwhite);
	rec_type->SetFont(helv12);
	rec_type->SetPattern(new PSPattern(0, (float)1));
	rec_type->SetTransformer(new Transformer(1, 0, 0, 1, 588.672, 482.704));
	rec_type->Align(BottomCenter,rec_name,TopCenter);

	tmp_pict = new Picture;
	tmp_pict->Append(rec_name,rec_type);
	tmp_pict->GetBox(left,bottom,right,top);

	// rectangle around the record name
	if(use_gdct_style_stuff)
		right=(right>left+140)?right:(left+140);
	else
		right=(right>left+80)?right:(left+80);
	rect = new SF_Rect(left,bottom,right,top,stdgraphic);
	rect->SetColors(fgblack,bgwhite);
	rect->FillBg(1);
	rect->SetPattern(new PSPattern(0, (float)1));
	rect->SetTransformer(new Transformer(1, 0, 0, 1, 588.672, 482.704));
	rect->Align(TopCenter,tmp_pict,TopCenter);

	this->Append(rect,tmp_pict);
}

RecGraphic::~RecGraphic() { }

void RecGraphic::GetTypeName(const char*& type,const char*& name)
{
	type=rec_type->GetOriginal();
	name=rec_name->GetOriginal();
}

Graphic* RecGraphic::Copy()
{
	Graphic* g = Picture::Copy();
	return g;
}

// ----------------------------Data field graphic stuff----------------------
DataGraphic::DataGraphic(const char* c)
{
	Coord left, bottom, right, top;
	Catalog* cat = unidraw->GetCatalog();
	PSColor* fgblack = cat->ReadColor("fgcolor",1);
	PSColor* fgblue = cat->ReadColor("fgcolor",7);
	PSColor* bgwhite = cat->ReadColor("bgcolor",10);
	PSFont* helv14;
	PSFont* helv12;
	PSFont* helv10;
	if(use_gdct_style_stuff)
	{
		helv14 = cat->ReadFont("font",5);
		helv12 = cat->ReadFont("font",4);
		helv10 = cat->ReadFont("font",13);
	}
	else
	{
		helv14 = cat->ReadFont("font",12);
		helv12 = cat->ReadFont("font",12);
		helv10 = cat->ReadFont("font",12);
	}

	rec_name = new TextGraphic(c,12);
	rec_name->FillBg(0);
	rec_name->SetColors(fgblue,bgwhite);
	rec_name->SetFont(helv10);
	rec_name->SetPattern(new PSPattern(0, (float)1));
	rec_name->SetTransformer(new Transformer(1, 0, 0, 1, 588.672, 482.704));
	rec_name->GetBox(left,bottom,right,top);
	if(use_gdct_style_stuff)
		right=left+130;
	else
		right=left+60;
	rect = new SF_Rect(left,bottom,right,top,stdgraphic);
	rect->SetColors(fgblack,bgwhite);
	rect->FillBg(1);
	rect->SetPattern(new PSPattern(0, (float)1));
	rect->SetTransformer(new Transformer(1, 0, 0, 1, 588.672, 482.704));
	rect->Align(TopCenter,rec_name,TopCenter);

	this->Append(rect,rec_name);
}

DataGraphic::~DataGraphic() { }

int DataGraphic::operator!=(DataGraphic& dg)
{
    return ( (*rec_name) != (*dg.rec_name) );
}

// ------------------------Input output field graphic stuff-----------------
FieldGraphic::FieldGraphic(const char* c)
{
	Coord left, bottom, right, top;
	Catalog* cat = unidraw->GetCatalog();
	PSColor* fgblack = cat->ReadColor("fgcolor",1);
	PSColor* fgblue = cat->ReadColor("fgcolor",7);
	PSColor* bgwhite = cat->ReadColor("bgcolor",10);
	PSFont* helv10;
	if(use_gdct_style_stuff)
		helv10 = cat->ReadFont("font",13);
	else
		helv10 = cat->ReadFont("font",12);

	rec_name = new TextGraphic(c,12);
	rec_name->FillBg(0);
	rec_name->SetColors(fgblue,bgwhite);
	rec_name->SetFont(helv10);
	rec_name->SetPattern(new PSPattern(0, (float)1));
	rec_name->SetTransformer(new Transformer(1, 0, 0, 1, 588.672, 482.704));
	rec_name->GetBox(left,bottom,right,top);

	if(use_gdct_style_stuff)
		right=left+65;
	else
		right=left+30;
	rect = new SF_Rect(left,bottom,right,top,stdgraphic);
	rect->SetColors(fgblack,bgwhite);
	rect->FillBg(1);
	rect->SetPattern(new PSPattern(0, (float)1));
	rect->SetTransformer(new Transformer(1, 0, 0, 1, 588.672, 482.704));
	rect->Align(TopCenter,rec_name,TopCenter);

	this->Append(rect,rec_name);
}

FieldGraphic::~FieldGraphic() { }

// ------------------------rec component stuff--------------------
RecComp::RecComp()
{
	Coord x=0, y=0;
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();

	// the job here is to make an empty reccomp for the menu palette
	// this is the prototype thing

	rec_graphic=nil;
	a_new_rec_name=nil;
	a_new_rec_type=nil;
	dbInitEntry(db,&rec);

	head_pict = new Picture;
	data_pict = new Picture;
	in_pict = new Picture;
	out_pict = new Picture;
	Picture* main_pict = new Picture;

	rec_graphic = new RecGraphic("Type","Record");

	in_list = new UList;
	out_list = new UList;
	data_list = new UList;

	// the main pin for the record
	PinGraphic* pin_g = new PinGraphic(x,y,stdgraphic);
	rec_pincomp = new NodeComp(pin_g);

	rec_pincomp->SetMobility(Fixed);
	rec_pincomp->SetTransMethod(Out);
	Graphic* pin_graphic = rec_pincomp->GetPin();
	Graphic* pg = rec_pincomp->GetGraphic();
	pg->Align(CenterRight,rec_graphic,CenterLeft);
	head_pict->Append(pin_graphic, rec_graphic);
	out_pict->Align(TopLeft,in_pict,TopRight);
	in_pict->Align(TopRight,data_pict,BottomCenter);
	data_pict->Align(TopCenter,head_pict,BottomCenter);
	main_pict->Append(head_pict,data_pict);
	main_pict->Append(in_pict,out_pict);
	SetGraphic(main_pict);
	ConfigChanged(0);
}

RecComp::RecComp(Coord x, Coord y, char* rec_name, char* rec_type)
{
	char new_rname[100];
	static int dup_seqno=0;
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();
	DBENTRY dbe;
	int sindx,sval;
	char ssval[50],spreval[90];

	dbInitEntry(db,&rec);
	invalid_record=0;
	rec_graphic=nil;
	a_new_rec_name=nil;
	a_new_rec_type=nil;
	strcpy(new_rname,rec_name);

	if(dbFindRecord(&rec,rec_name)==0)
	{
		// the record already exists in the database
		if(strcmp(rec_type,dbGetRecordTypeName(&rec))==0)
		{
			// correct record type
			if(dbIsVisibleRecord(&rec))
			{
				// record exists and is graphical already
				// take this as a duplicating action
				// cerr << "doing dup rec action " << rec_name << "\n";
				dbInitEntry(db,&dbe);

				// this is a duplicate of the record, so copy it
				for(sindx=strlen(new_rname)-1;sindx>=0;sindx-=1)
				{
					if(rec_name[sindx]>='0' && rec_name[sindx]<='9') continue;
					else break;
				}

				if(sindx<(strlen(new_rname)-1))
				{
					sindx++;
					sval=atoi(&new_rname[sindx]);
					new_rname[sindx]='\0';
					strcpy(spreval,new_rname);
					sval++;
				}
				else
				{
					strcpy(spreval,new_rname);
					sval=dup_seqno++;
				}

				do
				{
					sprintf(new_rname,"%s%d",spreval,sval++);
				}
				while(dbFindRecord(&dbe,new_rname)==0);

				if(dbCopyRecord(&rec,new_rname,0))
					invalid_record=3;
				else
				{
					if(dbFindRecord(&rec,new_rname))
						invalid_record=6;
					else
						dbVisibleRecord(&rec);
				}

				// I should be clearing all link data here,but I won't !!!!
			}
			else
			{
				// the record is hidden - make visible
				dbVisibleRecord(&rec);
				// create a text graphic annotation for the user with the link
				// info, clear all the link data ????? old note
			}
		}
		else
		{
			// record in database does not match this record type - error
			invalid_record=1;
		}
	}
	else
	{
		// this is a new record
		if(dbFindRecordType(&rec,rec_type)==0)
		{
			if(dbCreateRecord(&rec,rec_name))
			{
				// bad, bad, bad
				invalid_record=2;
			}
			else
			{
				dbVisibleRecord(&rec);
				dbFindField(&rec,"VAL");
			}
		}
		else
		{
			invalid_record=5;
		}
	}

	// ----------------------------------------------------------------
	// create all the areas of the record object
	head_pict = new Picture;
	data_pict = new Picture;
	in_pict = new Picture;
	out_pict = new Picture;
	Picture* main_pict = new Picture;

	// create head parts to align to - currently lines
	head_data = new Line(0, 0, 10, 0, stdgraphic);
	head_in = new Line(0, 0, 10, 0, stdgraphic);
	head_out = new Line(0, 0, 10, 0, stdgraphic);

	// create the lists to hold current displayed fields
	in_list = new UList;
	out_list = new UList;
	data_list = new UList;

	// put the head items into the appropriate areas
	data_pict->Append(head_data);
	in_pict->Append(head_in);
	out_pict->Append(head_out);

	// create the name of the record
	rec_graphic=new RecGraphic(dbGetRecordTypeName(&rec),new_rname);

	// the main pin for the record - a lot of code or what?
	PinGraphic* pin_g = new PinGraphic(x,y,stdgraphic);
	rec_pincomp = new NodeComp(pin_g);
	rec_pincomp->SetNodeType(FldNode);

	NodeVar* nv = rec_pincomp->GetNodeVar();
	char* fld;

	if((fld=nv->GetField())==NULL) fld="VAL";
	nv->SetRecField(fld,dbGetRecordName(&rec));
    rec_pincomp->SetRecord(&rec);
    rec_pincomp->SetSide('l');
	rec_pincomp->SetMobility(Fixed);
	rec_pincomp->SetTransMethod(Out);
	Graphic* pin_graphic = rec_pincomp->GetPin();
	Graphic* pg = rec_pincomp->GetGraphic();
	pg->Align(CenterRight,rec_graphic,CenterLeft);
	AlignDataItems();
	AlignOutItems();
	AlignInItems();

	// ajust the heading - record name and forward link connector
	head_pict->Append(pin_graphic, rec_graphic);
	head_pict->SetTag(rec_pincomp);

	// align all the areas
	out_pict->Align(TopLeft,in_pict,TopRight);
	in_pict->Align(TopRight,data_pict,BottomCenter);
	data_pict->Align(TopCenter,head_pict,BottomCenter);

	// set up the entire thing
	main_pict->Append(head_pict,data_pict);
	main_pict->Append(in_pict,out_pict);
	SetGraphic(main_pict);
	ConfigChanged(0);
}

RecComp::~RecComp()
{
	UList* u;
	Picture *pic_curr;
	NodeComp *pincomp,*pincomp1;
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();

	for(u = in_list->First(); u!=in_list->End(); u=u->Next())
	{
		pincomp = (NodeComp*)(*u)();
		pic_curr = (Picture*)pincomp->GetGraphic()->Parent();
		pic_curr->Remove(pincomp->GetGraphic());
		delete pincomp;
	}
	for(u = out_list->First(); u!=out_list->End(); u=u->Next())
	{
		pincomp = (NodeComp*)(*u)();
		pic_curr = (Picture*)pincomp->GetGraphic()->Parent();
		pic_curr->Remove(pincomp->GetGraphic());
		delete pincomp;
	}
	for(u = data_list->First(); u!=data_list->End(); u=u->Next())
	{
		pincomp = (NodeComp*)(*u)();
		u=u->Next();
		pincomp1 = (NodeComp*)(*u)();
		pic_curr = (Picture*)pincomp->GetGraphic()->Parent();
		pic_curr->Remove(pincomp->GetGraphic());
		pic_curr->Remove(pincomp1->GetGraphic());
		delete pincomp;
		delete pincomp1;
	}

	head_pict->Remove(rec_pincomp->GetGraphic());
	delete rec_pincomp;

	delete in_list;
	delete out_list;
	delete data_list;

	if(a_new_rec_name) delete [] a_new_rec_name;
	if(a_new_rec_type) delete [] a_new_rec_type;

	if(invalid_record==0)
	{
		dbDeleteRecord(&rec);
		dbFinishEntry(&rec);
	}
}

void RecComp::First(Iterator& i) { i.SetValue(rec_pincomp); }
void RecComp::Last(Iterator& i) { i.SetValue(rec_pincomp); }

void RecComp::Next(Iterator& i)
{
	void* v=i.GetValue();

	if(v==nil) i.SetValue(rec_pincomp);
	else i.SetValue(nil);
}

void RecComp::Prev(Iterator& i) { Next(i); }
boolean RecComp::Done(Iterator i) { return i.GetValue()==nil; }

GraphicComp* RecComp::GetComp(Iterator i) { return (GraphicComp*)i.GetValue(); }

Graphic *RecComp::GetGraphic()
{
	Graphic *graphic= GraphicComp::GetGraphic();
	return graphic;
}

RecGraphic* RecComp::GetRecGraphic() { return rec_graphic; }
NodeComp* RecComp::GetNodeComp() { return rec_pincomp; }

boolean RecComp::RenameRec(char* new_name)
{
	DBENTRY dbe;

	dbCopyEntryContents(&rec,&dbe);
	if(dbRenameRecord(&dbe,new_name))
		return true;
	if(dbFindRecord(&rec,new_name))
		return true;

	RecGraphic* tg = new RecGraphic(
		dbGetRecordTypeName(&rec),dbGetRecordName(&rec));

	head_pict->Remove(rec_graphic);
	rec_graphic = tg;
	Graphic* pg = rec_pincomp->GetGraphic();
	head_pict->Append(rec_graphic);
	pg->Align(CenterRight,rec_graphic,CenterLeft);

	out_pict->Align(TopLeft,in_pict,TopRight);
	in_pict->Align(TopRight,data_pict,BottomCenter);
	data_pict->Align(TopCenter,head_pict,BottomCenter);

	UList* u;
	NodeComp* node;
	NodeVar* node_var;
	char* fname;

	for(u = data_list->First(); u!=data_list->End(); u=u->Next())
	{
		node = (NodeComp*)(*u)();
		node_var = node->GetNodeVar();

		if((fname=node_var->GetField())==NULL)
			fname="VAL";
		else if(fname[0]=='\0')
			fname="VAL";

		node_var->SetRecField(fname,new_name);
		node->Transmit();
	}

	node = GetNodeComp();
	node_var = node->GetNodeVar();
	if((fname=node_var->GetField())==NULL)
	{
		fname="VAL";
		// cerr << "this field NULL " << node_var->GetRecord() << "\n";
	}
	else if(fname[0]=='\0')
	{
		fname="VAL";
		// cerr << "this field [0]=NULL " << node_var->GetRecord() << "\n";
	}
	node_var->SetRecField(fname,new_name);
	node->Transmit();

	Update();
	return false;
}

void RecComp::Update()
{
	Notify();
	ConfigChanged(0);
}

void RecComp::Read (istream& in)
{
	GraphicComp::Read(in);
	Catalog* catalog = unidraw->GetCatalog();
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetCatDatabase();
	char rec_name[50];

	// (1) get the associated record and changed config flag
	in >> rec_name >> cfgc;
	// resolve the record by finding it in the database
	dbInitEntry(db,&rec);
	if(dbFindRecord(&rec,rec_name))
		invalid_record=4;
	dbFindField(&rec,"VAL");

	// clear the new rec stuff
	a_new_rec_name = nil;
	a_new_rec_type = nil;

	// create all the areas of the record object
	head_pict = new Picture;
	data_pict = new Picture;
	in_pict = new Picture;
	out_pict = new Picture;
	Picture* main_pict = new Picture;

	// create head parts to align to - currently lines
	head_data = new Line(0, 0, 10, 0, stdgraphic);
	head_in = new Line(0, 0, 10, 0, stdgraphic);
	head_out = new Line(0, 0, 10, 0, stdgraphic);

	// create the lists to hold current displayed fields
	in_list = new UList;
	out_list = new UList;
	data_list = new UList;

	// put the head items into the appropriate areas
	data_pict->Append(head_data);
	in_pict->Append(head_in);
	out_pict->Append(head_out);

	// create the name of the record
	rec_graphic = new RecGraphic(dbGetRecordTypeName(&rec),rec_name);
	// (2) read the record node in 
	rec_pincomp = (NodeComp*)catalog->ReadComponent(in);
	Transformer *t = catalog->ReadTransformer(in);
	main_pict->SetTransformer(t);
	Unref(t);

	Graphic* pin_graphic = rec_pincomp->GetPin();
	Graphic* pg = rec_pincomp->GetGraphic();
	pg->Align(CenterRight,rec_graphic,CenterLeft);

	// read in all the nodes now
	int tot_in, tot_out, tot_data, i;
	NodeComp *node,*node_front,*node_back;
	FieldGraphic* a_field;
	DataGraphic* a_data;
    Graphic *a_pin_graphic,*a_pin_graphic_front,*a_pin_graphic_back;
    Graphic *a_pg,*a_pg_front,*a_pg_back;
	Picture* tmp_pict;
	DBENTRY* fp;

	// (3) these are total of each type to read in
	in >> tot_in >> tot_out >> tot_data;

	for(i=0;i<tot_in;i++)
	{
		// (4) get an in node
		node=(NodeComp*)catalog->ReadComponent(in);
		fp=node->GetRecord();
    	a_field = new FieldGraphic(dbGetFieldName(fp));
		a_pin_graphic = node->GetPin();
		a_pg = node->GetGraphic();
    	a_field->Align(CenterLeft,a_pg,CenterRight);
    	tmp_pict = new Picture;
    	tmp_pict->Append(a_pin_graphic,a_field);
		tmp_pict->SetTag(node);
		in_pict->Append(tmp_pict);
		in_list->Append(new UList(node));
	}
	for(i=0;i<tot_out;i++)
	{
		// (5) get an out node
		node=(NodeComp*)catalog->ReadComponent(in);
		fp=node->GetRecord();
    	a_field = new FieldGraphic(dbGetFieldName(fp));
		a_pin_graphic = node->GetPin();
		a_pg = node->GetGraphic();
		a_field->Align(CenterRight,a_pg,CenterLeft);
    	tmp_pict = new Picture;
    	tmp_pict->Append(a_pin_graphic,a_field);
		tmp_pict->SetTag(node);
		out_pict->Append(tmp_pict);
		out_list->Append(new UList(node));
	}
	for(i=0;i<tot_data;i+=2)
	{
		// (6) get the data nodes
		node_front=(NodeComp*)catalog->ReadComponent(in);
		node_back=(NodeComp*)catalog->ReadComponent(in);
		fp=node_front->GetField();
    	a_data = new DataGraphic(dbGetFieldName(fp));
    	a_pin_graphic_front = node_front->GetPin();
    	a_pin_graphic_back = node_back->GetPin();
    	a_pg_front = node_front->GetGraphic();
    	a_pg_back = node_back->GetGraphic();
    	a_pg_back->Align(CenterLeft,a_data,CenterRight);
    	a_data->Align(CenterLeft,a_pg_front,CenterRight);
    	tmp_pict = new Picture;
    	tmp_pict->Append(a_pin_graphic_front,a_data,a_pin_graphic_back);
		tmp_pict->SetTag(node_front);
    	data_pict->Append(tmp_pict);
    	data_list->Append(new UList(node_front));
    	data_list->Append(new UList(node_back));
	}

	AlignDataItems();
	AlignOutItems();
	AlignInItems();

	// ajust the heading - record name and forward link connector
	head_pict->Append(pin_graphic, rec_graphic);
	head_pict->SetTag(rec_pincomp);

	// align all the areas
	out_pict->Align(TopLeft,in_pict,TopRight);
	in_pict->Align(TopRight,data_pict,BottomCenter);
	data_pict->Align(TopCenter,head_pict,BottomCenter);

	// set up the entire thing
	main_pict->Append(head_pict,data_pict);
	main_pict->Append(in_pict,out_pict);
	SetGraphic(main_pict);
	ConfigChanged(0);
}

void RecComp::Write (ostream& out)
{
#if DEBUG_VDCT
    out << "[0 RecComp::Write]";
#endif    
	GraphicComp::Write(out);
	Catalog* catalog = unidraw->GetCatalog();

#if DEBUG_VDCT
    out << "[1 After GrapicsComp::Write]";
#endif    
	// (1) write out the record name and changed config flag
	out << dbGetRecordName(&rec) << " " << cfgc << " ";
#if DEBUG_VDCT
    out << "[2 After dbgetRecordName]";
#endif    
	// (2) write out the node for the record
	catalog->WriteComponent(rec_pincomp, out);
#if DEBUG_VDCT
    out << "[3 After rec_pincomp]";
#endif    
	catalog->WriteTransformer(GetGraphic()->GetTransformer(), out);
#if DEBUG_VDCT
    out << "[4 After transformer]";
#endif    

	// write out all the nodes here
	UList* u;
	int tot_in, tot_out, tot_data;
	NodeComp* node;

	// count the total nodes in each area
	for(tot_in=0,u=in_list->First();u!=in_list->End();u=u->Next()) tot_in++;
	for(tot_out=0,u=out_list->First();u!=out_list->End();u=u->Next()) tot_out++;
	for(tot_data=0,u=data_list->First();u!=data_list->End();u=u->Next())
		tot_data++;

	// (3) write out the totals for each area
	out << tot_in << " " << tot_out << " " << tot_data << " ";
#if DEBUG_VDCT
    out << "[5 After totals]";
#endif    

	for(u = in_list->First(); u!=in_list->End(); u=u->Next())
	{
		node = (NodeComp*)(*u)();
		// (4) write an in node
		catalog->WriteComponent(node, out);
	}
#if DEBUG_VDCT
    out << "[6 After loop over in_list]";
#endif    
	for(u = out_list->First(); u!=out_list->End(); u=u->Next())
	{
		node = (NodeComp*)(*u)();
		// (5) write an out node
		catalog->WriteComponent(node, out);
	}
#if DEBUG_VDCT
    out << "[7 After loop over out_list]";
#endif    
	for(u = data_list->First(); u!=data_list->End(); u=u->Next())
	{
		node = (NodeComp*)(*u)();
		// (6) write a data node
		catalog->WriteComponent(node, out);
	}
#if DEBUG_VDCT
    out << "[8 After loop over data_list]";
    out << "[All Done]";
#endif    
}

void RecComp::WriteVDCT(FILE *fp)
{
#if 0
	fprintf(fp,"#! SKIP(1)\n#! [REC_COMP]\n");
#endif

	float left=0,bottom=0,right=0,top=0;
	int lvdct,tvdct;
#if 0
	// Graphic
	GetGraphic()->GetBounds(left,bottom,right,top);
	PageToVDCT(left,top,lvdct,tvdct);
	fprintf(fp,"#! Record(\"%s\",%d,%d,0,0,\"%s\")\n",
	  dbGetRecordName(&rec),lvdct,tvdct,dbGetRecordName(&rec));
#else
	// RecGraphic This one is further right
	GetRecGraphic()->GetBounds(left,bottom,right,top);
	PageToVDCT(left,top,lvdct,tvdct);
	fprintf(fp,"#! Record(\"%s\",%d,%d,0,0,\"%s\")\n",
	  dbGetRecordName(&rec),lvdct,tvdct,dbGetRecordName(&rec));
#endif
	// Loop over fields and make them invisible
	int dctonly=TRUE;
	long rc=dbFirstField(&rec,dctonly);
	while(!rc) {
		// 0=NON_DEFAULT_VISIBLE
		// 1=ALWAYS_VISIBLE 
		// 2=NEVER_VISIBLE 
		int visibility=2;
		fprintf(fp,"#! Visibility(\"%s.%s\",%d)\n",
		  dbGetRecordName(&rec),dbGetFieldName(&rec),
		  visibility);
		rc=dbNextField(&rec,dctonly);
	}
}

boolean RecComp::HasLinks()
{
	UList* u;
	NodeComp* node;

	for(u = in_list->First(); u!=in_list->End(); u=u->Next())
	{
		node=(NodeComp*)(*u)();
		if(node->HasConnections()) return true;
	}
	for(u = out_list->First(); u!=out_list->End(); u=u->Next())
	{
		node=(NodeComp*)(*u)();
		if(node->HasConnections()) return true;
	}
	for(u = data_list->First(); u!=data_list->End(); u=u->Next())
	{
		node = (NodeComp*)(*u)();
		if(node->HasConnections()) return true;
	}
	if(rec_pincomp->HasConnections()) return true;

	return false;
}

void RecComp::Interpret(Command* cmd)
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();
	Component* c = GetRoot();

	if(cmd->IsA(DELETE_CMD))
	{
		if(!invalid_record)
		{
			dbDeleteRecord(&rec);
			dbFinishEntry(&rec);
		}
		GraphicComp::Interpret(cmd);
	}
	else
		GraphicComp::Interpret(cmd);
}

void RecComp::Uninterpret(Command* cmd)
{
	if(cmd->IsA(DELETE_CMD))
		 GraphicComp::Interpret(cmd);
	else
		 GraphicComp::Uninterpret(cmd);
}

DBENTRY* RecComp::GetRec() { return &rec; }
ClassId RecComp::GetClassId () { return REC_COMP; }
boolean RecComp::IsA(ClassId id) {return REC_COMP==id||GraphicComp::IsA(id);}

void RecComp::SetNewRecName(char* string)
{
	if(a_new_rec_name) delete [] a_new_rec_name;
	a_new_rec_name=new char[strlen(string)+1];
	strcpy(a_new_rec_name,string);
}

void RecComp::SetNewRecType(char* string)
{
	if(a_new_rec_type) delete [] a_new_rec_type;
	a_new_rec_type=new char[strlen(string)+1];
	strcpy(a_new_rec_type,string);
}

Component* RecComp::Copy()
{
	Coord x,y;
	RecComp* copy;
	PinGraphic* pin = rec_pincomp->GetPin();
	NodeComp* nodeComp;
	UList new_field_list, null_list;
	UList* u;
	Transformer* t;

	pin->GetOriginal(x,y);

	if(a_new_rec_name)
	{
		copy=new RecComp(x,y,a_new_rec_name, a_new_rec_type);
		delete [] a_new_rec_name;
		delete [] a_new_rec_type;
		a_new_rec_name=NULL;
		a_new_rec_type=NULL;
	}
	else
	{
		copy=new RecComp(x,y,dbGetRecordName(&rec),dbGetRecordTypeName(&rec));
	}

	if(copy->Invalid())
	{
		delete copy;
		cerr << "Record copy function invalid - " << dbGetRecordName(&rec);
		cerr << " of type " << dbGetRecordTypeName(&rec) << "\n";
		return NULL;
	}
	
	// put the selected fields into a list
	for(u = in_list->First(); u!=in_list->End(); u=u->Next())
	{
		nodeComp = (NodeComp*)(*u)();
		new_field_list.Append(new UList(dbGetFieldName(nodeComp->GetField())));
	}
	for(u = out_list->First(); u!=out_list->End(); u=u->Next())
	{
		nodeComp = (NodeComp*)(*u)();
		new_field_list.Append(new UList(dbGetFieldName(nodeComp->GetField())));
	}
	for(u = data_list->First(); u!=data_list->End(); u=u->Next())
	{
		nodeComp = (NodeComp*)(*u)();
		new_field_list.Append(new UList(dbGetFieldName(nodeComp->GetField())));
	}

	t=GetGraphic()->GetTransformer();
	copy->GetGraphic()->SetTransformer(new Transformer(t));
	copy->FixUpFields(&new_field_list, &null_list);
    copy->Notify();
	return copy;
}

void RecComp::AddInItem(DBENTRY* fp)
{
	UList* u;
	NodeComp* nodeComp;
	Coord x=0, y=0;
    Picture* tmp_pict=nil;
	NodeVar* nv;
    FieldGraphic* a_field;
    PinGraphic* a_pin_g;
    NodeComp* a_pincomp;
    Graphic* a_pin_graphic;
    Graphic* a_pg;

	// check if the node has been added already
	for(u = in_list->First(); u!=in_list->End(); u=u->Next())
	{
		nodeComp = (NodeComp*)(*u)();
		if( strcmp(nodeComp->GetFieldName(),dbGetFieldName(fp))==0 ) return;
	}

    a_field = new FieldGraphic(dbGetFieldName(fp));
	a_pin_g = new PinGraphic(x,y,stdgraphic);
	a_pincomp = new NodeComp(a_pin_g);
	a_pincomp->SetNodeType(FldNode);
	nv = a_pincomp->GetNodeVar();
	nv->SetRecField("0","0");
    a_pincomp->SetRecord(fp);
    a_pincomp->SetSide('l');
	a_pincomp->SetMobility(Fixed);
	a_pincomp->SetTransMethod(In);
	a_pin_graphic=a_pincomp->GetPin();
	a_pg=a_pincomp->GetGraphic();
    a_field->Align(CenterLeft,a_pg,CenterRight);
    tmp_pict = new Picture;
    tmp_pict->Append(a_pin_graphic,a_field);
	tmp_pict->SetTag(a_pincomp);
	in_pict->Append(tmp_pict);
	in_list->Append(new UList(a_pincomp));
}

void RecComp::AddOutItem(DBENTRY* fp)
{
	UList* u;
	NodeComp* nodeComp;
	Coord x=0, y=0;
    Picture* tmp_pict=nil;
	NodeVar* nv;
    FieldGraphic* a_field;
    PinGraphic* a_pin_g;
    NodeComp* a_pincomp;
    Graphic* a_pin_graphic;
    Graphic* a_pg;

	// check if the node has been added already
	for(u = out_list->First(); u!=out_list->End(); u=u->Next())
	{
		nodeComp = (NodeComp*)(*u)();
		if( strcmp(nodeComp->GetFieldName(),dbGetFieldName(fp))==0 ) return;
	}

	a_field = new FieldGraphic(dbGetFieldName(fp));
	a_pin_g = new PinGraphic(x,y,stdgraphic);
	a_pincomp = new NodeComp(a_pin_g);
	a_pincomp->SetNodeType(FldNode);
	nv = a_pincomp->GetNodeVar();
	nv->SetRecField("0","0");
    a_pincomp->SetRecord(fp);
    a_pincomp->SetSide('r');
	a_pincomp->SetMobility(Fixed);
	a_pincomp->SetTransMethod(In);
	a_pin_graphic = a_pincomp->GetPin();
	a_pg = a_pincomp->GetGraphic();
	a_field->Align(CenterRight,a_pg,CenterLeft);
	tmp_pict = new Picture;
	tmp_pict->Append(a_pin_graphic,a_field);
	tmp_pict->SetTag(a_pincomp);
	out_pict->Append(tmp_pict);
	out_list->Append(new UList(a_pincomp));
}

void RecComp::AddDataItem(DBENTRY* fp)
{
	UList* u;
	NodeComp* nodeComp;
	Coord x=0, y=0;
    Picture* tmp_pict=nil;
	NodeVar* nv;
    DataGraphic* a_data;
    PinGraphic *a_pin_g_front,*a_pin_g_back;
    NodeComp *a_pincomp_front,*a_pincomp_back;
    Graphic *a_pin_graphic_front,*a_pin_graphic_back;
    Graphic *a_pg_front,*a_pg_back;

	// check if the node has been added already
	for(u = data_list->First(); u!=data_list->End(); u=u->Next())
	{
		nodeComp=(NodeComp*)(*u)();
		if( strcmp(nodeComp->GetFieldName(),dbGetFieldName(fp))==0 ) return;
	}

    a_data = new DataGraphic(dbGetFieldName(fp));

	// left side pin
	a_pin_g_front = new PinGraphic(x,y,stdgraphic);
	a_pincomp_front = new NodeComp(a_pin_g_front);
	a_pincomp_front->SetNodeType(FldNode);
	nv = a_pincomp_front->GetNodeVar();
	nv->SetRecField(dbGetFieldName(fp),dbGetRecordName(fp));
   	a_pincomp_front->SetRecord(fp);
	a_pincomp_front->SetMobility(Fixed);
    a_pincomp_front->SetSide('l');
	a_pincomp_front->SetTransMethod(Out);
	a_pin_graphic_front = a_pincomp_front->GetPin();
	a_pg_front = a_pincomp_front->GetGraphic();

	// right side pin
    a_pin_g_back = new PinGraphic(x,y,stdgraphic);
    a_pincomp_back = new NodeComp(a_pin_g_back);
	a_pincomp_back->SetNodeType(FldNode);
	nv = a_pincomp_back->GetNodeVar();
	nv->SetRecField(dbGetFieldName(fp),dbGetRecordName(fp));
   	a_pincomp_back->SetRecord(fp);
    a_pincomp_back->SetSide('r');
    a_pincomp_back->SetMobility(Fixed);
	a_pincomp_back->SetTransMethod(Out);
    a_pin_graphic_back = a_pincomp_back->GetPin();
    a_pg_back = a_pincomp_back->GetGraphic();

    a_pg_back->Align(CenterLeft,a_data,CenterRight);
    a_data->Align(CenterLeft,a_pg_front,CenterRight);
    tmp_pict = new Picture;
    tmp_pict->Append(a_pin_graphic_front,a_data,a_pin_graphic_back);
	tmp_pict->SetTag(a_pincomp_front); // or this?
    data_pict->Append(tmp_pict);

	// store as a pair
    data_list->Append(new UList(a_pincomp_front));
    data_list->Append(new UList(a_pincomp_back));
}

void RecComp::FixUpFields(UList* alist, UList* rlist)
{
	UList* u, *v;
	char* n;
	long rc;
	DBENTRY fp;

	dbCopyEntryContents(&rec,&fp);
	for(u=alist->First();u!=alist->End();u=u->Next())
	{
		n=(char*)(*u)();

		// resolve to a Field if possible
		for(rc=dbFirstField(&fp,0);rc==0;rc=dbNextField(&fp,0))
		{
			if( strcmp(n,dbGetFieldName(&fp))==0 )
			{
				switch(dbGetFieldType(&fp))
				{
				case DCT_INLINK:	AddInItem(&fp); break;
				case DCT_OUTLINK:
				case DCT_FWDLINK:	AddOutItem(&fp); break;
				default:			AddDataItem(&fp); break;
				}
				break;
			}
		}
		if(rc)
		{
			// field not found?????
		}
	}
	for(u=rlist->First();u!=rlist->End();u=u->Next())
	{
	    n=(char*)(*u)();
	    NodeComp *node;
	    for(v=in_list->First();v!=in_list->End();v=v->Next()) {
	    	node = (NodeComp*)(*v)();
	    	if( strcmp(n,node->GetFieldName())==0 ) {
		    in_list->Delete(node);
		    v=in_list;
		}
	    }
	    for(v=out_list->First();v!=out_list->End();v=v->Next()) {
	    	node = (NodeComp*)(*v)();
	    	if( strcmp(n,node->GetFieldName())==0 ) {
		    out_list->Delete(node);
		    v=out_list;
		}
	    }
	    for(v=data_list->First();v!=data_list->End();v=v->Next()) {
	    	node = (NodeComp*)(*v)();
	    	if( strcmp(n,node->GetFieldName())==0 ) {
		    data_list->Delete(node);
		    v=data_list;
		}
	    }
    	}
	
	AlignDataItems();
	AlignOutItems();
	AlignInItems();
	out_pict->Align(TopLeft,in_pict,TopRight);
	in_pict->Align(TopRight,data_pict,BottomCenter);
	data_pict->Align(TopCenter,head_pict,BottomCenter);
	ConfigChanged(1);
}

void RecComp::AlignInItems()
{
	UList* u;
	NodeComp* pincomp;
	Picture *pic_curr,*pic_prev;

	pic_prev=(Picture*)head_in;

	for(u=in_list->First();u!=in_list->End();u=u->Next())
	{
		pincomp=(NodeComp*)(*u)();
		pic_curr=(Picture*)pincomp->GetGraphic()->Parent();
		pic_prev->Align(TopCenter,pic_curr,BottomCenter);
		pic_prev=pic_curr;
	}
}

void RecComp::AlignOutItems()
{
	UList* u;
	NodeComp* pincomp;
	Picture *pic_curr,*pic_prev;

	pic_prev = (Picture*)head_out;

	for(u = out_list->First(); u!=out_list->End(); u=u->Next())
	{
		pincomp = (NodeComp*)(*u)();
		pic_curr = (Picture*)pincomp->GetGraphic()->Parent();
		pic_prev->Align(TopCenter,pic_curr,BottomCenter);
		pic_prev = pic_curr;
	}
}

void RecComp::AlignDataItems()
{
	UList* u;
	NodeComp* pincomp;
	Picture *pic_curr,*pic_prev;

	pic_prev = (Picture*)head_data;

	for(u = data_list->First(); u!=data_list->End(); u=u->Next())
	{
		u=u->Next();
		pincomp = (NodeComp*)(*u)();
		pic_curr = (Picture*)pincomp->GetGraphic()->Parent();
		pic_prev->Align(TopCenter,pic_curr,BottomCenter);
		pic_prev = pic_curr;
	}
}

// ------------------------rec view stuff--------------------
RecView::RecView(RecComp* subj) : GraphicView(subj)
{
	rec_pinview=nil;
	in_view_list=nil;
	out_view_list=nil;
	data_view_list=nil;
	view_list=nil;
}
 
RecView::~RecView()
{
	UList* u;
	Picture *pic_curr;
	NodeView *pinview,*pinview1;

	for(u = data_view_list->First(); u!=data_view_list->End(); u=u->Next())
	{
		pinview = (NodeView*)(*u)();
		u=u->Next();
		pinview1 = (NodeView*)(*u)();
		pic_curr = (Picture*)pinview->GetGraphic()->Parent();
		pic_curr->Remove(pinview->GetGraphic());
		pic_curr->Remove(pinview1->GetGraphic());
		delete pinview;
		delete pinview1;
	}
	for(u = out_view_list->First(); u!=out_view_list->End(); u=u->Next())
	{
		pinview = (NodeView*)(*u)();
		pic_curr = (Picture*)pinview->GetGraphic()->Parent();
		pic_curr->Remove(pinview->GetGraphic());
		delete pinview;
	}
	for(u = in_view_list->First(); u!=in_view_list->End(); u=u->Next())
	{
		pinview = (NodeView*)(*u)();
		pic_curr = (Picture*)pinview->GetGraphic()->Parent();
		pic_curr->Remove(pinview->GetGraphic());
		delete pinview;
	}

	head_pict->Remove(rec_pinview->GetGraphic());
	delete rec_pinview;
	delete data_view_list;
	delete out_view_list;
	delete in_view_list;
	delete view_list;
}

// make sure that these iterate through all three lists, data in out

void RecView::First(Iterator& i)
{ 
	UList* v = (UList*)view_list->First();
	it_view.SetValue(view_list->First());
	i.SetValue((PinView*)(*v)());
}
void RecView::Last(Iterator& i)
{
	UList* v = (UList*)view_list->Last();
	it_view.SetValue(view_list->Last());
	i.SetValue((PinView*)(*v)());
}
void RecView::Next(Iterator& i)
{
	UList* v = (UList*)it_view.GetValue();
	it_view.SetValue(v->Next());
	UList* vn = (UList*)it_view.GetValue();
	i.SetValue((PinView*)(*vn)());
}
void RecView::Prev(Iterator& i)
{
	UList* v = (UList*)it_view.GetValue();
	it_view.SetValue(v->Prev());
	UList* vn = (UList*)it_view.GetValue();
	i.SetValue((PinView*)(*vn)());
}
boolean RecView::Done(Iterator)
{
	UList* v = (UList*)it_view.GetValue();
	if(view_list->End()==v) return true;
	else return false;
}
 
void RecView::SetView(GraphicView* gv,Iterator& i) { i.SetValue(gv); }
GraphicView* RecView::GetView(Iterator i) { return (GraphicView*)i.GetValue(); }
RecComp* RecView::GetRecComp() { return (RecComp*)GetSubject(); }
ClassId RecView::GetClassId () { return REC_VIEW; }
boolean RecView::IsA (ClassId id) { return REC_VIEW==id||GraphicView::IsA(id); }
void RecView::Interpret(Command* cmd) { GraphicView::Interpret(cmd); }
void RecView::Uninterpret(Command* cmd) { GraphicView::Uninterpret(cmd); }

void RecView::Update()
{
	RecComp* comp = GetRecComp();
	Graphic* graphic = GetGraphic();
	IncurDamage(graphic);

	if(comp->ConfigChanged())
	{
		SyncFields();
		Dct* dct_ed = (Dct*)GetViewer()->GetEditor();
		ModifStatusVar* nv=(ModifStatusVar*)dct_ed->GetState("ModifStatusVar");
		nv->SetModifStatus(true);
	}

	RecGraphic* rg = comp->GetRecGraphic();
	TextGraphic* tg = rg->GetRecText();
	TextGraphic* tg1 = rec_graphic->GetRecText();

	if( (*tg)!=(*tg1) )
	{
		//----------------added from rename-------------------
		if(rec_graphic)
		{
			const char *type, *name;
			comp->GetRecGraphic()->GetTypeName(type,name);
			RecGraphic* g = new RecGraphic(type,name);
			head_pict->Remove(rec_graphic);
			delete rec_graphic;
			rec_graphic = g;
			Graphic* pg = rec_pinview->GetGraphic();
			head_pict->Append(rec_graphic);
			pg->Align(CenterRight,rec_graphic,CenterLeft);
		}
		// ------------------------------------------------------

		out_pict->Align(TopLeft,in_pict,TopRight);
		in_pict->Align(TopRight,data_pict,BottomCenter);
		data_pict->Align(TopCenter,head_pict,BottomCenter);

		Dct* dct_ed = (Dct*)GetViewer()->GetEditor();
		ModifStatusVar* nv=(ModifStatusVar*)dct_ed->GetState("ModifStatusVar");
		nv->SetModifStatus(true);
		unidraw->Update();
	}

	*graphic = *comp->GetGraphic();
	IncurDamage(graphic);
	rec_pinview->Update();
	EraseHandles();
}

Manipulator *RecView::CreateManipulator(Viewer* v, Event& e,
						Transformer* t, Tool* tt)
{
	Manipulator* manip;
	manip = GraphicView::CreateManipulator(v, e, t, tt);
	return manip;
}
 
Command *RecView::InterpretManipulator(Manipulator* m)
{
	DragManip* dm = (DragManip*) m;
	Dct* ed = (Dct*)dm->GetViewer()->GetEditor();
	Tool* tool = dm->GetTool();
	Command* comm = nil;
	RecComp* rec_comp = GetRecComp();
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db=du->GetDatabase();
	Transformer* rel = dm->GetTransformer();
	SlidingRect* slidingRect = (SlidingRect*) dm->GetRubberband();
    Coord cx0, cy0, cx1, cy1;
    float cx, cy, px, py;
	static int rec_counter=0;

	if (tool->IsA(GRAPHIC_COMP_TOOL))
	{
		// must get a record name and type from the user
		NewRecDialog nrd("New Graphic Record Creation");
		char *recname, *rectype;
		boolean go_again;
		DBENTRY dbe;

		ed->InsertDialog(&nrd);
		global_dialog_wn_delete(&nrd);

		do
		{
			go_again=false;

			if(!nrd.Accept())
			{
				if( nrd.GetEntry(rectype,recname)==false )
				{
					// this is bad entry from user
					go_again=true;
				}
				else
				{
					// this is good entry from user
					// check to see if record name already exists

					dbInitEntry(db,&dbe);
					if(dbFindRecord(&dbe,recname)==0)
					{
						if(dbIsVisibleRecord(&dbe))
						{
							global_warning(ed,"Record Already Exists.");
							go_again=true;
						}
						else
						{
							char hid_link[1500];
							DBENTRY fp;
							int hid_count=0;
							long rc;

							dbCopyEntryContents(&dbe,&fp);
							rec_comp->SetNewRecName(recname);
							rec_comp->SetNewRecType(rectype);
							strcpy(hid_link,dbGetRecordName(&fp));
							strcat(hid_link,"\nHIDDEN LINK DATA:\n");

							for(rc=dbFirstField(&fp,0);rc==0;rc=dbNextField(&fp,0))
							{
								switch(dbGetFieldType(&fp))
								{
								case DCT_FWDLINK:
								case DCT_INLINK:
								case DCT_OUTLINK:
									if(dbGetLinkType(&fp)==DCT_LINK_PV)
									{
										strcat(hid_link,dbGetFieldName(&fp));
										strcat(hid_link,": ");
										strcat(hid_link,dbGetString(&fp));
										strcat(hid_link,"\n");
										dbPutString(&fp,"0");
										hid_count++;
									}
									break;
								default: break;
								}
							}

							if(hid_count>0)
							{
								TextComp* tc = new TextComp(
									new TextGraphic(hid_link,stdgraphic));
						
   								slidingRect->GetCurrent(cx0, cy0, cx1, cy1);
   								px = float(cx0 + cx1) / 2;
   								py = float(cy0 + cy1) / 2;
   								rel->InvTransform(px, py, cx, cy);
								Graphic* grcomp = tc->GetGraphic();
								grcomp->GetCenter(px, py);
								grcomp->Translate(cx - px, cy - py - 35);
								ComponentView* tv = tc->Create(COMPONENT_VIEW);
								tc->Attach(tv);
								tv->Update();
								PasteCmd* pc =
									new PasteCmd(ed,new Clipboard(tc));
								pc->Execute();
								delete pc;
							}
						}
						// ed->RemoveDialog(&nrd);
					}
					else
					{
						rec_comp->SetNewRecName(recname);
						rec_comp->SetNewRecType(rectype);
					}
				}
			}
			else
			{
				// cancel was hit
				ed->RemoveDialog(&nrd);
				return nil;
			}
		} while(go_again==true);
		ed->RemoveDialog(&nrd);
	}

	comm = GraphicView::InterpretManipulator(m);
	return comm;
}

Graphic* RecView::GetGraphic()
{
	Graphic* graphic = GraphicView::GetGraphic();
	char *rec_name, *rec_type;
	DBENTRY* rp;

	if(graphic==NULL)
	{
		view_list = new UList;
		RecComp* rec_comp = GetRecComp();
		if(rec_comp==NULL) { cerr << "NULL rec_comp! - exiting"; exit(0); }
		NodeComp* rec_pincomp = rec_comp->GetNodeComp();
		if(rec_pincomp==NULL) { cerr << "nil rec_pincomp"; exit(0); }
		rec_pinview = (NodeView*)rec_pincomp->Create(COMPONENT_VIEW);
		if(rec_pinview==NULL) { cerr << "nil rec_pinview"; exit(0); }

		rec_pincomp->Attach(rec_pinview);
		Graphic* pin_g_view = rec_pinview->GetGraphic();
		rp=rec_pincomp->GetRecord();

		if(rp==NULL)
		{
			rec_name="Record";
			rec_type="Type";
		}
		else
		{
			rec_name=dbGetRecordName(rp);
			rec_type=dbGetRecordTypeName(rp);
		}

		rec_graphic = new RecGraphic(rec_type,rec_name);
		pin_g_view->Align(CenterRight,rec_graphic,CenterLeft);
		view_list->Append(new UList(rec_pinview));
		head_pict = new Picture;
		in_pict = new Picture;
		data_pict = new Picture;
		out_pict = new Picture;
		graphic = new Picture;
		head_data = new Line(0, 0, 10, 0, stdgraphic);
		head_in = new Line(0, 0, 10, 0, stdgraphic);
		head_out = new Line(0, 0, 10, 0, stdgraphic);
		data_pict->Append(head_data);
		in_pict->Append(head_in);
		out_pict->Append(head_out);
		data_view_list = new UList;
		out_view_list = new UList;
		in_view_list = new UList;
		SyncFields();
		head_pict->Append(pin_g_view, rec_graphic);
		out_pict->Align(TopLeft,in_pict,TopRight); // BottomCenter);
		in_pict->Align(TopRight,data_pict,BottomCenter);
		data_pict->Align(TopCenter,head_pict,BottomCenter);
		graphic->Append(head_pict,data_pict);
		graphic->Append(in_pict,out_pict);
		SetGraphic(graphic);
	}
	return graphic;
}

void RecView::AddInItem(NodeComp* tmp_pincomp)
{
	DBENTRY* fp = tmp_pincomp->GetField();
	NodeView* tmp_pinview = (NodeView*)tmp_pincomp->Create(COMPONENT_VIEW);
	tmp_pincomp->Attach(tmp_pinview);
	Graphic* tmp_gview = tmp_pinview->GetGraphic();
	FieldGraphic* tmp_graphic = new FieldGraphic(dbGetFieldName(fp));
	tmp_graphic->Align(CenterLeft,tmp_gview,CenterRight);
	Picture* tmp_pict = new Picture;
	tmp_pict->Append(tmp_gview,tmp_graphic);
	in_pict->Append(tmp_pict);
	in_view_list->Append(new UList(tmp_pinview));
	view_list->Append(new UList(tmp_pinview));
}

void RecView::AddOutItem(NodeComp* tmp_pincomp)
{
	DBENTRY* fp = tmp_pincomp->GetField();
	NodeView* tmp_pinview = (NodeView*)tmp_pincomp->Create(COMPONENT_VIEW);
	tmp_pincomp->Attach(tmp_pinview);
	Graphic* tmp_gview = tmp_pinview->GetGraphic();
	FieldGraphic* tmp_graphic = new FieldGraphic(dbGetFieldName(fp));
	tmp_graphic->Align(CenterRight,tmp_gview,CenterLeft);
	Picture* tmp_pict = new Picture;
	tmp_pict->Append(tmp_gview,tmp_graphic);
	out_pict->Append(tmp_pict);
	out_view_list->Append(new UList(tmp_pinview));
	view_list->Append(new UList(tmp_pinview));
}

void RecView::AddDataItem(NodeComp* left_pincomp,NodeComp* right_pincomp)
{
	DBENTRY* fp=left_pincomp->GetField();
	NodeView* left_pinview=(NodeView*)left_pincomp->Create(COMPONENT_VIEW);
	NodeView* right_pinview=(NodeView*)right_pincomp->Create(COMPONENT_VIEW);
	left_pincomp->Attach(left_pinview);
	right_pincomp->Attach(right_pinview);
	Graphic* left_gview = left_pinview->GetGraphic();
	Graphic* right_gview = right_pinview->GetGraphic();
	Graphic* tmp_graphic = new DataGraphic(dbGetFieldName(fp));
	right_gview->Align(CenterLeft,tmp_graphic,CenterRight);
	tmp_graphic->Align(CenterLeft,left_gview,CenterRight);
	Picture* tmp_pict = new Picture;
	tmp_pict->Append(left_gview,tmp_graphic,right_gview);
	data_pict->Append(tmp_pict);
	data_view_list->Append(new UList(left_pinview));
	data_view_list->Append(new UList(right_pinview));
	view_list->Append(new UList(left_pinview));
	view_list->Append(new UList(right_pinview));
}

void RecView::AlignItems(UList* the_list, Picture* to)
{
	UList* u;
	Picture *prev_pict,*tmp_pict;
	NodeView* pinview;

	if(!the_list) return;
	prev_pict = to;

	for(u = the_list->First(); u!=the_list->End(); u=u->Next())
	{
		if(the_list==data_view_list) u=u->Next();
		pinview = (NodeView*)(*u)();
		tmp_pict = (Picture*)pinview->GetGraphic()->Parent();
		if(prev_pict) prev_pict->Align(TopCenter,tmp_pict,BottomCenter);
		else head_out->Align(TopCenter,tmp_pict,BottomCenter);
		pinview->Update();
		prev_pict=tmp_pict;
	}
}

void RecView::SyncFields()
{
	UList *u,*v;
	NodeComp *pincomp,*pincomp1,*nodecomp;
	NodeView *pinview;
	RecComp* rec_comp = GetRecComp();
	UList* in_list = rec_comp->GetInList();
	UList* out_list = rec_comp->GetOutList();
	UList* data_list = rec_comp->GetDataList();

	for(u=in_list->First();u!=in_list->End();u=u->Next())
	{
		pincomp=(NodeComp*)(*u)();

		for(v = in_view_list->First(); v!=in_view_list->End(); v=v->Next())
		{
			pinview = (NodeView*)(*v)();
			nodecomp = (NodeComp*)pinview->GetPinComp();
			if(nodecomp->GetField()==pincomp->GetField()) break;
		}

		if(v==in_view_list->End())
		{
			// new item to add
			AddInItem(pincomp);
		}
	}

	for(u=out_list->First();u!=out_list->End();u=u->Next())
	{
		pincomp=(NodeComp*)(*u)();

		for(v = out_view_list->First(); v!=out_view_list->End(); v=v->Next())
		{
			pinview = (NodeView*)(*v)();
			nodecomp = (NodeComp*)pinview->GetPinComp();
			if(nodecomp->GetField()==pincomp->GetField()) break;
		}

		if(v==out_view_list->End())
		{
			// new item to add
			AddOutItem(pincomp);
		}
	}

	for(u=data_list->First();u!=data_list->End();u=u->Next())
	{
		pincomp=(NodeComp*)(*u)();
		pincomp1=(NodeComp*)(*(u->Next()))();

		for(v = data_view_list->First(); v!=data_view_list->End(); v=v->Next())
		{
			pinview = (NodeView*)(*v)();
			nodecomp = (NodeComp*)pinview->GetPinComp();
			if(nodecomp->GetField()==pincomp->GetField()) break;
		}

		if(v==data_view_list->End())
		{
			// new item to add
			AddDataItem(pincomp,pincomp1);
		}
	}
	
	for(v = in_view_list->First(); v!=in_view_list->End(); v=v->Next())
	{
	    pinview = (NodeView*)(*v)();
	    nodecomp = (NodeComp*)pinview->GetPinComp();
	    for(u=in_list->First();u!=in_list->End();u=u->Next())
	    {
		pincomp=(NodeComp*)(*u)();
	    	if( strcmp(nodecomp->GetFieldName(),pincomp->GetFieldName())==0 ) break;
	    }
	    
	    if(u==in_list->End())
	    {
		in_pict->Remove(pinview->GetGraphic()->Parent());
		in_view_list->Delete(pinview);
		view_list->Delete(pinview);
		v=in_view_list;
	    }
	}

	for(v = out_view_list->First(); v!=out_view_list->End(); v=v->Next())
	{
	    pinview = (NodeView*)(*v)();
	    nodecomp = (NodeComp*)pinview->GetPinComp();
	    for(u=out_list->First();u!=out_list->End();u=u->Next())
	    {
		pincomp=(NodeComp*)(*u)();
	    	if( strcmp(nodecomp->GetFieldName(),pincomp->GetFieldName())==0 ) break;
	    }
	    
	    if(u==out_list->End())
	    {
		out_pict->Remove(pinview->GetGraphic()->Parent());
		out_view_list->Delete(pinview);
		view_list->Delete(pinview);
		v=out_view_list;
	    }
	}

	for(v = data_view_list->First(); v!=data_view_list->End(); v=v->Next())
	{
	    pinview = (NodeView*)(*v)();
	    nodecomp = (NodeComp*)pinview->GetPinComp();
	    for(u=data_list->First();u!=data_list->End();u=u->Next())
	    {
		pincomp=(NodeComp*)(*u)();
	    	if( strcmp(nodecomp->GetFieldName(),pincomp->GetFieldName())==0 ) break;
	    }
	    
	    if(u==data_list->End())
	    {
		data_pict->Remove(pinview->GetGraphic()->Parent());
		data_view_list->Delete(pinview);
		view_list->Delete(pinview);
		v=data_view_list;
	    }
	}

	AlignItems(in_view_list,(Picture*)head_in);
	AlignItems(out_view_list,(Picture*)head_out);
	AlignItems(data_view_list,(Picture*)head_data);
	out_pict->Align(TopLeft,in_pict,TopRight);
	in_pict->Align(TopRight,data_pict,BottomCenter);
	data_pict->Align(TopCenter,head_pict,BottomCenter);
}

void RecView::RemoveField(NodeView* pinview)
{
UList *u;

	if( (u=in_view_list->Find(pinview)) )
	{
		in_pict->Remove(pinview->GetGraphic()->Parent());
		in_view_list->Delete(pinview);
	}
	else if( (u=out_view_list->Find(pinview)) )
	{
		out_pict->Remove(pinview->GetGraphic()->Parent());
		out_view_list->Delete(pinview);
	}
	else if( (u=data_view_list->Find(pinview)) )
	{
		data_pict->Remove(pinview->GetGraphic()->Parent());
		data_view_list->Delete(pinview);
	}

	view_list->Delete(pinview);
}

/* **************************** Emacs Editing Sequences ***************** */
/* Local Variables: */
/* tab-width: 4 */
/* c-basic-offset: 4 */
/* c-comment-only-line-offset: 0 */
/* c-file-offsets: ((substatement-open . 0) (label . 0)) */
/* End: */
