/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#ifndef __dctrec_H__
#define __dctrec_H__

#include <stream.h>

#include <Unidraw/iterator.h>
#include <Unidraw/Components/grcomp.h>
#include <Unidraw/Components/grview.h>
#include <Unidraw/Components/psview.h>
#include <Unidraw/Graphic/picture.h>

class SF_Rect;
class Command;
class NodeView;
class NodeComp;
class UList;
class Line;
class TextGraphic;

extern "C" {
#include "dbStaticLib.h"
};

// ---------------the graphic thing for records-----------------
class RecGraphic: public Picture
{
public:
	RecGraphic(const char* type, const char* name);
	virtual ~RecGraphic();
	TextGraphic* GetRecText() { return rec_name; }
	void GetTypeName(const char*& type,const char*& name);
	virtual Graphic* Copy();
private:
	Picture* tmp_pict;
	TextGraphic* rec_name;
	TextGraphic* rec_type;
	SF_Rect* rect;
};

// ---------------the graphic thing for fields-----------------
class FieldGraphic: public Picture
{
public:
	FieldGraphic(const char*);
	virtual ~FieldGraphic();
private:
	TextGraphic* rec_name;
	SF_Rect* rect;
};

// ---------------the graphic thing for data-----------------
class DataGraphic: public Picture
{
public:
	DataGraphic(const char*);
	virtual ~DataGraphic();
	int operator!=(DataGraphic&);
private:
	TextGraphic* rec_name;
	SF_Rect* rect;
};

// ---------------------the record component thing--------------------
class RecComp: public GraphicComp
{
public:
	RecComp();
	RecComp(Coord x, Coord y, char* rec_name, char* rec_type);
	virtual ~RecComp();

	virtual void First(Iterator&);
	virtual void Last(Iterator&);
	virtual void Next(Iterator&);
	virtual void Prev(Iterator&);
	virtual boolean Done(Iterator);
	virtual GraphicComp* GetComp(Iterator);

	virtual Graphic* GetGraphic();

	virtual void Interpret(Command*);
	virtual void Uninterpret(Command*);
	virtual void Update();
	virtual Component *Copy();

	virtual void Read(istream&);
	virtual void Write(ostream&);
	void WriteVDCT(FILE *fp);

	virtual ClassId GetClassId();
	virtual boolean IsA(ClassId);

	DBENTRY* GetRec();
	RecComp* GetRecComp();
	NodeComp* GetNodeComp();
	RecGraphic* GetRecGraphic();
	boolean RenameRec(char*);

	UList* GetInList() { return in_list; }
	UList* GetOutList() { return out_list; }
	UList* GetDataList() { return data_list; }

	void SetNewRecName(char*);
	void SetNewRecType(char*);

	boolean HasLinks();
	void FixUpFields(UList*, UList*);
	int ConfigChanged() { return cfgc; }
	void ConfigChanged(int i) { cfgc=i; }
	int Invalid() { return invalid_record?1:0; }
private:
	void AddDataItem(DBENTRY*);
	void AddInItem(DBENTRY*);
	void AddOutItem(DBENTRY*);
	void AlignDataItems();
	void AlignInItems();
	void AlignOutItems();

	DBENTRY rec;
	int cfgc;
	int invalid_record;
	RecGraphic* rec_graphic;
	NodeComp* rec_pincomp;
	Line	*head_data,*head_in,*head_out;
	UList	*in_list,*out_list,*data_list;
	Picture *head_pict,*in_pict,*out_pict,*data_pict;
	char	*a_new_rec_name,*a_new_rec_type;
};

// ---------------------the record view thing--------------------
class RecView: public GraphicView
{
public:
	RecView(RecComp* = nil);
	virtual ~RecView();

	virtual void First(Iterator&);
	virtual void Last(Iterator&);
	virtual void Next(Iterator&);
	virtual void Prev(Iterator&);
	virtual boolean Done(Iterator);

	void SetView (GraphicView*, Iterator&);
	GraphicView* GetView (Iterator);

	virtual Graphic* GetGraphic();

	virtual void Interpret(Command*);
	virtual void Uninterpret(Command*);
	virtual void Update();

	virtual Manipulator *CreateManipulator(Viewer*,Event&,Transformer*,Tool*);
	virtual Command* InterpretManipulator(Manipulator*);
		 
	virtual ClassId GetClassId();
	virtual boolean IsA(ClassId);

	RecComp* GetRecComp();
	void RemoveField(NodeView*);

private:

	void AddDataItem(NodeComp*,NodeComp*);
	void AddInItem(NodeComp*);
	void AddOutItem(NodeComp*);

	void SyncFields();
	void AlignItems(UList*,Picture*);

	RecGraphic* rec_graphic;
	NodeView* rec_pinview;
	Iterator it_view;
	UList* view_list;

	Picture* in_pict;
	Picture* data_pict;
	Picture* out_pict;
	Picture* head_pict;

	// the lists for all the current fields displayed
	UList* data_view_list;
	UList* in_view_list;
	UList* out_view_list;

	// the head alignment thing for fields to align to
	Line* head_data;
	Line* head_in;
	Line* head_out;

};

#endif
