/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#ifndef dctvars_h
#define dctvars_h

#include <Unidraw/statevars.h>

extern "C" {
#include "dbStaticLib.h"
};

class Component;

// ---------------------state variable for nodes--------------------
class NodeVar : public NameVar
{
public:
    NodeVar(const char* name = "", char* = nil, char* = nil);
	virtual ~NodeVar();

    void SetRecField(char*,char*);
    char* GetField();
    char* GetRecord();

    virtual StateVar& operator = (StateVar&);

    virtual StateVar* Copy();
    virtual void Read(istream&);
    virtual void Write(ostream&);
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
	char* _field;
	char* _record;
};

inline char* NodeVar::GetField() { return _field; }
inline char* NodeVar::GetRecord() { return _record; }

// ---------------------state variable for links on wires--------------------
class PVLinkVar : public NameVar
{
public:
    PVLinkVar(Component* c=nil,DBENTRY* d=nil,
		const char* p="PP",const char* m="MS");
	virtual ~PVLinkVar();

    void SetRecField(DBENTRY*);
    DBENTRY* GetRecord();
	const char* GetLinkString(); // return the string in form pp/ms
	const char* GetLinkPP();
	const char* GetLinkMS();
    void SetLink(const char*,const char*); // pp/ms set both
	void SetComponent(Component*);
	Component* GetComponent();
    virtual StateVar& operator = (StateVar&);
    virtual StateVar* Copy();
    virtual void Read(istream&);
    virtual void Write(ostream&);
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
	Component* _comp;
	DBENTRY _record;
	int valid_record;
	char* _pp;
	char* _ms;
	char _ppms[8];
};

inline Component* PVLinkVar::GetComponent() { return _comp; }
inline void PVLinkVar::SetComponent(Component* c) { _comp=c; }
inline DBENTRY* PVLinkVar::GetRecord() { return &_record; }
inline const char* PVLinkVar::GetLinkPP() { return _pp; }
inline const char* PVLinkVar::GetLinkMS() { return _ms; }

#endif
