/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>

extern "C" {
#include "dbStaticLib.h"
};

char* type_names[] = {
	"String",
	"Integer",
	"Real",
	"Menu",
	"Menuform",
	"Inlink",
	"Outlink",
	"Fwdlink",
	"Noaccess"
};

char* link_names[] = {
	"Constant",
	"Form",
	"PV"
};

#define RC_NORMAL	0
#define RC_DONE		1
#define RC_QUIT		2
#define RC_ERROR	3

typedef long (*FUNC)(DBENTRY* dbe);
typedef char* (*NFUNC)(DBENTRY* dbe);

char* itemsNone[] = { NULL };

char* itemsMain[] = {
	"o=open",
	"q=quit",
	"s=save",
	"a=save as", 
	"p=list all records", 
	"e=edit records",
	NULL
};

char* itemsRecord[] = {
	"c=create",
	"d=delete",
	"e=edit",
	NULL
};

char* itemsField[] = {
	"e=edit",
	// "s=show",
	NULL
};

class Uninode
{
public:
	Uninode()			{ string=NULL; next=NULL; }
	Uninode(char* name)	{ string=name; next=NULL; }
	Uninode(char* name,Uninode* x)	{ string=strdup(name); next=x; }
	~Uninode()			{ if(string) delete [] string; }

	char* string;
	Uninode* next;
};

DBBASE* db=NULL;
char* inc_path=NULL;
char* inc_path_copy=NULL;
char* env_path=NULL;
char* log_name=NULL;
char* db_name=NULL;
Uninode* dbd_list=NULL;
Uninode* inc_list=NULL;
Uninode* inc_list_tail=NULL;
int tot_table=0;
int verbose_save=0;
int seq=0;

void SetIncPath(char* path);
void ClearList(Uninode*& list);
char* ResolveName(char* fname);
int SetDefaultDBDs(char** list,int total);
int AddDBD(char* name);
int ReadDefaultDBDs(DBBASE*& db);

class menu
{
public:
	menu(void) { dbInitEntry(db,&dbe); parent=NULL; items=itemsNone; }
	menu(menu* p) { Set(p->Get()); parent=p; items=itemsNone; }

	void Set(DBENTRY& ndbe) { dbCopyEntryContents(&ndbe,&dbe); }
	void Set(DBENTRY* ndbe) { dbCopyEntryContents(ndbe,&dbe); }
	DBENTRY* Get(void) { return &dbe; }

	int Run(void);
	virtual int Command(char item);
	virtual void List(void);
	char** Items() { return items; }
	void PrintList(FUNC first, FUNC next, NFUNC fname);
	int GetString(char* prompt, char* dest);

	menu* parent;
protected:
	DBENTRY dbe;
	char** items;
};

class menuMain : public menu
{
public:
	menuMain(void) { items=itemsMain; }
	menuMain(menu* p):menu(p) { items=itemsMain; }
	virtual int Command(char item);
	virtual void List(void);
};

class menuRecords : public menu
{
public:
	menuRecords(menu* p):menu(p) { items=itemsRecord; }
	virtual int Command(char item);
	virtual void List(void);
};

class menuFields : public menu
{
public:
	menuFields(menu* p):menu(p) { items=itemsField; }
	virtual int Command(char item);
	virtual void List(void);
};

// ---------------------------menu methods------------------------------

int menu::GetString(char* prompt, char* dest)
{
	int rc;

	printf("%s: ",prompt);
	gets(dest);

	if(dest[0])
		rc=0;
	else
		rc=1;

	return rc;
}

int menu::Run(void)
{
	int i,rc=RC_NORMAL;
	char** it = Items();
	char choice[100];

	while(rc==RC_NORMAL)
	{
		for(i=0;it[i];i++) printf(" %s\n",it[i]);
		printf(" l=list\n");
		printf(" <CR>=go back\n");
		printf("Enter above action item: ");
		gets(choice);
		if(strlen(choice)==0)
			rc=RC_DONE;
		else
		{
			if(choice[0]=='l')
				List();
			else
			{
				if(Command(choice[0])==RC_QUIT)
					rc=RC_QUIT;
			}
		}
	}
	return rc;
}

void menu::List(void) { printf("Nothing to list\n"); }
int menu::Command(char item)
{
	printf("Nothing to do with %c\n",item);
	return RC_DONE;
}

void menu::PrintList(FUNC first, FUNC next, NFUNC fname)
{
	long rc;
	int tot=0,cur;
	char buf[81];

	buf[0]='\0';
	for(rc=first(Get());rc==0;rc=next(Get()))
	{
		cur=strlen(fname(Get()));
		if((cur+tot)>sizeof(buf)-5)
		{
			buf[tot]='\0'; // buf[tot+1]='\0';
			puts(buf);
			tot=0;
			buf[0]='\0';
		}
		strcpy(&buf[tot],fname(Get()));
		tot+=cur;
		buf[tot++]=' ';
	}
	if(tot>0)
	{
		buf[tot]='\0'; // buf[tot+1]='\0';
		puts(buf);
	}
}

// ---------------------------menuMain methods------------------------------

int menuMain::Command(char item)
{
	int rc=0,v;
	char fname[250];
	menu* m;
	char* rname;

	switch(item)
	{
	case 'o': // open
		if(GetString("Enter db file to open",fname)==0)
		{
			if(rname=ResolveName(fname))
			{
				if(dbReadDatabase(&db,rname,NULL,NULL))
				{
					printf("Could not open db file <%s>\n",rname);
					delete [] rname;
				}
				else if(strstr(rname,".dbd")==NULL)
				{
					if(db_name) delete [] db_name;
					db_name=rname;
				}
				else
					delete [] rname;
			}
			else
				printf("Could not locate db file <%s>\n",fname);
		}
		break;
	case 's': // save
		if(db_name==NULL)
		{
			if(GetString("Enter db file name",fname)==0)
				db_name=strdup(fname);
		}
		if(db_name)
		{
			if(dbWriteRecord(db,db_name,NULL,verbose_save))
				printf("Cannot save database to file %s\n",db_name);
			else
				printf("Database saved to file %s\n",db_name);
		}
		else
			printf("Cannot save database\n");
		break;
	case 'a': // save as
		if(GetString("Enter db file name",fname)==0)
		{
			if(db_name) delete [] db_name;
			db_name=strdup(fname);
			if(dbWriteRecord(db,db_name,NULL,verbose_save))
				printf("Cannot save database to file %s\n",db_name);
			else
				printf("Database saved to file %s\n",db_name);
		}
		break;
	case 'e': // edit record of a type
		if(GetString("Enter record type name",fname)==0)
		{
			if(dbFindRecordType(Get(),fname))
				printf("Record type %s does not exist\n",fname);
			else
			{
				m=new menuRecords(this);
				rc=m->Run();
				delete m;
			}
		}
		break;
	case 'p': // show all record in the database
		for(v=dbFirstRecordType(Get());v==0;v=dbNextRecordType(Get()))
		{
			if(dbGetNRecords(Get()))
			{
				printf("---------(%s)----------\n",dbGetRecordTypeName(Get()));
				PrintList(dbFirstRecord,dbNextRecord,dbGetRecordName);
			}
		}
		break;
	case 'q': // quit
		rc=RC_QUIT;
		break;
	default:
		printf("Unknown option entered\n");
		break;
	}
	return rc;
}

void menuMain::List(void)
{
	PrintList(dbFirstRecordType,dbNextRecordType,dbGetRecordTypeName);
}

// --------------------------menuRecords methods-----------------------------

int menuRecords::Command(char item)
{
	int rc=RC_NORMAL;
	char fname[50];
	char nname[150];
	menu* m;

	switch(item)
	{
	case 'c':
		printf("Enter a \".\" for the record name to stop creating records\n");
		do
		{
			sprintf(nname,"Enter new name <%s%d default>",log_name,seq);
			fname[0]='\0';
			if(GetString(nname,fname)==0)
			{
				if(fname[0]=='.')
					rc=RC_DONE;
			}
			else
				sprintf(fname,"%s%d",log_name,seq++);

			if(rc!=RC_DONE)
			{
				if(dbCreateRecord(Get(),fname))
					printf("Cannot create record %s\n",fname);
				else
					printf("Created record %s\n",fname);
			}
		}
		while(rc==RC_NORMAL);
		break;
	case 'd':
		if(GetString("Enter record name to delete",fname)==0)
		{
			if(dbFindRecord(Get(),fname))
				printf("\nCannot find record %s\n",fname);
			else if(dbDeleteRecord(Get()))
				printf("\nCannot delete record %s\n",fname);
			else
				printf("\nDeleted record %s\n",fname);
		}
		break;
	case 'e':
		if(GetString("Enter record name",fname)==0)
		{
			if(dbFindRecord(Get(),fname))
				printf("Record %s does not exist\n",fname);
			else
			{
				m=new menuFields(this);
				rc=m->Run();
				delete m;
			}
		}
		break;
	default:
		printf("Invalid choice entered\n");
		break;
	}
	return rc;
}

void menuRecords::List(void)
{
	PrintList(dbFirstRecord,dbNextRecord,dbGetRecordName);
}

// ---------------------------menuFields methods------------------------------

int menuFields::Command(char item)
{
	long rc=RC_NORMAL;
	char fname[200];
	char val[200];
	char** c;
	int t,tot,i;

	do
	{
		if(GetString("Enter field name (<ENTER> to stop)",fname)==0)
		{
			if(dbFindField(Get(),fname))
				printf("\nCannot find field %s\n",fname);
			else
			{
				printf("Field Type: %s\n",type_names[dbGetFieldType(Get())]);
				printf("Current value: %s\n",dbGetString(Get()));
				switch(t=dbGetFieldType(Get()))
				{
				case DCT_MENU:
				case DCT_MENUFORM:
					c=dbGetMenuChoices(Get());
					tot=dbGetNMenuChoices(Get());
					for(i=0;i<tot;i++)
					{
						printf("(%s)",c[i]);
						if(i%5==4) putchar('\n');
					}
					putchar('\n');
					if(t==DCT_MENUFORM)
						printf("Related field: %s\n",dbGetRelatedField(Get()));
					break;
				case DCT_INLINK:
				case DCT_OUTLINK:
				case DCT_FWDLINK:
					printf("LinkType: %s\n",link_names[dbGetLinkType(Get())]);
					break;
				default:
					break;
				}
				if(GetString(dbGetPrompt(Get()),val)==0)
				{
					if(dbPutString(Get(),val))
						printf("Put to field failed\n");
				}
			}
		}
		else
			rc=RC_DONE;
	}
	while(rc==RC_NORMAL);

	return rc;
}

static long myFirstField(DBENTRY* d) { return dbFirstField(d,1); }
static long myNextField(DBENTRY* d) { return dbNextField(d,1); }

void menuFields::List(void)
{
	PrintList(myFirstField,myNextField,dbGetFieldName);
}

// ---------------------------------main------------------------------------

menu* current=NULL;

int main(int argc, char* argv[])
{
	int j,state,i,s,opt_num=1;
	char *rname,*pc;

	if(argc>opt_num && strcmp(argv[opt_num],"-v")==0)
	{
		++opt_num;
		verbose_save=1;
	}

	if((log_name=(char*)getenv("LOGNAME"))==NULL)
		log_name="xxx";

	if((inc_path=(char*)getenv("EPICS_DB_INCLUDE_PATH"))==NULL)
	{
		inc_path=strdup(".  ./dbd  ../dbd  ../../dbd ../src");
		env_path=new char[strlen(inc_path)+23];
		sprintf(env_path,"EPICS_DB_INCLUDE_PATH=%s",env_path);
		putenv(env_path);
	}

	SetIncPath(inc_path);
	fprintf(stderr,"Using dbd include path \"%s\"\n",inc_path);

	if(SetDefaultDBDs(&argv[opt_num],(argc>opt_num)?argc-opt_num:0))
	{
		// no dbds read, should search for .dbd files in current directory

		if((db=dbAllocBase())==NULL)
		{
			fprintf(stderr,"Cannot initialize an EPICS database\n");
			exit(1);
		}
	}

	current=new menuMain;
	current->Run();
	delete current;

	return 0;
}

void SetIncPath(char* path)
{
	Uninode* x;
	char* pc;

	// clear old path
	ClearList(inc_list);
	inc_list_tail=NULL;
	tot_table=0;
	if(inc_path_copy) delete [] inc_path_copy;
	inc_path_copy=strdup(path);

	for(pc=strtok(inc_path_copy," \t");pc;pc=strtok(NULL," \t"))
	{
		++tot_table;
		x=inc_list_tail;
		inc_list_tail=new Uninode(pc);
		if(x) x->next=inc_list_tail;
		else  inc_list=inc_list_tail;
	}
}

void ClearList(Uninode*& list)
{
	Uninode *x,*n;

	for(x=list;x;)
	{
		n=x->next;
		delete [] x;
		x=n;
	}
	list=NULL;
}

char* ResolveName(char* fname)
{
	int i;
	int len;
	char* rc;
	Uninode* node;

	if(fname[0]=='/')
	{
		if(access(fname,R_OK)==0)
			rc=strdup(fname);
		else
			rc=NULL;
	}
	else if(inc_list)
	{
		len=strlen(inc_path)+strlen(fname)+1;
		rc=new char[len+1];

		for(node=inc_list;node;node=node->next)
		{
			sprintf(rc,"%s/%s",node->string,fname);
			if(access(rc,R_OK)==0) break;
		}
		if(node==NULL)
		{
			delete [] rc;
			rc=NULL;
		}
	}
	else
		rc=NULL;

	return rc;
}

int SetDefaultDBDs(char** list,int total)
{
	int i,rc=1;

	ClearList(dbd_list);

	for(i=0;i<total;i++)
	{
		if(strstr(list[i],".dbd"))
			if(AddDBD(list[i])==0) rc=0;
	}

	if(rc)
	{
		// see if a dbd file exists in the current directory
		DIR* dirp=opendir(".");
		dirent* dp;
		while((dp=readdir(dirp)))
		{
			if(strstr(dp->d_name,".dbd"))
				if(AddDBD(dp->d_name)==0) rc=0;
		}
		closedir(dirp);
	}
	return rc;
}

int AddDBD(char* name)
{
	char* rname;
	int rc=0;
	Uninode* x=dbd_list;

	dbd_list=new Uninode(name,x);

	if((rname=ResolveName(dbd_list->string)))
	{
		if(dbReadDatabase(&db,rname,NULL,NULL))
		{
			fprintf(stderr,"Read failed: dbd file %s\n",rname);
			rc=1;
		}
		else
			fprintf(stderr,"Read dbd file: %s\n",rname);

		delete [] rname;
	}
	else
		rc=1;

	return rc;
}

int ReadDefaultDBDs(DBBASE*& base)
{
	Uninode* x;
	char* rname;
	int rc=1;

	if(dbd_list)
	{
		for(x=dbd_list;x;x=x->next)
		{
			if((rname=ResolveName(x->string)))
			{
				if(dbReadDatabase(&base,rname,NULL,NULL))
					fprintf(stderr,"Read failed: dbd file %s\n",rname);
				else
				{
					rc=0;
					fprintf(stderr,"Read dbd file: %s\n",rname);
				}
				delete [] rname;
			}
		}
	}
	return rc;
}

